 
c****************************************************************************

	subroutine cesam(etat,opa,conv,nuc,lim_ext,tdetau,
	1	coeff_diff,perte_m,ctes,des_cesam)

c	calcul de modele de structure interne et d'evolution stellaire
c	CESAM4

c	methode collocation de de Boor p. 277
c	adapte a la structure interne

c	avec fonction de repartition

c	Auteur: P. Morel, Departement J.D. Cassini, O.C.A., Observatoire de Nice

c	07 10 96 : gravite effective dans vaissala
c	10 10 96 : introduction de la rotation dans thermo
c	21 03 97 : bug dans calcul de z_opa (signale par Sacha)
c	08 05 97 : introduction de l'option kipp pour calcul du TdS
c	26 05 97 : fichiers cohe, coca, debut de combustion helium, carbone
c	11 06 97 : simplification de la gestion de t_inf
c	26 06 97 : remplacement du moment angulaire par la vitesse angulaire
c	18 07 97 : correction d'energie exedentaire de rotation
c	18 08 97 : modif nom elem chim
c	25 08 97 : mise en place des variables euleriennes
c	22 09 97 : abondances externes dans le fichier *.HR
c	04 11 97 : abondances centrales dans le fichier *.HR
c	18 11 97 : mstar dans le fichier *.HR
c	12 11 99 : calcul de Mue avec les taux d'ionisation
c	19 11 99 : rectification de call time, des dates, de thermo,
c		   utilisation de saha pour nh1, nhe1, nhe2, lamb
c	18 01 00 : correction si e_tot=0
c	02 02 00 : e_grav < 50% e_nuc ----> serie pple (au lieu de 99%)

c	avec pression turbulente 7 inconnues 
c	sans pression turbulente 6 inconnues, Ptot=Pgaz

c entrees:
c	noms des routines de physique

c---------------------------------------------------------------------

c routines externes de physique
c	etat: equation d'etat
c	opa: opacite
c	conv: convection
c	nuc: reactions thermonucleaires
c	lim_ext: condition limite externe
c	tdetau: loi t(tau)
c	ctes: definition des constantes
c	coeff_diff: coefficients de diffusion
c	perte_m : perte de masse

c fonctions des principales routines externes numeriques
c	cesam : gestion generale des calculs, des ecritures
c	des_cesam: dessin du modele durant l'execution

c fonctions des principales routines internes numeriques
c	resout : initialisation, formation, resolution du probleme aux limites
c	evol : gestion de l'evolution de la composition chimique
c	static_m : formation des equations de l'equilibre quasi statique
c	lim_zc : determination des limites ZR/ZC
c	update : transfert t+dt ---> t ---> t-dt
c	diffus, eq_diffus : calcul de la diffusion
c	coll_atm, eq_atm : restitution d'atmosphere

c------------------------------------------------------------------

c les parametres de dimensions sont definis dans le fichier cesam.parametres
c	ils sont introduits par INCLUDE cesam.parametres

c------------------------------------------------------------------------

c	Les COMMONs ne contiennent que des quantites constantes
c	au cours de l'evolution (elles peuvent changer lors de l'initialisation)
c	les parmetres qu'ils contiennent sont decrits dans les fichiers
c	"INCLUDE" ainsi que l'endroit des initialisations

c COMMON /atmosphere/ elements pour le calcul de l'atmosphere
c		fichier INCLUDE atmosphere.common

c COMMON /convection/ parametres pour la convection, 
c		fichier INCLUDE convection.common 

c COMMON /ctephy/ constantes physiques utilisees, fichier INCLUDE ctephy.common

c COMMON /evol_chim/ parametres pour l'evolution de la composition chimique
c		fichier INCLUDE evol_chim.common 

c COMMON /modele/ caracteristiques du modele, fichier INCLUDE modele.common

c------------------------------------------------------------------

c	fichier cesam.parametres

c dimensions maximales (en parametre) :
c	pn : nombre maximal de points de raccord
c	pne : nombre maximal d'equations d'ordre 1
c	pm_qs : ordre maximal des splines pour la collocation
c	pm_ch : ordre maximal des splines pour l'interpolation de la comp.chim.
c	pnzc : nombre maximal de zones convectives
c	pnelem : nombre maximal d'elements diffuses
c	pn_atm : nombre maximal de couches dans l'atmosphere
c	pnreac : nombre maximal de reactions nucleaires

c	le centre est a la couche 1, la limite externe en couche n
c	le nombre de couches est VARIABLE il est determine dans le SSP repartit
c	une couche est place a chaque limite ZR/ZC (a 5% pres)

c	increment temporel :

c		dt=delta t / 10**6 ans

c	variable independante : q l'indice (Reel) de couche 

c	variables dependantes :

c		ksi=ln P pression cgs
c		eta=ln T temperature K
c		dzeta=(r/rsol)**2 
c		lambda=(l/lsol)**2/3
c		mu=(m/mtot)**2/3
c		psi=dQ/dq=cte avec Q(mu,t)=fonction d'espacement
c		u=Ptot/(Pgas+Prad)

c		xchim(i),i=1,nbelem : abondances / mole en f(m/Mtot)
c		 + W + Z  (w : vitesse angulaire)

c	ordre et indices d'identification des elements definis dans
c	reac_nuc et resout, Exemple:
c	H1 H2 He3 He4 Li7 Be7 C12 C13 N14 N15 O16 O17 W Z
c	1  2  3   4   5   6   7   8   9   10   11  12 13 14
	
c	avant le debut d'un pas temporel:

c	mc_t,mct_t,nc_t,knotc_t,chim_t:   comp. chim. a age-dt
c	mc,mct,nc,knotc,chim: comp. chim. a age
c	bp,q,qt,knot,p,t,r,l,m:  var. pples. a age
c	m23,m23t_t,m_ch,n,knot23_t,tdst: TdS a age t

c_______________________________________________________________________

c	Fichiers utilises 

c	les fichiers de sortie ont le nom choisi par l'utilisateur nom_du_modele
c	suivit d'un attribut qui les distingue
c	l'attribut "_B" est specifique aux fichiers binaires

c	FOR002	listing de sortie, cree dans cesam ferme et garde dans
c		cesam, nom : nom_du_modele.lis

c	FOR003	lecture des donnees, utilise temporairement dans resout
c		nom : nom_du_modele.don

c	FOR004  modele initial ou repris (binaire),
c		utilise temporairement dans cesam

c	FOR011 lecture des tables d'opacite 
c		utilise temporairement dans routines d'opacite

c	FOR011-->017 lecture des tables d'equation d'etat
c		utilise temporairement dans certains SSP d'equation d'etat

c	FOR020	tables de reactions nucleaires, ouvert et ferme dans reac_tab

c	FOR024  ecriture modele initial d'age 0 homogene, ZAMS ou PMS,
c		ouvert et ferme dans cesam
c		nom : nom_du_modele_B.hom en binaire

c	FOR025  dernier modele, ouvert et ferme dans cesam
c		nom : nom_du_modele_B.dat en binaire

c	FOR026  point de reprise, ouvert et ferme dans cesam
c		nom : nom_du_modele_B.rep en binaire

c	FOR030  liste pour oscillation ouvert et ferme dans cesam
c		nom : nom_du_modele.osc en ASCII

c	FOR031	dernier modele calcule de l'atmosphere
c		ouvert et ferme dans lim_teff
c		nom : nom_du_modele_B.atm en binaire

c	FOR048  liste pour dessins ouvert et ferme dans cesam
c		nom : nom_du_modele.des en ASCII
c	des interventions sont necessaires dans cesam pour sa creation:
c		enlever les "c" de commentaires

c	FOR053	fichier pour tracer un diag. HR et les ZC,
c		ouvert et ferme dans cesam
c	        nom : nom_du_modele.HR en ASCII

c	FOR060	tables d'EOS de LAOL

c--------------------------------------------------------------------

c	variables euleriennes ou lagrangiennes

c	en_masse = .true. variables lagrangiennes m23=m**23, r2=r**2
c	en_masse = .false. variables euleriennes m23=m, r2=r
	
c-----------------------------------------------------------------------

c		P R O G R A M M E   P R I N C I P A L

c	declaration de dimensions, gestion du calcul

	implicit none

c parametres: des maxima pour :
c	nb. de points de liaison, ordre splines pour coll., ordre des equations,
c	nb. d'elements chimiques, nb. d'equations, nb. de couches dans atmosph.
c	nb. zones convectives, ordre des splines d'interp. de la comp. chim.
c	nb. reactions thermonucleaires

	include 'atmosphere.common'
	include 'cesam.parametres'
	include 'cesam.version'
	include 'convection.common'	
	include 'ctephy.common'
	include 'evol_chim.common'		
	include 'modele.common'
	
	integer n,i,k,j,un23,n_t,knot,i_pp,i_cno,i_3a,i_gr,ipms,knota,
	1	nc_t,knotc_t,nc,knotc,lim_t,jlim_t(2*pnzc),long,values(8),
	2	lx,nadd,knot_t,jlim(2*pnzc),lim,reprise,ipn,knot23_t
	data ipms/3/

	real*8	xchim(pnelem),xchim1(pnelem),dxchim(pnelem),dcompg(pnelem*pn),
	1	pt(pn),p(pn),t(pn),r(pn),l(pn),m(pn),w(pn),z(pn),grad(pn),ro(pn),
	2	compchim(pnelem*pn),eint(pn),gamma1(pn),
	3	epsilon(5*pn),bidt(pn),compg(pnelem*pn),lteff,teff,agep,y,
	4	nh1(pn),nhe1(pn),nhe2(pn),beta(pn),	
	4	alfa(pn),dcapdr(pn),dcapdt(pn),
	5	depsdr(pn),depsdt(pn),anupp(pn),anupep(pn),anub8(pn),
	6	anube7(pn),anun13(pn),anuo15(pn),dcompchim(pnelem*pn),
	7	vaissala(pn),xdot(pn),ydot(pn),rdot(pn),lamb(pn)
	
	real*8	dlpp(pn),grad_mj(pn),gradrad(pn),gradconv(pn),
	1	dgradpt,dgradp,dgradt,dgradl,dgradr,dgradm,dgradx,dgradlpp,
	1	gradad(pn),dgradadp,dgradadt,dgradadx,mue(pn),mu(pn),
	2	hp(pn),dhppt,dhpp,dhpt,dhpx,dhpr,dhpm,
	3	mue_atm(pn_atm),mu_atm(pn_atm)

	real*8	drop,drot,dup,dut,d2p,d2ro,epsilo(5),depsp,depst,
	1	kap(pn),dkapp,dkapt,drox,dux,
	2	depsx(pnelem),dkapx,aradias3,dcomp(pnelem),pertem,
	3	delta(pn),deltap,deltat,deltax,cp(pn),dcpp,dcpt,dcpx,
	4	ro_t,u_t,bid,gamma1_atm(pn_atm),beta_atm(pn_atm),	
	5	dm,e_tot,e_pp,e_cno,e_3a,e_gr,p_t,t_t,nuc_m(pnelem),
	6	f(pnelem),dfdx(pnelem),tdst(pn),m23t_t(pqt),f_tau,df_tau,
	7	fonc(0:5),poly(0:5),absc(0:5),d2f_tau,dgradrs,dgradtau,
	8	gam,dgampt,dgamp,dgamt,dgamx,dgamm,dgaml,dgamr,dgamlpp,
	9	dgamrs,dgamtau,ioni(0:pzi,pnchim),nel,zbar(pnchim)
		
	real*8	pt_atm(pn_atm),t_atm(pn_atm),r_atm(pn_atm),m_atm(pn_atm),
	1	tau(pn_atm),ro_atm(pn_atm),k_atm(pn_atm),gradr_atm(pn_atm),
	2	grada_atm(pn_atm),grad_atm(pn_atm),chim1g(pnelem),dro_teff,
	3	nh1_atm(pn_atm),nhe1_atm(pn_atm),nhe2_atm(pn_atm),dro_grav,
	4	alfa_atm(pn_atm),delta_atm(pn_atm),cp_atm(pn_atm),ro_ext,
	5	vais_atm(pn_atm),gradc_atm(pn_atm),ldcapdr_a(pn_atm),dtsdg,
	6	ldcapdt_a(pn_atm),lnpt_a(pn_atm),lnt_a(pn_atm),grad_mj_a(pn_atm),
	7	rstar,ptext,text,dptdl,dptdr,dtdl,dtdr,mext,dml,dmr,dtsdteff,
	8	logg,hp_atm(pn_atm),rp1,rp2,lp1,lp2,tds,grav,dtsdtau,
	9	pext,dpl,dpr,p_atm(pn_atm),dlpp_atm(pn_atm)
	
	real*8	cte1,cte3,cte5,cte7,cte8,cte16

	real*8	bp(pbp),q(pn),qt(pqt),chim(pchim),mc(pnch),mct(pchimt),
	1	r2(pn),m23(pn),m_zc(2*pnzc),r_zc(2*pnzc),r_ov(2*pnzc),
	2	age,bp_t(pbp),q_t(pn),qt_t(pqt),chim_t(pchim),mc_t(pnch),
	3	mct_t(pchimt),r2_t(pn),m23_t(pn),dt,dtp,m_zc_t(2*pnzc),
	4	r_zc_t(2*pnzc),r_ov_t(2*pnzc),
	6	mstar,mstar_t,chimg(pnelem),dchimg(pnelem),dw(pn),
	7	iw2,iw2_t,diw2
	
	integer iconst,ivar,ivers	!pour ecrire fichier oscillations
	real*8 glob(15),var(22+pnelem,pn+pn_atm)

	character*1 oui
	character*2 c_qs,l2,d_qs
	character*3 cr,ch,l3
	character*4 cm,l4
	character*5 zone	
	character*8 date	
	character*9 mois(12)
	data mois/'Janvier  ','Fevrier  ','Mars     ','Avril    ',
	1	  'Mai      ','Juin     ','Juillet  ','Aout     ',
	2	  'Septembre','Octobre  ','Novembre ','Decembre '/
	character*10 time
	character*30 chaine
	character*50 nom_fich1,modelb
	character*80 titre,methode
	
	logical convec(pn),llist,statut,lconv(2*pnzc),lconv_t(2*pnzc),rad_atm,
	1	pms,pms_pr,zams,zams_pr,post_e,zams_e,passe,dessin,post,der,
	2	post_pr,cohe,cohe_e,cohe_pr,sort,ecrit,coca,coca_e,coca_pr
	data passe/.false./

	external ctes,etat,opa,conv,nuc,lim_ext,tdetau,perte_m,
	1	coeff_diff,iben,des_cesam
	
	save

c-----------------------------------------------------------------------

2000	format((1x,1p8d10.3))

	beta_g=1.	!pour GONG1 beta_g et lambda_g seront differents
	lambda_g=1.

c	calcul a effectuer, point de reprise

	write(6,*)' '
	write(6,*)'CESAM, version: ',version
	write(6,*)' '
199	print*,'Si on poursuit une evolution : taper 1 puis RETURN'
	write(6,*)' '
	print*,'Si on initialise un modele de ZAMS : taper 2 puis RETURN'
	write(6,*)' '
	print*,'Si on initialise un modele de PMS : taper 3 puis RETURN'
	
c	on entre un23 = 1, 2 ou 3 comme indiquer ci dessus
c	puis, suivant les cas on donne a un23 les valeurs suivantes:
	
c	un23 :  1 poursuite d'une evolution, modele repris en binaire
c		2 modele initial en binaire, ZAMS
c		3 modele initial en binaire, PMS
c	       -2 modele initial en ASCII, ZAMS
c	       -3 modele initial en ASCII, PMS

	read(5,*)un23
	
	if(un23 .eq. 1)then
	 write(6,6)
6	 format(1x,'entrer le nom du fichier binaire du modele dont on ',
	1	'poursuit l''evolution')
	 read(5,'(a)')nom_fich1
	 write(6,*)'on utilise le modele ',nom_fich1
	 write(6,*)' '
	 open(unit=4,form='unformatted',status='old',file=nom_fich1)
	 read(4)age,nbelem,nchim,mtot,alpha,model,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw,
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close (unit=4)
c	 print*,'apres lecture dt ',dt
	 
	 write(6,*)'On poursuit l''evolution du modele: ',model(:long(model))
	 write(6,201)age
201	 format(1x,' d''age:',1pd10.3)
	 write(6,*)
	 pms=.false.
	 zams=.false.
	 post=.false.
	 cohe=.false.	!debut de la combustion de He
	 coca=.false.	!debut de la combustion de C	
	
	elseif(un23 .eq. 2)then
	 pms=.false.
	 zams=.true.
	 post=.false.
	 cohe=.false.
	 coca=.false.
	 print*,'le modele initial de ZAMS est-il donne en binaire ? o/n'
	 read(5,'(a)')oui
	 if(oui .ne. 'n')then
	  write(6,7)
7	  format(1x,'entrer le nom du fichier binaire du modele pris pour '
	1	'modele initial')	  
	  read(5,'(a)')nom_fich1
	  write(6,*)'on utilise le modele ',nom_fich1
	  write(6,*)' '
	
	  open(unit=4,form='unformatted',status='old',file=nom_fich1)
	  read(4)age,nbelem,nchim,mtot,alpha,model,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	  close (unit=4)

	  write(6,*)'On prend pour modele initial: ',model(:long(model))
	  write(6,*)' '	
	  un23=2	!fichier en binaire (intruction redondante) 
	 else
	  write(6,9)
9	  format(1x,'entrer le nom du fichier ASCII du modele pris pour '
	1	'modele initial')
	  read(5,'(a)')nom_fich1	!on lira ce fichier dans resout
	  open(unit=4,form='formatted',status='old',file=nom_fich1)
	  write(6,*)'CESAM utilise le modele ',nom_fich1
	  write(6,*)' '
	  un23=3		!fichier en ASCII
	 endif
	 age=0.d0
	 dt=0.
	  
	elseif(un23 .eq. 3)then
	 pms=.true.
	 zams=.false.
	 post=.false.
	 cohe=.false.
	 coca=.false.
	 print*,'le modele initial de PMS est-il donne en binaire ? o/n'
	 read(5,'(a)')oui
	 if(oui .eq. 'o')then
	  write(6,*)'CESAM utilise un modele initial de PMS en binaire'	  
	  write(6,7)
	  read(5,'(a)')nom_fich1
	  write(6,*)'CESAM utilise le modele ',nom_fich1
	  write(6,*)' '
	  open(unit=4,form='unformatted',status='old',file=nom_fich1)
	  read(4)age,nbelem,nchim,mtot,alpha,model,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	  close (unit=4)
	  un23=-2 !fichier en binaire et, pour la PMS, pas de modele homogene
	 else
	  write(6,*)'CESAM utilise un modele initial de PMS en ASCII'
	  write(6,*)' '
	  write(6,9)
	  read(5,'(a)')nom_fich1
	  open(unit=4,form='formatted',status='old',file=nom_fich1)
	  write(6,*)'CESAM utilise le modele ',nom_fich1
	  write(6,*)' '
	  un23=-3  !fichier en ASCII et, pour la PMS, pas de modele homogene
	 endif
	 age=0.d0
	 dt=0.	 
	 write(6,*)'calcul de l''evolution pre-sequence principale:'
	 write(6,*)'entrer la constante de contraction C (2.d-2)'
	 read(5,*)c_iben
	 ipms=1
	 
c	 ipms =	1, 2, 3 gere le calcul PMS,
c		1 et 2 : premier et second modele d'initialisation
c		3 : le modele ignore qu'il est issu de la pms 
	 
	else
	 print*,'ERREUR, entrer 1, 2 ou 3, vous avez tape',un23
	 goto199 
	endif

c	identificateurs des fichiers des donnees, du point de reprise,
c       des resultats,
c	du modele d'age 0, des oscillations

	write(6,*)'entrer l''identificateur du modele'
	write(6,*)'exemple: soleil_evolue'
	read(5,'(a)')nom_fich2
	write(6,*)'identificateur des fichiers du modele : ',
	1	nom_fich2(:long(nom_fich2))
	write(6,*)' '

c	fichier pour le listing

	open(unit=2,form='formatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'.lis')

c	fichier pour le diagramme HR

	open(unit=53,form='formatted',status='unknown',access='append',
	1	file=nom_fich2(:long(nom_fich2))//'.HR')
	agep=-1.d30	!initialisation pour lister le premier modele

c	write(6,*)'avant resout pms,ipms,un23,dt',pms,ipms,un23,dt

	if(pms)then
	
	 call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,nuc,lim_ext,tdetau,coeff_diff,perte_m)
	
c	 initialisation de Iben, t, ro fictifs

	 call iben(t(1),ro(1),xchim,dcomp,bidt,.false.,1,
	1	epsilo,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt)
c	 pause'entre les deux resout'

	 call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,iben,lim_ext,tdetau,coeff_diff,perte_m)

	else
	 call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,nuc,lim_ext,tdetau,coeff_diff,perte_m)
	endif
c	print*,'apres les 3 resout dt ',dt
	  
c	quelques constantes

	aradias3=aradia/3.
		
	cte1=g*msol/rsol**2
	cte3=4.d0*pi*rsol**3/msol
	cte5=-4.d0*pi*g/3.d0*rsol**2
	cte7=rsol/secon6
	cte8=lsol/pi/rsol**2/aradia/clight
	cte16=rsol**2/3.d0/secon6
	
	der=cpturb .lt.0.d0	!on tient compte de dln Pgaz/dln Ptot

	encode(4,3,cm)int(mtot*100.)
	encode(3,1,ch)int(x0*1000.)
	encode(3,1,cr)int(alpha*100.)
1	format(i3)
2	format(i2)
3	format(i4)

c	identification de la methode

	encode(2,2,c_qs)m_qs
	encode(2,2,d_qs)m_ch

	methode='CESAM version '//version

	if(en_masse)then
	 methode=methode(:long(methode))//' lagr colloc'//c_qs//d_qs
	else
	 methode=methode(:long(methode))//' eulr colloc'//c_qs//d_qs
	endif	
	methode=methode(:long(methode))//' '//precision
	if(diffusion)then
	 methode=methode(:long(methode))//' diffus'
	else
	 methode=methode(:long(methode))//' no diffus'
	endif
	
c	fait-on le dessin sur ecran?	

	write(6,*)'dessin du modele en cours de l''evolution ? o/n'
	read(5,'(a)')oui
	dessin=oui .eq. 'o'
	
c	on garde le modele d'age 0 sur le fichier xxxx_B.hom ou xxxx_B.pms

	if(abs(un23) .ge. 2)then	!cas d'un modele homogene
	 dt=0.
	 if(pms)then
	  open(unit=24,form='unformatted',status='unknown', !1ier modele PMS
	1	file=nom_fich2(:long(nom_fich2))//'_B.pms')
	  modelb='m'//cm//'X'//ch//'a'//cr//'_B.pms'
	 else
	  open(unit=24,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.hom')
	  modelb='m'//cm//'X'//ch//'a'//cr//'_B.hom'
	 endif
	 write(24)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=24)
	endif

c	garde des points de reprise

c	write(6,*)'faut-il garder tous les modeles intermediaires ? o/n'
c	read(5,'(a)')oui
c	statut=oui .eq. 'o'
	statut=.false.		!elimine

c	ecriture des solutions a l'ecran

c	write(6,*)'faut-il ecrire les modeles sur l''ecran ? o/n'
c	read(5,'(a)')oui
c	llist=oui .eq. 'o'
	llist=.false.		!elimine
	
	dtp=dt		!pas temporel precedent
	
c	gestion du premier pas temporel 

	if(ipms .gt. 2)then	!sauf pour les modeles initiaux PMS
	 if(un23 .eq. 1)then	!si reprise d'une evolution
	  if(dt .gt. 0.d0)then
	   write(6,111)dt
111	   format(1x,'pas temporel precedent=',1pd10.3,
	1	' 10**6 ans, le garde-t-on ? o/n')
	   read(5,'(a)')oui
	   if(oui .ne. 'o')then
	    write(6,*)'entrer le nouveau pas temporel, unite: 10**6 ans'
	    read(5,*)dt
	    if(dt .gt. dtmax)then
	     write(6,112)dtmax
112	     format(1x,'dt > dtmax, le pas temporel est ramene a dtmax=',
	1	1pd10.3)
	    endif
	   endif	!dt>0
	  else		!un23 .ne. 1
	   write(6,113)dt0
113	   format(1x,'utilise-t-on dt0=',1pd10.3,' pour le pas temporel? o/n')
	   read(5,'(a)')oui
	   if(oui .eq. 'o')then
	    dt=dt0
	   else
	    write(6,*)'entrer le nouveau pas temporel, unite: 10**6 ans'
	    read(5,*)dt	    
	    if(dt .gt. dtmax)then
	     write(6,112)dtmax
	    endif
	   endif
	   if(age+dt .gt. agemax)then
	    write(6,53)age+dt,agemax
	    write(2,53)age+dt,agemax	    
53	    format(1x,'age+dt=',1pd10.3,' > agemax =',1pd10.3,
	1	' ajustement de dt')
	    dt=max(0.d0,agemax-age)
	   endif
	   write(2,50)dt
	   write(6,50)dt
50	   format(//,t2,'reprise de l''evolution avec le pas temporel',
	1	1pd10.3,' 10**6 ans',//)
	  endif	!dt>0
	  
	 else		!un23 > 1
	  if(agemax .gt. 0.d0)then
	   write(6,113)dt0
	   read(5,'(a)')oui
	   if(oui .eq. 'o')then
	    dt=dt0
	   else	  
	    write(6,*)'entrer le pas temporel unite: 10**6 ans'
	    read(5,*)dt
	   endif		!initialisation d'une evolution
	   if(dt .gt. agemax)then
	    write(6,54)dt,agemax
	    write(2,54)dt,agemax	    
54	    format(1x,'dt=',1pd10.3,' > agemax =',1pd10.3,
	1	' ajustement de dt')
	    dt=agemax
	   endif
	   write(2,51)dt
	   write(6,51)dt
51	   format(//,t2,'debut d''evolution avec le pas temporel',
	1	1pd10.3,' 10**6 ans',//)
	
	  else
	   print*,'calcul d''un modele d''age 0'
	   dt=0.
	  endif		!agemax > 0
	 endif		!un23 = 1
	endif		!ipms > 2

c	open(unit=48,form='formatted',status='unknown',
c	1	file=nom_fich2(:long(nom_fich2))//'.des')

c'''''''''''''''''''''' retour ''''''''''''''''''''''''''''''''''''''''''''''

c	fichier du point de reprise

	reprise=10		!avec 1, 2, ... , 9 ca ne marche pas ????

100	do i=1,n			!extraction p,t,r,l,m
	 call sbsp1dn(ne,bp,q,qt,n,mpr,knot,.true.,q(i),lx,f,dfdx)
	 pt(i)=exp(f(1))	!Ptot
	 if(pturb)then		!avec pression turbulente 7 inconnues
	  p(i)=exp(f(7))		!Pgas	 
	 else			!sans pression turbulente 6 inconnues
	  p(i)=pt(i)
	 endif	 
	 if(pturb .and. der)then
	  dlpp(i)=dfdx(7)/dfdx(1)	!dln Pgaz/dln Ptot		
	 else
	  dlpp(i)=1.d0
	 endif
	 t(i)=exp(f(2))	!variable ln T
	 if(en_masse)then
	  r(i)=sqrt(abs(f(3)))	!rayon/Rsol
	  l(i)=sqrt(abs(f(4)))**3	!l/Lsol
	  m(i)=sqrt(abs(f(5)))**3	!m/Msol
	 else	 
	  r(i)=abs(f(3))	!rayon/Rsol
	  l(i)=f(4)		!l/Lsol
	  m(i)=abs(f(5))	!m/Msol
	  f(5)=m(i)**(2./3.)
	 endif
	 grad_mj(i)=dfdx(2)/dfdx(1)		!le gradient reel pour Marie Jo
c	 write(6,2000)p(i),t(i),r(i),l(i),m(i),grad_mj(i)
c	 write(6,2000)r(i),r2(i),m(i),m23(i)
	 
c	 la composition chimique

	 call sbsp1dn(nbelem,chim,mc,mct,nc,m_ch,
	1	knotc,.true.,min(f(5),mc(nc)),lx,xchim,dxchim)
	
	 do j=1,nbelem
	  chimg(j)=xchim(j)
	  dchimg(j)=dxchim(j)
	 enddo
	 call chim_gram(chimg,dchimg,nuc_m)	  
	 do j=1,nbelem
	  compchim(nbelem*(i-1)+j)=xchim(j)
	  compg(nbelem*(i-1)+j)=chimg(j)	  
	  if(f(5) .gt. 0.d0)then
	   dcompchim(nbelem*(i-1)+j)=dxchim(j)*2.d0/3.d0/sqrt(f(5))	  
	   dcompg(nbelem*(i-1)+j)=dchimg(j)*2.d0/3.d0/sqrt(f(5))
	  else
	   dcompchim(nbelem*(i-1)+j)=0
	   dcompg(nbelem*(i-1)+j)=0	   
	  endif
	  if(i .eq. n)then
	   xchim1(j)=xchim(j)	!pour l'atmosphere
	   chim1g(j)=chimg(j)	!pour l'atmosphere
	  endif	   
	 enddo
	 
c	 la rotation
	
	 if(iw .gt. 1)then
	  w(i)=xchim(iw)
	  dw(i)=dxchim(iw)
	 else
	  w(i)=w_rot
	  dw(i)=0.
	 endif

	enddo

c	on ecrit la solution a l'ecran
c	la difference entre Ptot et Pgaz est ignoree

	if(llist)then
	 write(6,101)age
101	 format(1x,'age=',1pd10.3)
	 write(6,102)
102	 format(t9,'1-m',t24,'p',t36,'t',t49,'r',t60,'l',t71,'X')
	 do k=n,1,-1		
	  write(6,2001)1.d0-m(k)/mstar,p(k),t(k),r(k),l(k),compg(nbelem*(k-1)+1)
2001	  format((1x,1pd17.10,1p4d12.5,1pd10.3))
	 enddo
	endif

c	tabulation du TdS et interpolation en m23,
c	calcul des proportions en energies PP, CNO, 3alpha et gravifiques
c	on ne tient pas compte de la difference entre Ptot et Pgaz

c	print*,'tabulation TdS/dt',ipms
c	write(6,2000)dtp
c	pause'tabulation TdS'
	e_tot=0.
	e_pp=0.
	e_cno=0.
	e_3a=0.
	e_gr=0.
	do i=1,n
	 if(i .eq. 1)then	!dm sert a integrer les e_pp etc..
	  dm=m(2)-m(1)		!au facteur mstar pres
	 elseif(i .eq. n)then
	  dm=m(n)-m(n-1)
	 else
	  dm=m(i+1)-m(i-1)
	 endif
	 do j=1,nbelem
	  chimg(j)=compg(nbelem*(i-1)+j)
	  xchim(j)=compchim(nbelem*(i-1)+j)
	 enddo
	 call etat(p(i),t(i),chimg,.false.,	!en t+dt
	1	ro(i),drop,drot,drox,eint(i),dup,dut,dux,
	2	delta(i),deltap,deltat,deltax,cp(i),dcpp,dcpt,dcpx,
	3	gradad(i),dgradadp,dgradadt,dgradadx,	
	4	alfa(i),beta(i),gamma1(i),bidt)
	 if(ipms .le. 2)then	!pour l'initialisation PMS
	  call iben(t(i),ro(i),xchim,dcomp,bidt,.false.,3,
	1	epsilo,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt)
	  tdst(i)=-epsilo(1)*secon6	!dans iben cT sort dans epsilo(1)
	  epsilo(1)=0	!mis a 0 pour le bilan des sources d'energie
	  
	 else		!ipms>2
	  call nuc(t(i),ro(i),xchim,dcomp,bidt,.false.,3,
	1	epsilo,bidt,bidt,depsx,bidt,bidt,bidt,bidt,bidt,bidt)
	  if(dtp .gt. 0.)then	!1ier passage ipms=3, dt=0 --> ct dans TdS
	   call inter('m23',bp_t,q_t,qt_t,n_t,knot_t,m23(i),f,dfdx,r2_t,m23_t)	
	   if(pturb)then		!avec pression turbulente 7 inconnues
	    p_t=exp(f(7))	 
	   else			!sans pression turbulente 6 inconnues
	    p_t=exp(f(1))	 
	   endif
	   t_t=exp(f(2))
	   if(kipp)then	!approximation de Kippenhahan
	    tdst(i)=(cp(i)*(t(i)-t_t)-delta(i)/ro(i)*(p(i)-p_t))/dtp	   
	   else
	    bid=abs(m23(i))	!interpolation en m**2/3 pour xchim	   
	    if(.not.en_masse)bid=bid**(2./3.)
	    call sbsp1dn(nbelem,chim_t,mc_t,mct_t,nc_t,m_ch,
	1	knotc_t,.true.,min(bid,mc_t(nc_t)),lx,chimg,dchimg)
	    call chim_gram(chimg,dchimg,nuc_m)		!X, Y, Z
c	    write(6,2000)bid,p_t,t_t	   
	    call etat(p_t,t_t,chimg,.false.,
	1	ro_t,drop,drot,drox,u_t,dup,dut,dux,
	2	delta(i),deltap,deltat,deltax,cp(i),dcpp,dcpt,dcpx,
	3	gradad(i),dgradadp,dgradadt,dgradadx,	
	4	alfa(i),beta(i),gamma1(i),bidt)
	    tdst(i)=(eint(i)-u_t-p(i)/ro(i)**2*(ro(i)-ro_t))/dtp
	   endif	!kipp
c	   write(6,2000)tdst(i),eint(i),u_t,p(i),p_t,t_t,ro(i),ro_t

	   iw2=(w(i)*r(i))**2	
	   if(iw .gt. 1 .and. r(i) .ne. 0.)then		!rotation
	    if(en_masse)then
	     iw2_t=(f(3)*chimg(iw)/r(i))**2		!energie cinetique
	    else
	     iw2_t=(f(3)**2*chimg(iw)/r(i))**2		!energie cinetique	    
	    endif 
	   else
	    iw2_t=0.
	   endif
	   diw2=cte16*(iw2-iw2_t)/dtp

	  else		!dtp > 0
	   if(pms)then	!1ier passage apres modeles initiaux PMS (ipms=3)
	    tdst(i)=-c_iben*t(i)*secon6		!PMS
	    epsilo(1)=0.	!initialisation
	   else
	    tdst(i)=0		!modele de ZAMS
	   endif
	  endif		!dtp > 0
	 endif 		!ipms < 2
	 
	 tdst(i)=tdst(i)+diw2
	 
c	 les energies et proportions en %
	 
	 e_pp =e_pp +dm*epsilo(2)
	 e_cno=e_cno+dm*epsilo(3)
	 e_3a =e_3a +dm*epsilo(4)
	 e_gr =e_gr +dm*abs(tdst(i))/secon6
	enddo	
	e_tot=e_pp+e_cno+e_3a+e_gr
	if(e_tot .gt. 0.d0)then
	 i_pp=nint(100.*e_pp/e_tot)
	 i_cno=nint(100.*e_cno/e_tot)
	 i_3a=nint(100.*e_3a/e_tot)
	 i_gr=nint(100.*e_gr/e_tot)
	else
	 i_pp=0
	 i_cno=0
	 i_3a=0
	 i_gr=0
	endif
	
c	print*,'energies',i_pp,i_cno,i_gr
c	write(6,2000)e_tot,e_pp,e_cno,e_3a,e_gr,dt
c	pause'apres i_gr'
	 	
	call sbsp1dn(1,tdst,m23,m23t_t,n,m_ch,	!xchim, dxchim: VT
	1	knot23_t,.false.,m23(1),lx,xchim,dxchim)
c	do i=1,n
c	 call sbsp1dn(1,tdst,m23,m23t_t,n,m_ch,			!test
c	1	knot23_t,.true.,m23(i),lx,xchim,dxchim)
c	 print*,lx,m23(i),xchim(1)
c	enddo
c	pause'TdS'

c	write(6,2000)compg(nbelem*(n-1)+1),ab_min(1)
	pms_pr=pms
	zams_pr=zams
	post_pr=post
	cohe_pr=cohe
	coca_pr=coca
	if(compg(1) .gt. 1.d-3)then
c	 if(i_pp+i_cno+i_3a .ge. 99)then
	 if(i_pp+i_cno+i_3a .ge. 50)then	 
	  chaine='modele de la serie principale '
	  pms=.false.
	  zams=.true.
	  post=.false.
	  cohe=.false.
	  coca=.false.
	 else
	  chaine='modele de pre-serie principale'
	  pms=.true.
	  zams=.false.
	  post=.false.
	  cohe=.false.
	  coca=.false.
	 endif
	elseif(t(1) .lt. 1.d8)then
	 chaine= 'modele post serie principale  '
	 pms=.false.
	 zams=.false.
	 post=.true.
	 cohe=.false.
	 coca=.false.
	elseif(t(1) .lt. 6.d8)then
	 chaine= 'modele avec combustion helium '
	 pms=.false.
	 zams=.false.
	 post=.false.
	 cohe=.true.
	 coca=.false.	 
	else
	 chaine= 'modele avec combustion carbone'
	 pms=.false.
	 zams=.false.
	 post=.false.
	 cohe=.false.
	 coca=.true.
	endif
	
c	sauf pour le premier passage (alors passe=.false.)	

	zams_e= .not.zams_pr .and. zams .and. passe	!on passe PMS--->ZAMS
	post_e= .not.post_pr .and. post .and. passe	!on passe ZAMS-->POST
	cohe_e= .not.cohe_pr .and. cohe .and. passe	!on passe POST-->COHE
	coca_e= .not.coca_pr .and. coca .and. passe	!on passe COHE-->COCA	
	passe=.true.
	
	if(zams_e)then
	 open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.zams')
	 modelb='m'//cm//'X'//ch//'a'//cr//'_B.zams'
	 write(26)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=26)	!fermeture du fichier de ZAMS
	endif
	if(post_e)then
	 open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.post')
	 modelb='m'//cm//'X'//ch//'a'//cr//'_B.post'
	 write(26)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=26)	!fermeture du fichier de POST
	endif

	if(cohe_e)then
	 open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.cohe')
	 modelb='m'//cm//'X'//ch//'a'//cr//'_B.cohe'
	 write(26)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=26)	!fermeture du fichier de COHE
	endif
	if(coca_e)then
	 open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.coca')
	 modelb='m'//cm//'X'//ch//'a'//cr//'_B.coca'
	 write(26)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=26)	!fermeture du fichier de COCA
	endif

c	write(6,*)'e_tot,e_pp,e_cno,e_3a,e_gr',e_tot,e_pp,e_cno,e_3a,e_gr
c	pause

c---------------------------------------------------------------------

c		E C R I T U R E S 

c---------------------------------------------------------------------

	logg=log10(msol*mstar/rsol/rsol*g)	!pour ecriture de Log Teff
	
	lteff=log10(cte8*l(n)/r(n)**2)/4    !teff approche pour le test d'arret
	
	sort=		(age .ge. agemax-1.d-5)
	1	.or. (compg(1) .lt. x_stop)
	2	.or. (abs(compg(1)-x_stop) .lt. 0.001)	
	3	.or. (t(1) .ge. t_stop)
	4	.or. (zams .and. arret .eq. 'zams') 
	5	.or. (post .and. arret .eq. 'post')
	6	.or. (cohe .and. arret .eq. 'cohe')
	7	.or. (coca .and. arret .eq. 'coca')	
	
	if(log_teff .gt. 0.)then
	 sort=sort .or. lteff .gt. log_teff
	else
	 sort=sort .or. lteff .lt. -log_teff
	endif	
	
	ecrit=sort .or. zams_e .or. post_e .or. cohe_e .or. coca_e
	1	.or. (age-agep .ge. dtlist)
		
c	ecriture du *.osc, du *.dat et arret	
	
	if(ecrit)then
c	 write(48,*)'**n/age/m,p,t,ro,r,l,xchim,dxchim/epsi(5),xdot,ydot**'
c	 write(48,*)n
c	 write(48,2000)age
c	 write(6,2000)dt
c	 pause'dt'
	 
	 do i=1,n
	  do j=1,nbelem
	   xchim(j)=compchim(nbelem*(i-1)+j)
	   dxchim(j)=dcompchim(nbelem*(i-1)+j)
	   chimg(j)=compg(nbelem*(i-1)+j)
	   dchimg(j)=dcompg(nbelem*(i-1)+j)
	  enddo
c	  write(6,2000)m(i),r(i),t(i),pt(i),p(i),l(i),(chimg(j),j=1,nbelem)

c	  elements lourds	  
	  
	  if(iz .gt. 0)then
	   z(i)=xchim(iz)
	  elseif(ihe4 .gt. 1 .and. .not.z_cte)then
	   z(i)=1.d0-chimg(1)-chimg(ihe4)-chimg(ihe4-1)
	  else
	   z(i)=z0	  
	  endif

	  if(ipms .le. 2)then			!TdS par Iben
	   call thermo(pt(i),p(i),t(i),xchim,m(i),l(i),r(i),dlpp(i),dxchim,mstar,
	1	ro(i),drop,drot,drox,eint(i),dup,dut,dux,r_zc,r_ov,lim,
	2	grad(i),dgradpt,dgradp,dgradt,dgradx,dgradm,dgradl,dgradr,dgradlpp,
	3	gam,dgampt,dgamp,dgamt,dgamx,dgamm,dgaml,dgamr,dgamlpp,	
	4	epsilo,depsp,depst,depsx,kap(i),dkapp,dkapt,dkapx,
	5	delta(i),deltap,deltat,deltax,cp(i),dcpp,dcpt,dcpx,
	6	gradad(i),dgradadp,dgradadt,dgradadx,
	7	hp(i),dhppt,dhpp,dhpt,dhpx,dhpr,dhpm,gradrad(i),alfa(i),beta(i),
	8	gamma1(i),convec(i),.true.,etat,opa,conv,iben)
	   epsilo(1)=0
	  else
	   call thermo(pt(i),p(i),t(i),xchim,m(i),l(i),r(i),dlpp(i),dxchim,mstar,
	1	ro(i),drop,drot,drox,eint(i),dup,dut,dux,r_zc,r_ov,lim,
	2	grad(i),dgradpt,dgradp,dgradt,dgradx,dgradm,dgradl,dgradr,dgradlpp,
	3	gam,dgampt,dgamp,dgamt,dgamx,dgamm,dgaml,dgamr,dgamlpp,	
	4	epsilo,depsp,depst,depsx,kap(i),dkapp,dkapt,dkapx,
	5	delta(i),deltap,deltat,deltax,cp(i),dcpp,dcpt,dcpx,
	6	gradad(i),dgradadp,dgradadt,dgradadx,
	7	hp(i),dhppt,dhpp,dhpt,dhpx,dhpr,dhpm,gradrad(i),alfa(i),beta(i),
	8	gamma1(i),convec(i),.true.,etat,opa,conv,nuc)
	  endif		!ipms
	  
	  dcapdr(i)=dkapp/drop
	  dcapdt(i)=dkapt-dcapdr(i)*drot
	  depsdr(i)=depsp/drop
	  depsdt(i)=depst-depsdr(i)*drot
	  
c	  estimation des taux d'ionisation
c	  et calcul des poids moleculaires moyens

	  call saha(xchim,t(i),ro(i),ioni,zbar,nel,lamb(i))
	  nh1(i)=ioni(1,1)
	  if(ihe4 .gt. 1)then
	   nhe1(i)=ioni(1,ihe4)
	   nhe2(i)=ioni(2,ihe4)
	  else
	   nhe1(i)=0.d0
	   nhe2(i)=0.d0
	  endif
	  do j=1,nchim	  
	   mue(i)=mue(i)+zbar(j)*xchim(j)
	   mu(i)=mu(i)+(1.d0+zbar(j))*xchim(j)
	  enddo
	  mue(i)=1.d0/mue(i)	  
	  mu(i)=1.d0/mu(i)	  
	  
c	  print*,i,pturb
c	  write(6,2000)pt(i),p(i),t(i),xchim(1),m(i),l(i),r(i),dlpp(i)	  
c	  write(6,2000)drop,dkapp,dcapdr(i)
c	  write(6,2000)dcapdt(i),dkapt,dcapdr(i),drot
c	  write(6,2000)depsdr(i),depsp,depsdt(i),depst,depsdr(i)	  	  

c	  calcul d'une des formes de Vaissala (cf. Kippenhahan & Weigert p. 42) 
c	  1/gamma1 dlnP/dr - dln ro/dr =
c	  r / Hp delta ( grad_ad - grad ) - 4 pi r^3 dro/dX  dX/dm
c	  le 4 pi r^3 dro/dX  dX/dm est une approx. de phi grad_mu

	  if(hp(i) .le. 0.d0)then		!au centre
	   vaissala(i)=0.d0
	  else		!on prend grad_mj a cause des overshoot
	   vaissala(i)=r(i)*rsol/hp(i)*(alfa(i)*(dlpp(i)-1.d0)
	1	+delta(i)*(gradad(i)-grad_mj(i)))
	2	-cte3*r(i)**3*drox*dchimg(1)
	
c	   write(6,*)'m,r,drot,t,p,grad,gradad,drox/vai,dchimg,nuc'
c	   write(6,2000)m(i),r(i),drot,t(i),p(i),grad(i),gradad(i),drox
c	   write(6,2000)vaissala(i),dchimg(1)

	  endif
	  convec(i)=.not.convec(i)   !car thermo sort avec convec=.true. dans ZR
	  if(convec(i))then
	   gradconv(i)=grad(i)
	  else
	   gradconv(i)=0.d0
	  endif
	  
c 	  production de neutrinos

	  call nuc(t(i),ro(i),xchim,bidt,bidt,.false.,4,
	1	bidt,bidt,bidt,bidt,anupp(i),anupep(i),anub8(i),anube7(i),
	2	anun13(i),anuo15(i))

c	  dX/dt et dY/dt

	  if(ipms .le. 2)then
	   call iben(t(i),ro(i),xchim,dcomp,bidt,.false.,2,
	1	bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt)
	  else
	   call nuc(t(i),ro(i),xchim,dcomp,bidt,.false.,2,
	1	bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt,bidt)
	  endif
	  xdot(i)=dcomp(1)/secon6*ah
	  if(ihe4 .le. 1)then
	   ydot(i)=0.
	  else
	   ydot(i)=dcomp(ihe4)/secon6*ahe4
	  endif

c	  les epsilon et dR/dt

	  call sbsp1dn(1,tdst,m23,m23t_t,n,m_ch,
	1	knot23_t,.true.,m23(i),lx,tds,bid)
	  epsilo(5)=tds/secon6
	  if(dt .le. 0.)then
	   rdot(i)=0.
	  else
	   call inter('m23',bp_t,q_t,qt_t,n_t,knot_t,m23(i),f,dfdx,r2,m23)
	   if(en_masse)then
	    rdot(i)=(sqrt(abs(bp(ne*(i-1)*m_qs+3)))-sqrt(f(3)))/dt*cte7
	   else	   
	    rdot(i)=(abs(bp(ne*(i-1)*m_qs+3))-f(3))/dt*cte7	   
	   endif
	  endif
	  epsilo(1)=epsilo(1)-epsilo(5)
c	  write(6,*)'epsilo'
c	  write(6,2000)(epsilo(j),j=1,5)
	  do j=1,5
	   epsilon(5*(i-1)+j)=epsilo(j)
	  enddo
	 enddo
	 
c	 atmosphere, temperature effective, rayon exterieur

c	 write(6,*)'l(n),r(n),xchim en n'
c	 write(6,2000)l(n),r(n),(chim1g(j),j=1,nchim)
	 call lim_ext(.true.,l(n),r(n),chim1g,
	1	ptext,dptdl,dptdr,text,dtdl,dtdr,
	2	mext,dml,dmr,pext,dpl,dpr,teff,rstar,
	3	pt_atm,t_atm,m_atm,tau,r_atm,p_atm,dlpp_atm,mstar,
	4	tdetau,etat,opa,conv)
	 do i=1,nchim
	  dxchim(i)=0
	 enddo
	 do i=1,n_atm		!thermo pour l'atmosphere
	  grav=cte1*m_atm(i)/r_atm(i)**2	 	 
	  call tdetau(tau(i),teff,grav,bid,dtsdtau,dtsdteff,dtsdg,
	1	ro_ext,dro_grav,dro_teff,f_tau,df_tau,d2f_tau)
c	  write(6,2000)tau,f_tau,df_tau,d2f_tau

c	  les gradients

	  call thermo_atm(pt_atm(i),p_atm(i),t_atm(i),chim1g,m_atm(i),l(n),
	1	r_atm(i),dlpp_atm(i),tau(i),f_tau,df_tau,d2f_tau,rstar,	
	2	ro_atm(i),drop,drot,k_atm(i),dkapp,dkapt,
	3	grada_atm(i),dgradadp,dgradadt,
	4	grad_atm(i),dgradpt,dgradp,dgradt,dgradr,dgradrs,dgradm,
	5	dgradtau,dgradlpp,gam,dgampt,dgamp,dgamt,dgamr,dgamrs,dgamm,
	6	dgamtau,dgamlpp,hp_atm(i),dhppt,dhpp,dhpt,dhpr,dhpm,
	7	delta_atm(i),deltap,deltat,cp_atm(i),gradr_atm(i),
	8	alfa_atm(i),beta_atm(i),gamma1_atm(i),rad_atm,.true.,
	9	etat,opa,conv)
	  if(rad_atm)then
	   gradc_atm(i)=0.
	  else
	   gradc_atm(i)=grad_atm(i)
	  endif
	 
	  lnpt_a(i)=log(pt_atm(i))
	  lnt_a(i)=log(t_atm(i))
c	  write(6,2000)p_atm(i),t_atm(i),xchim1(1),m_atm(i),l(n),r(n)

	  ldcapdr_a(i)=dkapp/drop
	  ldcapdt_a(i)=dkapt-ldcapdr_a(i)*drot
c	  write(6,2000)m_atm(i),r_atm(i)
	  
c	  dans l'atmosphere pas de gradient de comp. chim
c	  pour vaissala on prend ou bien le vrai gradient grad_mj
c	  ou celui calcule aux points de grille
c	  X=cte dans l'atmosphere	 
	 	 
	  vais_atm(i)=r_atm(i)*rsol/hp_atm(i)*(alfa_atm(i)*(dlpp_atm(i)-1.d0)
	1	+delta_atm(i)*(grada_atm(i)-grad_atm(i)))

c	  write(6,*)'m,r,drot,t,p,gradient,gradad,vai'
c	  write(6,2000)m_atm(i),r_atm(i),drot,t_atm(i),p_atm(i),grad_atm(i),
c	1	grada_atm(i),vais_atm(i)
  
c	  estimation des taux d'ionisation
c	  et calcul des poids moleculaires moyens

	  call saha(xchim1,t_atm(i),ro_atm(i),ioni,zbar,nel,bid)
	  nh1_atm(i)=ioni(1,1)
	  if(ihe4 .gt. 1)then
	   nhe1_atm(i)=ioni(1,ihe4)
	   nhe2_atm(i)=ioni(2,ihe4)
	  else
	   nhe1_atm(i)=0.d0
	   nhe2_atm(i)=0.d0
	  endif
	  do j=1,nchim	  
	   mue_atm(i)=mue_atm(i)+zbar(j)*xchim1(j)
	   mu_atm(i)=mu_atm(i)+(1.d0+zbar(j))*xchim1(j)
	  enddo
	  mue_atm(i)=1.d0/mue_atm(i)	  
	  mu_atm(i)=1.d0/mu_atm(i)
	 enddo
	 
c	estimation du gradient  

	 if(n_atm .gt. m_ch)then
	  do i=1,n_atm	!on prend l'oppose pour tabuler ln T en fct. de ln P
	   lnpt_a(i)=-lnpt_a(i)	!les abs. devant etre croissantes
	  enddo
	  call sbsp1d(lnt_a,lnpt_a,bidt,n_atm,m_ch,
	1	knota,.false.,lnpt_a(1),lx,d2p,grad_mj_a(1))	!d2p: VT
	  do i=1,n_atm
	   call sbsp1d(lnt_a,lnpt_a,bidt,n_atm,m_ch,knota,.true.,
	1	lnpt_a(i),lx,d2p,grad_mj_a(i))		!d2p: VT
	  enddo
	  do i=1,n_atm	!grad_mj aux points de grille
	   lnpt_a(i)=-lnpt_a(i)
	   grad_mj_a(i)=-grad_mj_a(i)	!on prend l'oppose
c	   write(6,2000)p_atm(i),grad_mj_a(i),grad_atm(i)	  
c	   vais_atm(i)=r_atm(i)*rsol/hp_atm(i)*(alpha_atm(i)*(dlpp_atm(i)-1.d0))
c	1	+delta_atm(i)*(grada_atm(i)-grad_mj_a(i)))	   
	  enddo
c	  pause'atm cesam'
	 endif
	
c	 sorties sur ecran

c	 print*,mstar,mtot
	 pertem=mstar/mtot-1.d0
	 if(ihe4 .le. 1)then
	  write(6,205)age,log10(teff),log10(l(n)),log10(r(n)),
	1	logg+log10(m(n)/r(n)**2),
	2	pt(1),t(1),ro(1),compg(nbelem*(1-1)+1),
	3	i_pp,i_cno,i_3a,i_gr,pertem,mstar,chaine
205	  format(1x,'*********',/,1x,'age=',1pd10.3,' log Teff=',1pd10.3,
	1	' log L/Lsol=',1pd10.3,' log R/Rsol=',1pd10.3,/,
	2	1x,'Log10 g=',1pd10.3,' Pc=',1pd10.3,' Tc=',1pd10.3,
	3	' Roc=',1pd10.3,' Xc=',1pd10.3,
	4	/,1x,'en. PP=',i3,'%, en. CNO=',i3,'%, en. 3 alpha=',i3,
	5	'%, en. grav=',i3,'%',/,1x,'Var. rel. de masse :',1pd10.3,
	6	', M*=',1pd10.3,'Msol, ',a30,/,1x,'*********')
	 else
	  write(6,206)age,log10(teff),log10(l(n)),log10(r(n)),
	1	logg+log10(m(n)/r(n)**2),
	2	pt(1),t(1),ro(1),compg(nbelem*(1-1)+1),
	3	i_pp,i_cno,i_3a,i_gr,compg(nbelem*(1-1)+ihe4),
	4	pertem,mstar,chaine
206	  format(1x,'*********',/,1x,'age=',1pd10.3,' log Teff=',1pd10.3,
	1	' log L/Lsol=',1pd10.3,' log R/Rsol=',1pd10.3,/,
	2	1x,'Log10 g=',1pd10.3,' Pc=',1pd10.3,' Tc=',1pd10.3,
	3	' Roc=',1pd10.3,' Xc=',1pd10.3,
	4	/,1x,'en. PP=',i3,'%, en. CNO=',i3,'%, en. 3 alpha=',i3,
	5	'%, en. grav=',i3,'%',' Yc=',1pd10.3,/,1x,
	6	'Var. rel. de masse :',1pd10.3,
	7	', M*=',1pd10.3,'Msol, ',a30,/,1x,'*********')	
	 endif
	 
c	 print*,dessin
c	 pause'avant dessin'

	 if(dessin)call des_cesam(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,teff,
	1	r2,m23,age,lim,lconv,m_zc,mstar)
	 
	 write(53,52)age,nbelem,lim,(lconv(i),i=1,lim)	!pour les ZC et diag. HR
52	 format(1pd22.15,2i3,20l2)
	 write(53,2002)log10(teff),log10(l(n)),log10(r(n)),mstar,
	1	(mstar-m_zc(i),r_zc(i),r_ov(i),i=1,lim)
2002	 format(1p6d13.6)
	 write(53,2003)(nom_elem(i),compg(i),compg(nbelem*(n-1)+i),i=1,nbelem)
2003	 format(a4,1p2d12.5)

c	 derivees secondes au centre

	 d2p=cte5*ro(1)**2*rstar**2/p(1)
	 fonc(0)=ro(1)
	 fonc(1)=ro(2)
	 fonc(2)=ro(3)
	 fonc(3)=ro(4)
	 fonc(4)=ro(5)
	 fonc(5)=ro(6)
	 absc(0)=r(1)
	 absc(1)=r(2)
	 absc(2)=r(3)
	 absc(3)=r(4)
	 absc(4)=r(5)
	 absc(5)=r(6)

c	 write(6,*)'ro/r/deriv'
c	 write(6,2000)(fonc(i),i=0,5)
c	 write(6,2000)(absc(i),i=0,5)

	 call newton(0,m_qs+1,fonc,absc,0.d0,poly,2) 
	 d2ro=rstar**2/ro(1)*poly(2)

c	 write(6,2000)(poly(i),i=0,2)
c	 write(6,*)'rstar,d2ro,ro,poly(2)'
c	 write(6,2000)rstar,d2ro,ro(n),poly(2)
c	 write(6,*)'d2p,d2ro,rstar,p(n),ro(n)'
c	 write(6,2000)d2p,d2ro,rstar,p(n),ro(n)

c	 verification de la solution: estimation des premiers et 2d membres
c	 des equations d'evolution

c 	 do i=2,n-1
c	  write(6,2000)(t(i+1)-t(i-1))/(m(i+1)-m(i-1))/(mtot*msol),
c	1	-g*m(i)*mtot*msol/4./pi/(rsol*r(i))**4*grad(i)*t(i)/p(i),
c	2	(pt(i+1)-pt(i-1))/(m(i+1)-m(i-1))/(mtot*msol),
c	3	-g*m(i)*mtot*msol/4./pi/(rsol*r(i))**4,
c	4	(r(i+1)-r(i-1))*rsol/(m(i+1)-m(i-1))/(mtot*msol),
c	5	1./4./pi/(rsol*r(i))**2/ro(i),
c	6	(l(i+1)-l(i-1))*lsol/(m(i+1)-m(i-1))/(mtot*msol),
c	7	epsilon(5*(i-1)+1)
c	 enddo
c	 if(.true.)stop

c	 sortie

	 if(sort)then
	  call list(n,m,pt,p,ro,t,r,l,compg,age,teff,rstar,hp,mstar,
	1	methode,precix,.true.,kap,epsilon,eint,xdot,ydot,rdot,gamma1,
	2	nh1,nhe1,nhe2,alfa,beta,delta,cp,vaissala,convec,
	3	gradconv,gradrad,gradad,dcapdt,dcapdr,depsdr,depsdt,d2p,d2ro,
	4	anupp,anupep,anub8,anube7,anun13,anuo15,lamb,w,z,chaine,
	5	pt_atm,p_atm,t_atm,m_atm,tau,r_atm,ro_atm,k_atm,gradr_atm,grada_atm,
	6	gradc_atm,n_atm,i_pp,i_cno,i_3a,i_gr)
	
	  write(6,*)' '
	  write(6,*)'nom du modele : ',model
	  write(6,*)' '
	  write(6,*)'nom du fichier des NAMELISTs des donnees : ',
	1	nom_fich2(:long(nom_fich2))//'.don'
	  write(6,*)' '
	  write(6,*)'nom du fichier du listing des resultats en ASCII : ',
	1	nom_fich2(:long(nom_fich2))//'.lis'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele d''age 0 PMS en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.pms'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele d''age 0 homogene en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.hom'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele de ZAMS en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.zams'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele de POST en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.post'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele de COHE en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.cohe'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele de COCA en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.coca'
	  write(6,*)' '
	  write(6,*)'nom du fichier du modele final en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.dat'
	  write(6,*)' '
	  write(6,*)'nom des fichiers des modeles intermediaires en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.rep'
	  write(6,*)' '
	  write(6,*)'nom du fichier pour trace du diag. HR et ZC : ',
	1	nom_fich2(:long(nom_fich2))//'.HR'
	  if(n_atm .gt. 0)then
	   write(6,*)' '
	   write(6,*)'nom du fichier du modele d''atmosphere en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.atm'
	  endif	
	
	  write(2,*)' '
	  write(2,*)'nom du modele : ',model
	  write(2,*)' '
	  write(2,*)'nom du fichier des NAMELISTs des donnees : ',
	1	nom_fich2(:long(nom_fich2))//'.don'
	  write(2,*)' '
	  write(2,*)'nom du fichier du listing des resultats en ASCII : ',
	1	nom_fich2(:long(nom_fich2))//'.lis'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele d''age 0 PMS en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.pms'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele d''age 0 homogene en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.hom'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele de ZAMS en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.zams'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele de POST en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.post'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele de COHE en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.cohe'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele de COCA en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.coca'
	  write(2,*)' '
	  write(2,*)'nom du fichier du modele final en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.dat'
	  write(2,*)' '
	  write(2,*)'nom des fichiers des modeles intermediaires en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.rep'
	  write(2,*)' '
	  write(2,*)'nom du fichier pour trace du diag. HR et ZC : ',
	1	nom_fich2(:long(nom_fich2))//'.HR'
	  if(n_atm .gt. 0)then
	   write(2,*)' '
	   write(2,*)'nom du fichier du modele d''atmosphere en binaire : ',
	1	nom_fich2(:long(nom_fich2))//'_B.atm'
	  endif
	  write(6,*)' '
	  
	  write(2,*)' '	
	  write(2,*)'***********************************'
	  write(2,*)' '
	  write(2,*)' Sortie de CESAM version: ',version
	  write(2,*)' '
	  write(2,*)'Ce code d''evolution stellaire a ete elabore',
	1	'dans le cadre du Groupement de Recherche Structure Interne',
	2	'des Etoiles et des Planetes Geantes.'
	  write(2,*)' '
	  write(2,*)'  Si son utilisation vous a donne satisfaction, le but ',
	1	'poursuivi par tous ceux qui ont y ont contribue ',
	2	'aura ete atteint.'
	  write(2,*)' '
	  write(2,*)'P. Morel, ON. Decembre 1989, V 1'
	  write(2,*)'P. Morel, OCA. Octobre 1991, V 2'
	  write(2,*)'P. Morel, OCA. Avril 1993, V 3'
	  write(2,*)'P. Morel, OCA. Decembre 1997, V 4'
	  
	  close(unit=2)

c	  on garde le modele final

	  open(unit=25,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.dat')
	  modelb='m'//cm//'X'//ch//'a'//cr//'_B.dat'
	  write(25)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	  close(unit=25)		!fichier *.dat
	  close(unit=53)		!fichier du diagramme HR et des ZC

c	  fichier d'oscillation

	  write(6,*)' '
	  write(6,*)'formation du fichier pour oscillations ? o/n'
	  read(5,'(a)')oui
	  if(oui .eq. 'o')then
	   open(unit=30,form='formatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'.osc')
	   write(6,*)'nom du fichier ASCII pour le calcul des oscillations :',
	1	nom_fich2(:long(nom_fich2))//'.osc'

c	   date

	   call date_and_time(date,time,zone,values)

	   titre=modelb(:long(modelb))//' du '//date(7:8)//' '//
	1	mois(values(2))(:long(mois(values(2))))//
	2	' '//date(1:4)//' ('//time(1:2)//'h'//time(3:4)//')'
	3	//' - P. Morel, OCA'
	
c------------------------------------------------------	
	
	   write(30,*)titre(:long(titre))
	   write(30,*)'fichier pour le calcul des oscillations'
	   write(30,*)' fichier: '//nom_fich2(:long(nom_fich2))//'.osc'
	   write(30,*)methode(:long(methode))
	   write(30,90)nbelem,(nom_elem(i),i=1,nbelem)
90	   format(i3,14(1x,a4))

	   iconst=15		!nb de global
	   ivar=22		!nb de variables
	   ivers=4		!numero de version

	   glob(1)=mstar*msol		!Masse a l'interieur de R*
	   glob(2)=rstar*rsol		!R* rayon a R(tau*)
	   glob(3)=l(n)*lsol		!Luminosite
	   glob(4)=z0			!Z initial
	   glob(5)=x0			!X initial
	   glob(6)=alpha		!Parametre de longueur de melange
	   glob(7)=9./4.		!Parametre de convection arbitraire
	   glob(8)=1./162.		!Parametre de convection arbitraire
	   glob(9)=compg(nbelem*(n-1)+1)		!X dans ZC
	   glob(10)=compg(nbelem*(n-1)+ihe4)		!Y dans ZC
	   if(ihe4 .gt. 1)then
	    glob(10)=compg(nbelem*(n-1)+ihe4)+compg(nbelem*(n-1)+ihe4-1)
	   else
	    glob(10)=1.d0-glob(9)-z0
	   endif
	   glob(11)=d2p			!Derivee seconde au centre
	   glob(12)=d2ro		!Derivee seconde au centre
	   glob(13)=age			!Temps d'evolution
	   glob(14)=2./9.		!Parametre de convection arbitraire
	   glob(15)=w_rot
	   
	   nadd=max(0,n_atm-1)
	   if(nadd .gt. 0)then		!on ajoute l'atmosphere sauf la couche 1
	    if(ihe4 .gt. 1)then
	     y=chim1g(ihe4)
	    else	     
	     y=1.d0-z0-chim1g(1)
	    endif
	    do i=1,nadd
	     ipn=n_atm-i+1
	     var(1,i)=r_atm(ipn)*rsol			!Rayon
	     var(2,i)=log(m_atm(ipn)/mstar)		!Ln M/Mtot
	     var(3,i)=t_atm(ipn)			!Temperature
	     var(4,i)=pt_atm(ipn)			!Pression
	     var(5,i)=ro_atm(ipn)			!Densite
c	     var(6,i)=grad_mj_a(ipn)			!Gradient
	     var(6,i)=grad_atm(ipn)			!Gradient
	     var(7,i)=l(n)*lsol				!Luminosite
	     var(8,i)=k_atm(ipn)			!Opacite
	     var(9,i)=0.				!Energie nuc+grav
	     var(10,i)=gamma1_atm(ipn)			!Grand Gamma1
	     var(11,i)=grada_atm(ipn)			!Gradient adiabatique
	     var(12,i)=delta_atm(ipn)			!Delta
	     var(13,i)=cp_atm(ipn)			!Cp
c	     var(14,i)=mue_atm(ipn)			!Mue^(-1)
	     var(14,i)=mu_atm(ipn)			!Mu	     
	     var(15,i)=vais_atm(ipn)			!Vaissala
	     var(16,i)=w(n)				!Omega
	     var(17,i)=ldcapdt_a(ipn)*t_atm(ipn)/k_atm(ipn) !dln kappa / dln T
	     var(18,i)=ldcapdr_a(ipn)*ro_atm(ipn)/k_atm(ipn) !dln kappa / dln ro
	     var(19,i)=0.				!d epsilon(nuc) / d ln T
	     var(20,i)=0.				!d epsilon(nuc) / d ln ro
	     var(21,i)=pt_atm(ipn)/p_atm(ipn)		!Ptot/Pgaz
	     var(22,i)=gradr_atm(ipn)			!Gradient radiatif	     
	     do j=1,nbelem
	      var(ivar+j,i)=chim1g(j)			!Abondances / gramme
	     enddo
	    enddo
	   endif		!atmosphere

c	   la structure interne

	   do i=n,1,-1	    
	    do j=1,nbelem		!par gramme
	     chimg(j)=compg(nbelem*(i-1)+j)
	    enddo
	    if(ihe4 .gt. 1)then
	     y=chimg(ihe4)
	    else
	     y=1.d0-z0-chimg(1)
	    endif
	    	    
	    ipn=n-i+nadd+1
	    var(1,ipn)=r(i)*rsol		!Rayon
	    if(m(i) .ne. 0.)then
	     var(2,ipn)=log(m(i)/mstar)		!Ln M/Mtot
	    else
	     var(2,ipn)=-1.d38		        !Ln M/Mtot
	    endif
	    var(3,ipn)=t(i)			!Temperature
	    var(4,ipn)=pt(i)			!Pression
	    var(5,ipn)=ro(i)			!Densite
c	    var(6,ipn)=grad_mj(i)
	    var(6,ipn)=grad(i)			!Gradient	    
	    var(7,ipn)=l(i)*lsol		!Luminosite
	    var(8,ipn)=kap(i)			!Opacite
	    var(9,ipn)=epsilon(5*(i-1)+1)	!Energie nuc+grav
	    var(10,ipn)=gamma1(i)		!Grand Gamma1
	    var(11,ipn)=gradad(i)		!Gradient adiabatique
	    var(12,ipn)=delta(i)		!Delta
	    var(13,ipn)=cp(i)			!Cp
c	    var(14,ipn)=mue(i)			!Mue^(-1)
	    var(14,ipn)=mu(i)			!Mu	    
	    var(15,ipn)=vaissala(i)		!Vaissala
	    var(16,ipn)=w(i)			!Omega
	    var(17,ipn)=dcapdt(i)*t(i)/kap(i)	!dln kappa / dln T
	    var(18,ipn)=dcapdr(i)*ro(i)/kap(i)	!dln kappa / dln ro
	    var(19,ipn)=depsdt(i)*t(i)		!d epsilon(nuc) / d ln T
	    var(20,ipn)=depsdr(i)*ro(i)		!d epsilon(nuc) / d ln ro
	    var(21,ipn)=pt(i)/p(i)		!Ptot/Pgaz
	    var(22,ipn)=gradrad(i)		!Gradient radiatif	        
	    do j=1,nbelem
	     var(ivar+j,ipn)=compg(nbelem*(i-1)+j)!Abondances / gramme
	    enddo
	   enddo
	   write(30,137)n+nadd,iconst,ivar,ivers,nchim,iw,iz
137	   format(7i10)
	   write(30,138)(glob(i),i=1,iconst)
138	   format(1p5d19.12)
	   do j=1,n+nadd
	    write(30,138)(var(i,j),i=1,ivar+nbelem)
	   enddo
	   close(unit=30)
	  endif
	  if(dessin)call pgend

	  write(6,*)' '
	  write(6,*)'***********************************'
	  write(6,*)' '
	  write(6,*)' Sortie de CESAM version: ',version
	  write(6,*)' '
	  write(6,*)'Ce code d''evolution stellaire a ete elabore',
	1	'dans le cadre du Groupement de Recherche Structure Interne',
	2	'des Etoiles et des Planetes Geantes.'
	  write(6,*)' '
	  write(6,*)'  Si son utilisation vous a donne satisfaction, le but ',
	1	'poursuivi par tous ceux qui ont y ont contribue ',
	2	'aura ete atteint.'
	  write(6,*)' '
	  write(6,*)'P. Morel, ON. Decembre 1989, V 1'
	  write(6,*)'P. Morel, OCA. Octobre 1991, V 2'
	  write(6,*)'P. Morel, OCA. Avril 1993, V 3'
	  write(6,*)'P. Morel, OCA. Decembre 1997, V 4'	  
	  write(6,*)' '
	  write(6,*)'***********************************'
	  write(6,*)' '
	  
	  stop	!****** F I N  D U  C A L C U L *******

	 else			!ecriture
	  call list(n,m,pt,p,ro,t,r,l,compg,age,teff,rstar,hp,mstar,
	1	methode,precix,.true.,kap,epsilon,eint,xdot,ydot,rdot,gamma1,
	2	nh1,nhe1,nhe2,alfa,beta,delta,cp,vaissala,convec,
	3	gradconv,gradrad,gradad,dcapdt,dcapdr,depsdr,depsdt,d2p,d2ro,
	4	anupp,anupep,anub8,anube7,anun13,anuo15,lamb,w,z,chaine,
	5	pt_atm,p_atm,t_atm,m_atm,tau,r_atm,ro_atm,k_atm,gradr_atm,grada_atm,
	6	gradc_atm,n_atm,i_pp,i_cno,i_3a,i_gr)
	  agep=age
	 endif

c	 ecriture du fichier de reprise car il n'y a pas eu arret
c	 et donc formation d'un *.dat puis ecriture reduite

	else
	 if(statut)then
	  reprise=reprise+1
c	  write(6,*)'reprise',reprise
	  if(reprise .le. 99)then
	   encode(2,2,l2)reprise
	   open(unit=26,form='unformatted',status='new',
	1	file=nom_fich2(:long(nom_fich2))//l2//'_B.rep')
	  elseif(reprise .le. 999)then
	   encode(3,1,l3)reprise
	   open(unit=26,form='unformatted',status='new',
	1	file=nom_fich2(:long(nom_fich2))//l3//'_B.rep')
	  elseif(reprise .le. 9999)then
	   encode(4,3,l4)reprise
	   open(unit=26,form='unformatted',status='new',
	1	file=nom_fich2(:long(nom_fich2))//l4//'_B.rep')
	  else
	   write(6,*)'les modeles ne sont plus tous conserves'
	   statut=.false.
	   open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.rep')
	  endif
	 else
	  open(unit=26,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.rep')
	 endif
	 
	 modelb='m'//cm//'X'//ch//'a'//cr//'_B.rep'
	 write(26)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	 close(unit=26)	!fermeture du fichier du point de reprise
c	 print*,'apres ecriture du .rep dt ',dt
	  
c	 ecriture reduite
	  
	 call lim_ext(.true.,l(n),r(n),chim1g,
	1	ptext,dptdl,dptdr,text,dtdl,dtdr,
	2	mext,dml,dmr,pext,dpl,dpr,teff,rstar,
	3	pt_atm,t_atm,m_atm,tau,r_atm,p_atm,dlpp_atm,mstar,
	4	tdetau,etat,opa,conv)

c	 print*,mstar,mtot
	 pertem=mstar/mtot-1.d0
	 if(ihe4 .le. 0)then
	  write(6,205)age,log10(teff),log10(l(n)),log10(r(n)),
	1	logg+log10(m(n)/r(n)**2),
	2	pt(1),t(1),ro(1),compg(nbelem*(1-1)+1),
	3	i_pp,i_cno,i_3a,i_gr,pertem,mstar,chaine
	 else
	  write(6,206)age,log10(teff),log10(l(n)),log10(r(n)),
	1	logg+log10(m(n)/r(n)**2),
	2	pt(1),t(1),ro(1),compg(nbelem*(1-1)+1),
	3	i_pp,i_cno,i_3a,i_gr,compg(nbelem*(1-1)+ihe4),
	4	pertem,mstar,chaine
	 endif

	 if(dessin)call des_cesam(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,teff,
	1	r2,m23,age,lim,lconv,m_zc,mstar)
	
	 write(53,52)age,nbelem,lim,(lconv(i),i=1,lim)		!pour les ZC et diag. HR	 
	 write(53,2002)log10(teff),log10(l(n)),log10(r(n)),mstar,
	1	(mstar-m_zc(i),r_zc(i),r_ov(i),i=1,lim)
	 write(53,2003)(nom_elem(i),compg(i),compg(nbelem*(n-1)+i),i=1,nbelem)
	 	 
	 call list(n,m,pt,p,ro,t,r,l,compg,age,teff,rstar,hp,mstar,
	1	methode,precix,.false.,kap,epsilon,eint,xdot,ydot,rdot,gamma1,
	2	nh1,nhe1,nhe2,alfa,beta,delta,cp,vaissala,convec,
	3	gradconv,gradrad,gradad,dcapdt,dcapdr,depsdr,depsdt,d2p,d2ro,
	4	anupp,anupep,anub8,anube7,anun13,anuo15,lamb,w,z,chaine,
	5	pt_atm,p_atm,t_atm,m_atm,tau,r_atm,ro_atm,k_atm,gradr_atm,grada_atm,
	6	gradc_atm,n_atm,i_pp,i_cno,i_3a,i_gr)
	endif				!ecritures

c---------------------------------------------------------------

c		E V O L U T I O N	T E M P O R E L L E

c---------------------------------------------------------------

c	pause'avant evolution'
	if(ipms .eq. 1)then
	 rp1=r(n)
	 lp1=l(n)
	 write(6,55)c_iben,rp1,lp1,t(1)
55	 format(1x,/,1x,'Pre Sequence Principale premier modele: C=',1pd10.3,/,
	1	1x,'Rext=',1pd10.3,', Lext=',1pd10.3,', Tc=',1pd10.3,/,
	2	1x,' ok? o/n')
	 read(5,'(a)')oui
c	 write(6,*)'oui ',oui
	 if(oui .eq. 'o')then
	  c_iben=c_iben*1.1
	  ipms=2
	  write(6,*)'calcul d''un nouveau modele avec 1.1 C'
	 else
	  write(6,*)' '
	  write(6,*)'entrer une nouvelle valeur pour C'
	  read(5,*)c_iben
	  ipms=1
	 endif
	 dt=0.
	 call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,iben,lim_ext,tdetau,coeff_diff,perte_m)
	 
	elseif(ipms .eq. 2)then
	 rp2=r(n)
	 lp2=l(n)
	 dt=mtot**2*g*msol/lsol/rsol*msol*2.*abs(rp1-rp2)/
	1	rp1/rp2/(lp1+lp2)/secon6
	 write(6,57)c_iben,rp2,lp2,t(1),dt
57	 format(1x,/,1x,'Pre Sequence Principale second modele: C=',1pd10.3,/,
	1	1x,'Rext=',1pd10.3,', Lext=',1pd10.3,', Tc=',1pd10.3,
	2	', dt=',1pd10.3,/,1x,' ok? o/n')
	 read(5,'(a)')oui
	 if(oui .eq. 'n')then		!nouvelle cte de contraction
	  write(6,*)' '
	  write(6,*)'entrer une nouvelle valeur pour C'
	  read(5,*)c_iben
	  ipms=1
	  age=0
	  dt=0.
	  call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,iben,lim_ext,tdetau,coeff_diff,perte_m)

	 else				!on est content
	  ipms=3
	  write(6,*)' '
	  write(6,*)'debut d''evolution PMS'
	  write(6,*)' '
	  age=0.
	  open(unit=24,form='unformatted',status='unknown',
	1	file=nom_fich2(:long(nom_fich2))//'_B.pms')
	  modelb='m'//cm//'X'//ch//'a'//cr//'_B.pms'
	  write(24)age,nbelem,nchim,mtot,alpha,modelb,x0,y0,ne,nreac,m_qs,m_ch,
	1	nom_elem,rot_solid,z_cte,w_rot,diffusion,iz,iw, 
	2	bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	3	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,
	4	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	5	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,lconv_t,
	6	mstar_t,en_masse
	  close(unit=24)
	  call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,nuc,lim_ext,tdetau,coeff_diff,perte_m)
	  
	 endif
	else
c	 pause'appel a resout'
	  call resout(bp,q,n,qt,knot,chim,mc,nc,mct,knotc,
	1	r2,m23,m_zc,r_zc,r_ov,lim,jlim,lconv,mstar,age,un23,
	2	bp_t,q_t,n_t,qt_t,knot_t,chim_t,mc_t,nc_t,mct_t,knotc_t,
	3	r2_t,m23_t,dt,m_zc_t,r_zc_t,r_ov_t,lim_t,jlim_t,
	4	lconv_t,mstar_t,m23t_t,knot23_t,tdst,
	4	ctes,etat,opa,conv,nuc,lim_ext,tdetau,coeff_diff,perte_m)
	endif
c	print*,'avant goto 100 dt ',dt
c	pause	

	age=age+dt
	dtp=dt		!pas temporel precedent

	goto 100

	end
