	
c*****************************************************************

	subroutine diff_burgers(melange,p,t,r,lum,m,ro,drox,kap,dkapx,
	1	gradrad,dgradradx,gradad,dgradadx,r_zc,lim,mstar,
	2	xchim,d,dd,v,dv)
	
c	inversion des equations de Burgers

c	P est recalculee a partir de ro, T, mu
c	on tient compte approximativement des ionisations
			
c	Auteur: P.Morel, OCA, Observatoire de Nice
c	CESAM4

c entrees
c	melange=.true. : on est dans une ZC, d=infini
c	p,t,r,lum,m : pression, temperature, luminosite locale, masse
c	ro,drox,kap,dkapx : densite, derivee/ X, opacite, derivee / X 
c	gradrad,dgradradx,gradad,dgradadx : gradient rad., ad. et derivees / X
c	r_zc,lim : rayons des limites ZR/ZC et nb. de limites
c	xchim,mstar : comp. chim. par mole, masse totale
c	les derivees / X, issues de thermo, sont par gramme	

c sorties
c	d, dd : coefficients d_ij de d x_j / d m et derivees / x_k
c	v, dv : coefficients v_i de x_i et derivees / x_k

c	Dimensions dans le programme appelant
c	d(nbelem,nbelem), dd(nbelem,nbelem,nbelem), v(nbelem), dv(nbelem,nbelem)

c	convention de notation :
c	equation de diffusion dXi/dt=dFi/dm + nuclear, i=1,nbelem
c	Fi=4 pi r**2 ro (4 pi r**2 ro D.dX/dm - Vi Xi)

c	d=D=(di1, di2,... din) avec Dij coefficient de d Xj / dm
c	dans le produit scalaire D.dX/dm=sum d_ij d Xj / dm

c	pour ligne d'indice i
c	v(i) coefficient de x_i,
c	dv(i,k)=dv(nbelem*(k-1)+i)=derivee v_i / x_k
c	d(i,j)=dv(nbelem*(j-1)+i)=coefficient d_ij de d x_j / dm
c	dd(i,j,k)=dd(nbelem*(nbelem*(k-1)+j-1)+i)= derivee de d_ij / x_k

c	avec cette routine, on ne peut diffuser Z comme un element trace

c	on calcule les derivees / x_k, k=1,nb, y compris pour les electrons x_e,
c	puis on considere que x_e=x_e(x_i), i=1,nchim

	implicit none
	
	include 'cesam.parametres'
	include 'ctephy.common'
	include 'evol_chim.common'

	integer ppl,ppc					!nb lignes, particules
	parameter(ppl=2*(pnchim+1)+1,ppc=pnchim+1)
	
	integer i,j,k,l,nl,nb,lim,lm(ppl),mm(ppl)
	
	real*8	p,t,ro,grav,xi(ppc),xchim(1),mi(ppc),gr,n0,n02k,
	1	mij(ppc**2),mijk(ppc**2),ai(ppc),cd(ppc**2),dcd(ppc**3),
	2	unpzb(ppc),om11(ppc**2),dom11(ppc**3),eta,
	3	mi1(ppc**2),mi2(ppc**2),mi3(ppc**2),mismj(ppc**2),
	4	kij(ppc**2),dkij(ppc**3),pij(ppc**2),dpij(ppc**3),
	5	fij(ppc**2),dfij(ppc**3),gamma(ppl),dgamma(ppl*ppc),
	6	d(1),v(1),zb(ppc),cgrav,d_turb,w_diff,d_conv,d_mel,
	7	dv(1),dd(1),r,lum,m,drox,drox1,b(ppl**2),db(ppl**2*ppc),
	8	kap,dkapx,gradrad,dgradradx,dgradradx1,ioni(0:pzi,pnchim),
	9	gradad,dgradadx,r_zc(1),mstar,a(ppl**2),da(ppl**2*ppc)
	
	real*8	zij(ppc**2),dzij(ppc**3),zpij(ppc**2),dzpij(ppc**3),
	1	zsij(ppc**2),dzsij(ppc**3),ro_n0_e,dro_n0_ex,
	2	mu,dmu(ppc),mu_p,dmu_px,
	3	cf(ppc**2),dcf(ppc**3),diag(ppc),ddiag(ppc**2),
	4	qij(ppc**2),dqij(ppc**3),bqij(ppc**2),dbqij(ppc**3),
	5	bid,vt(ppl),mt(ppl**2),
	6	ti,ts,di,ds,ddd,nel,gsro,dgsrox1,
	7	gsro_mu_grad,dgsro_mu_grad(ppc)
	
c*******************pour tests de derivation*******************	
c	integer der
c	real*8	a0(ppl**2),da0(ppl**2*ppc),dx,unpdx,stor,stor0,dstor
c*******************pour tests de derivation*******************
	
	logical	init,melange
	data init/.true./
	
	save	
	
2000	format(1x,1p10d10.3)
2001	format(1x,1p12d8.1)

c	initialisations

	if(init)then
	 init=.false.
	 
c	 coefficients de melange turbulent et convectif
	 
	 write(6,*)
	 write(2,*)
	 d_turb=10.d0
	 d_turb=0.d0	 
	 d_conv=1.d13
	 write(6,10)d_conv,d_turb
	 write(2,10)d_conv,d_turb
10	 format(1x,'Diff. micro. par eq. Burgers, dans ZR',/,
	1	1x,'tous les elements sont diffuses.',/,
	2	'On tient compte approximativement de l''ionisation',/,
	3	1x,'Dans ZC, Dconv=',1pd10.3,' Coeff. diff. turb.=',1pd10.3)

c	 limites et coefficients du melange supplementaire

	 ti=0.d0
	 ts=1.d6
	 di=1.d3
	 di=0.d0
	 ds=0.d0
	 ddd=(di-ds)/(ti-ts)	
	 write(2,13)di,ds,ti,ts
	 write(6,13)di,ds,ti,ts
13	 format(1x,'avec melange supplementaire pour T < Ts',/,
	1	1x,'Di=',1pd10.3,' Ds=',1pd10.3,' Ti=',1pd10.3,' Ts=',1pd10.3)
	
c	 coefficient de diffusion pour le moment angulaire
	
	 if(iw .gt. 0)then
	  w_diff=100.d0
c	  print*,'entrer w_diff'
c	  read*,w_diff
	  write(6,11)w_diff
	  write(2,11)w_diff	 
11	  format(t2,' avec diffusion constante du moment angulaire w_diff =',
	1	1pd10.3,//)
	 endif
	 if(iz .gt. 0)then
	  write(6,*)'diff_bgr ne peut pas diffuser Z en tant que trace, Abandon'
	  write(2,*)'diff_bgr ne peut pas diffuser Z en tant que trace, Abandon'
	  stop	  	 
	 endif
	 write(6,*)
	 write(2,*)
	 
 	 cgrav=msol*g/rsol**2	!pour le calcul de la gravite
 	 n0=1.d0/amu		!Avogadro	 
	 gr=kbol*n0		!Grand R gaz parfaits
	 n02k=16.d0/3.d0*n0**2	!pour les K_ij

	 nb=nchim+1		!nb. d'elements pour Burgers, +1:electron
	 nl=nchim+nb+2		!nombre de lignes de A, les wi, ri et E
 	 
	 do i=1,nchim		!masses
	  ai(i)=nucleo(i)	!masse atomique
	  mi(i)=ai(i)*amu	!masse
	 enddo
	 ai(nb)=me/amu		!pour l'electron
	 mi(nb)=me
	 zb(nb)=-1.d0
	 unpzb(nb)=0.d0
	
c	 masses reduites

	 do i=1,nb
	  do j=1,nb
	   mi1(nb*(j-1)+i)=mi(i)/(mi(i)+mi(j))
	   mi2(nb*(j-1)+i)=mi1(nb*(j-1)+i)**2
	   mij(nb*(j-1)+i)=mi(i)*mi(j)/(mi(i)+mi(j))
	   mijk(nb*(j-1)+i)=mij(nb*(j-1)+i)*n02k	!coefficient des k_ij
	   mismj(nb*(j-1)+i)=-mi(i)/mi(j)	!- pour \bar q  
	  enddo
	 enddo
	 do i=1,nb
	  do j=1,nb
	   mi3(nb*(j-1)+i)=mi1(nb*(j-1)+i)*mi1(nb*(i-1)+j)		
	  enddo
	 enddo
	
	endif	!fin des initialisations
	
c	dans une zone melangee	
	
	if(melange) then
	 do i=1,nbelem
	  v(i)=0.d0
	  d(nbelem*(i-1)+i)=d_conv
	 enddo
	 do i=1,nbelem**2
	  dv(i)=0.d0
	 enddo
	 do i=1,nbelem**3
	  dd(i)=0.d0
	 enddo
	 return
	endif	!melange
	
c	appel a Saha, calcul des taux d'ionisation
	
	call saha(xchim,t,ro,ioni,zb,nel,eta) !nel nb. elec. par volume
c	write(6,2000)(zb(i),i=1,nchim)

c	les xi

	xi(nb)=0.d0
	do i=1,nchim		!xi(nb) : nombre d'electrons par mole
	 xi(i)=xchim(i)		!X_i / A_i, transfert des abondances
	 unpzb(i)=1.d0+zb(i)	!1+Zi
	 xi(nb)=xi(nb)+zb(i)*xi(i)	 
	enddo
	
c	quantites qui dependent des x_i
c	il y a incoherence entre mu, ro et P

	mu=0.d0			!poids moleculaire moyen
	do i=1,nchim
	 mu=mu+xi(i)*unpzb(i)		!poids moleculaire moyen
	enddo
	mu=1.d0/mu
	do i=1,nchim
	 dmu(i)=-mu**2*unpzb(i)
	enddo
	dmu(nb)=0.d0			!pour les electrons
	mu_p=gr*t/ro			!R T / ro
	drox1=drox*ai(1)/ro		!derivees/ x_1 par mole a ro pres	
	dmu_px=-drox1			!a mu_p pres
	
c	la gravite	

	if(r*m .gt. 0.d0)then
 	 grav=cgrav*abs(m)/r**2			!gravite
 	else
 	 grav=0.d0
 	endif
	
c	quantites qui dependent de ro 	

	dgradradx1=dgradradx*ai(1)/gradrad	!a gradrad pres
	ro_n0_e=echarg*n0/ro
	dro_n0_ex=-ro_n0_e*drox1

	gsro=grav/ro
	dgsrox1=-gsro*drox1
	gsro_mu_grad=gsro*mu*gradrad
	do j=1,nb
	 dgsro_mu_grad(j)=gradrad*gsro*dmu(j)
	enddo
	dgsro_mu_grad(1)=dgsro_mu_grad(1)+gsro*mu*dgradradx1
		
c*******************pour tests de derivation*******************
c	dx=1.d-5
c	unpdx=1.d0+dx
c	do der=0,nb-1
c	if(der .ne. 0)then
c	 stor0=xi(der)
c	 stor=stor0*unpdx
c	 dstor=stor-stor0
c	 xi(der)=stor
c	endif
c*******************pour tests de derivation*******************

c	integrales de collision
	 
	call collision(nb,zb,mij,xi,ro,drox1,t,om11,zij,zpij,zsij,
	1	dom11,dzij,dzpij,dzsij)
		
	do i=1,nb	!coefficients pour d et f
	 do j=1,nb
	  cd(nb*(j-1)+i)=2.d0/5.d0*xi(j)*(3.d0*mi2(nb*(j-1)+i)
	1		+zpij(nb*(j-1)+i)*mi2(nb*(i-1)+j)
	2		+4.d0/5.d0*zsij(nb*(j-1)+i)*mi3(nb*(j-1)+i))
	  cf(nb*(j-1)+i)=-2.d0/5.d0*xi(j)*mi3(nb*(j-1)+i)*(3.d0
	1	+zpij(nb*(j-1)+i)-4.d0/5.d0*zsij(nb*(j-1)+i))
	  do k=1,nb
	   dcd(nb*(nb*(k-1)+j-1)+i)=2.d0/5.d0*xi(j)*
	1		(dzpij(nb*(nb*(k-1)+j-1)+i)*mi2(nb*(i-1)+j)
	2		+4.d0/5.d0*dzsij(nb*(nb*(k-1)+j-1)+i)*mi3(nb*(j-1)+i))
	   dcf(nb*(nb*(k-1)+j-1)+i)=-2.d0/5.d0*mi3(nb*(j-1)+i)*xi(j)*
	1	(dzpij(nb*(nb*(k-1)+j-1)+i)
	2	-4.d0/5.d0*dzsij(nb*(nb*(k-1)+j-1)+i))
	  enddo
	  dcd(nb*(nb*(j-1)+j-1)+i)=dcd(nb*(nb*(j-1)+j-1)+i)
	1	+cd(nb*(j-1)+i)/xi(j)
	  dcf(nb*(nb*(j-1)+j-1)+i)=dcf(nb*(nb*(j-1)+j-1)+i)
	1	+ cf(nb*(j-1)+i)/xi(j) 	  
	 enddo
	enddo
	 
c	kij, derivees

	do i=1,nb
	 do j=i,nb
	  kij(nb*(j-1)+i)=mijk(nb*(j-1)+i)*om11(nb*(j-1)+i)	  
	  if(j .ne. i)kij(nb*(i-1)+j)=kij(nb*(j-1)+i)
	  do k=1,nb
	   dkij(nb*(nb*(k-1)+j-1)+i)=mijk(nb*(j-1)+i)*dom11(nb*(nb*(k-1)+j-1)+i)
	   if(j .ne. i)dkij(nb*(nb*(k-1)+i-1)+j)=dkij(nb*(nb*(k-1)+j-1)+i)
	  enddo
	 enddo
	enddo
	 
	do i=1,nb
	 diag(i)=4.d0/25.d0*kij(nb*(i-1)+i)*zsij(nb*(i-1)+i)*xi(i)
	 do k=1,nb
	  ddiag(nb*(k-1)+i)=diag(i)*(dkij(nb*(nb*(k-1)+i-1)+i)/kij(nb*(i-1)+i)
	1	+dzsij(nb*(nb*(k-1)+i-1)+i)/zsij(nb*(i-1)+i))
	 enddo
	 ddiag(nb*(i-1)+i)=ddiag(nb*(i-1)+i)+diag(i)/xi(i)	!pour k=i 
	 do j=1,nb
	  pij(nb*(j-1)+i)=kij(nb*(j-1)+i)*xi(j)
	  do k=1,nb
	   dpij(nb*(nb*(k-1)+j-1)+i)=dkij(nb*(nb*(k-1)+j-1)+i)*xi(j)
	  enddo
	  dpij(nb*(nb*(j-1)+j-1)+i)=dpij(nb*(nb*(j-1)+j-1)+i)+kij(nb*(j-1)+i)

	  qij(nb*(j-1)+i)=pij(nb*(j-1)+i)*zij(nb*(j-1)+i)*mi1(nb*(i-1)+j)
	  do k=1,nb
	   dqij(nb*(nb*(k-1)+j-1)+i)=qij(nb*(j-1)+i)*
	1	(dpij(nb*(nb*(k-1)+j-1)+i)/pij(nb*(j-1)+i)
	2	+dzij(nb*(nb*(k-1)+j-1)+i)/zij(nb*(j-1)+i))
	  enddo
	  
	  bqij(nb*(j-1)+i)=qij(nb*(j-1)+i)*mismj(nb*(j-1)+i)
	  do k=1,nb
	   dbqij(nb*(nb*(k-1)+j-1)+i)=dqij(nb*(nb*(k-1)+j-1)+i)
	1	*mismj(nb*(j-1)+i)
	  enddo
	  	  	  
	  if(i .ne. j)then
	   diag(i)=diag(i)+kij(nb*(j-1)+i)*cd(nb*(j-1)+i)
	   do k=1,nb
	    ddiag(nb*(k-1)+i)=ddiag(nb*(k-1)+i)
	1	+dkij(nb*(nb*(k-1)+j-1)+i)*cd(nb*(j-1)+i)
	2	+kij(nb*(j-1)+i)*dcd(nb*(nb*(k-1)+j-1)+i)
	   enddo
	  endif
	  fij(nb*(j-1)+i)=kij(nb*(j-1)+i)*cf(nb*(j-1)+i)
	  do k=1,nb
	   dfij(nb*(nb*(k-1)+j-1)+i)=dkij(nb*(nb*(k-1)+j-1)+i)*cf(nb*(j-1)+i)
	1	+kij(nb*(j-1)+i)*dcf(nb*(nb*(k-1)+j-1)+i)
	  enddo
	 enddo
	enddo
	
c	formation du systeme

c	mise a 0

	do i=1,nl**2
	 a(i)=0.d0
	enddo
	do i=1,nl**2*nb
	 da(i)=0.d0
	enddo

c	la sous matrice Ae

 	do i=1,nchim
 	 a(nl*(nl-1)+i)=ro_n0_e*zb(i)		!derniere colonne : matrice Ae
  	 da(nl*(nl-1)+i)=dro_n0_ex*zb(i)	!derivee k=1, ie /X	 
  	enddo
  	  	
c	la sous matrice Ac

 	do j=1,nchim
  	 a(nl*(j-1)+nl-1)=ai(j)*xi(j)	!avant derniere ligne : matrice Ac
  	 da(nl*(nl*(j-1)+j-1)+nl-1)=ai(j)	!derivee k=j
  	enddo
 	do j=1,nb
 	 a(nl*(j-1)+nl)=zb(j)*xi(j)	!derniere ligne : matrice Ac
  	 da(nl*(nl*(j-1)+j-1)+nl)=zb(j)	!derivee k=j 	  	 
	enddo
	
c	la sous matrice Aw

	do i=1,nchim	!partie superieure
	 do j=1,nb
	  if(j .eq. i)then		!on est sur la diagonale
	   do l=1,nb	!somme pour les indices l non "diagonaux"
	    if(l .ne. j)then
	     a(nl*(j-1)+i)=a(nl*(j-1)+i)-pij(nb*(l-1)+i)
	     do k=1,nb
	      da(nl*(nl*(k-1)+j-1)+i)=da(nl*(nl*(k-1)+j-1)+i)
	1	-dpij(nb*(nb*(k-1)+l-1)+i)
	     enddo
	    endif
	   enddo	!l
	  else
	   a(nl*(j-1)+i)=pij(nb*(j-1)+i)
	   do k=1,nb
	    da(nl*(nl*(k-1)+j-1)+i)=dpij(nb*(nb*(k-1)+j-1)+i)
	   enddo
	  endif
	 enddo	!j
	enddo	!i
	do i=nb,nchim+nb	!partie inferieure
	 do j=1,nb
	  if(j .eq. i-nchim)then	!on est sur la "diagonale"
	   do l=1,nb		!somme pour les indices l non "diagonaux"
	    if(l .ne. j)then
	     a(nl*(j-1)+i)=a(nl*(j-1)+i)-qij(nb*(l-1)+i-nchim)
	     do k=1,nb
	      da(nl*(nl*(k-1)+j-1)+i)=da(nl*(nl*(k-1)+j-1)+i)
	1		-dqij(nb*(nb*(k-1)+l-1)+i-nchim)
	     enddo
	    endif
	   enddo
	  else
	   a(nl*(j-1)+i)=qij(nb*(j-1)+i-nchim)
	   do k=1,nb
	    da(nl*(nl*(k-1)+j-1)+i)=dqij(nb*(nb*(k-1)+j-1)+i-nchim)	   
	   enddo
	  endif
	 enddo	!j
	enddo	!i
	
c	la sous matrice Ar

	do i=1,nchim	!partie superieure
	 do j=1,nb
	  if(j .eq. i)then		!la "diagonale"
	   a(nl*(nb+j-1)+i)=-a(nl*(j-1)+i+nchim)	!anti symetrie
	   do k=1,nb
	    da(nl*(nl*(k-1)+nb+j-1)+i)=-da(nl*(nl*(k-1)+j-1)+i+nchim)	!idem
	   enddo
	  else
	   a(nl*(nb+j-1)+i)=bqij(nb*(j-1)+i)
	   do k=1,nb
	    da(nl*(nl*(k-1)+nb+j-1)+i)=dbqij(nb*(nb*(k-1)+j-1)+i)
	   enddo
	  endif
	 enddo	!j
	enddo	!i
	do i=nb,nchim+nb		!partie inferieure
	 do j=1,nb
	  if(j .eq. i-nchim)then	!la "diagonale"
	   a(nl*(nb+j-1)+i)=diag(j)
	   do k=1,nb
	    da(nl*(nl*(k-1)+nb+j-1)+i)=ddiag(nb*(k-1)+j)
	   enddo	  
	  else	  
	   a(nl*(nb+j-1)+i)=fij(nb*(j-1)+i-nchim)
	   do k=1,nb	   
	    da(nl*(nl*(k-1)+nb+j-1)+i)=dfij(nb*(nb*(k-1)+j-1)+i-nchim)
	   enddo	   
	  endif
	 enddo	!j
	enddo	!i
	
c	print*,'A avant inversion'
c	do i=1,nl
c	 write(6,2000)(a(nl*(j-1)+i),j=1,nl)
c	enddo
c	pause'A'
	
c	inverse A**(-1)

	call minv(a,nl,bid,lm,mm)
	if(bid .eq. 0.d0)pause'matrice non inversible'
		 
c	vecteur gamma

	do i=1,nl
	 gamma(i)=0.d0
	enddo
	do i=1,nl*nb
	 dgamma(i)=0.d0
	enddo
	 
	do i=1,nchim		!pour les nchim premieres lignes
	 gamma(i)=gsro*(ai(i)-mu)
	 do k=1,nb
	  dgamma(nl*(k-1)+i)=-gsro*dmu(k)
	 enddo
	 dgamma(i)=dgamma(i)+dgsrox1*(ai(i)-mu)	 
	enddo
	do i=nb,nl-2		!pour les nb lignes suivantes		
	 gamma(i)=gsro_mu_grad
	 do k=1,nb
	  dgamma(nl*(k-1)+i)=dgsro_mu_grad(k)
	 enddo
	enddo
	
c	write(6,2000)(gamma(i),i=1,nl)
c	write(6,2000)(ai(i),i=1,nb)
c	write(6,2000)ro_g,mu,ro_g_mu_grad
c	pause
	 
c	on extrait vt = A^(-1) gamma

	do i=1,nl
	 vt(i)=0.d0
	 do j=1,nl
	  vt(i)=vt(i)+a(nl*(j-1)+i)*gamma(j)
	 enddo
	enddo
	
c	pour V on prend les nchim premieres composantes

	do i=1,nchim
	 v(i)=vt(i)
	enddo
	
c	derivees des nchim composantes de V

	do k=1,nb
	 do i=1,nl			!d gamma / dx_k - dA/dx_k  V
	  do j=1,nl
	   dgamma(nl*(k-1)+i)=dgamma(nl*(k-1)+i)-da(nl*(nl*(k-1)+j-1)+i)*vt(j)
	  enddo
	 enddo
	 do i=1,nchim	!produit A^(-1) (d gamma / dx_k - dA/dx_k  V)
	  bid=0.d0	!il suffit des nchim premieres lignes
	  do j=1,nl
	   bid=bid+a(nl*(j-1)+i)*dgamma(nl*(k-1)+j)
	  enddo
	  if(k .le. nchim)then
	   dv(nbelem*(k-1)+i)=bid
	  else			!k=nb ie pour les electrons
	   do j=1,nchim		!d v_i /d x_j + d v_i / d x_e d x_e/ dx_j
	    dv(nbelem*(j-1)+i)=dv(nbelem*(j-1)+i)+bid*zb(j)
	   enddo
	  endif
	 enddo
	enddo
	
c	matrice B :  R T / ro ( Diag (1/x_i) - mu M )

	do i=1,nl**2
	 b(i)=0.d0
	 mt(i)=0.d0
	enddo
	do i=1,nl**2*nb
	 db(i)=0.d0
	enddo

	do i=1,nchim
	 b(nl*(i-1)+i)=1.d0/xi(i)			!Diag (1/x_i)
	 db(nl*(nl*(i-1)+i-1)+i)=-b(nl*(i-1)+i)/xi(i)	!der/ x_i
	 do j=1,nchim
	  b(nl*(j-1)+i)=mu_p*(b(nl*(j-1)+i)-mu*unpzb(j))	!- mu M
	  do k=1,nchim
	   db(nl*(nl*(k-1)+j-1)+i)=mu_p*(db(nl*(nl*(k-1)+j-1)+i)
	1	-dmu(k)*unpzb(j))
	  enddo
	  db(nl*(j-1)+i)=db(nl*(j-1)+i)+dmu_px*b(nl*(j-1)+i)	!k=1=X  
	 enddo
	enddo
	
c	print*,'B'
c	do i=1,nchim
c	 write(6,2001)(b(nl*(i-1)+j),j=1,nl)
c	enddo
c	pause

c	calcul de mt = A^(-1) B

	do i=1,nl
	 do j=1,nchim			!nchim colonnes suffisent
	  mt(nl*(j-1)+i)=0.d0
	  do l=1,nl
	   mt(nl*(j-1)+i)=mt(nl*(j-1)+i)+a(nl*(l-1)+i)*b(nl*(j-1)+l)
	  enddo
	 enddo
	enddo
	 
c	on extrait G, nchim premieres lignes et colonnes de G = mt =  A^(-1) B
c	mis dans D

	do i=1,nchim
	 do j=1,nchim
	  d(nbelem*(j-1)+i)=mt(nl*(j-1)+i)
	 enddo
	enddo
	
c	derivees des nchim X nchim premieres composantes de G 

	do k=1,nb
	 do i=1,nl			!d B / dx_k - dA/dx_k  G
	  do j=1,nchim			!nchim colonnes suffisent
	   do l=1,nl
	    db(nl*(nl*(k-1)+j-1)+i)=db(nl*(nl*(k-1)+j-1)+i)
	1		-da(nl*(nl*(k-1)+l-1)+i)*mt(nl*(j-1)+l)
	   enddo
	  enddo
	 enddo
	 do i=1,nchim	!produit A^(-1) (d B / dx_k - dA/dx_k  G)
	  do j=1,nchim	!il suffit des nchim premieres lignes/colonnes
	   vt(j)=0.d0	  
	   do l=1,nl
	    vt(j)=vt(j)+a(nl*(l-1)+i)*db(nl*(nl*(k-1)+j-1)+l)
	   enddo
	  enddo
	  if(k .le. nchim)then
	   do j=1,nchim
	    dd(nbelem*(nbelem*(k-1)+j-1)+i)=vt(j)
	   enddo
	  else			!k=nb ie pour les electrons
	   do j=1,nchim
	    do l=1,nchim	  
	     dd(nbelem*(nbelem*(l-1)+j-1)+i)=dd(nbelem*(nbelem*(l-1)+j-1)+i)  
	1	+vt(j)*zb(l)
	    enddo
	   enddo		  
	  endif
	 enddo
	enddo

c	-d et *xi(i) : notation de CESAM, on ajoute la diffusion turbulente
c	et la diffusion de melange
c	pour d, dd nbelem X nbelem  d, dd

	if(t .lt. ts)then	!diffusion de melange
	 d_mel=ds+(t-ts)*ddd
	else
	 d_mel=0.d0
	endif

	do i=1,nchim
	 do j=1,nchim
	  d(nbelem*(j-1)+i)=-d(nbelem*(j-1)+i)*xi(i) !pour CESAM
	  do k=1,nchim
	   dd(nbelem*(nbelem*(k-1)+j-1)+i)=-dd(nbelem*(nbelem*(k-1)+j-1)+i)
	1		*xi(i)
	  enddo
	  dd(nbelem*(nbelem*(i-1)+j-1)+i)=dd(nbelem*(nbelem*(i-1)+j-1)+i)
	1	+d(nbelem*(j-1)+i)/xi(i)		!pour k=i	  
	 enddo
	 d(nbelem*(i-1)+i)=d(nbelem*(i-1)+i)+d_turb+d_mel
	enddo			!diffusion turbulente + melange 

c****************************pour test de derivation********************	
c	if(der .eq. 0)then
c	 do i=1,nchim*nbelem
c	  a0(i)=d(i)
c	 enddo
c	 do i=1,nchim**2*nbelem
c	  da0(i)=dd(i)
c	 enddo
c	else
c	 do i=1,nchim
c	  print*,'dd/dx_k k,i=',der,i
c	  write(6,2000)((d(nbelem*(j-1)+i)-a0(nbelem*(j-1)+i))/dstor,
c	1	da0(nbelem*(nbelem*(der-1)+j-1)+i),j=1,nchim)
c	 enddo
c	 xi(der)=stor0
c	endif
c	enddo
c	pause
c****************************pour test de derivation********************	
	
c	si on diffuse le moment angulaire nbelem=nchim+1 
	
	if(iw .gt. 1)then
	 v(iw)=0.d0
	 do j=1,nbelem
	  dv(nbelem*(j-1)+iw)=0.d0
	  d(nbelem*(j-1)+iw)=0.d0
	  do k=1,nbelem
	   dd(nbelem*(nbelem*(k-1)+j-1)+iw)=0.d0
	  enddo
	 enddo	  
	 d(nbelem*(iw-1)+iw)=w_diff	!moment angulaire
	endif

	return
	
	end
