
c*************************************************************************

	subroutine diff_mipr(melange,p,t,r,l,m,ro,drox,kap,dkapx,
	1	gradrad,dgradradx,gradad,dgradadx,r_zc,lim,mstar,
	2	xi,d,dd,v,dv)

c	calcul des coefficients de diffusion microscopique
c	d'apres Michaux-Proffit, Vienne, Inside the stars IAU 137, p.250
c	on tient compte de Z: Vy=-xVx/(1-x-z)

c	si iz .gt. 1, Z est considere comme un element test de charge 8
c	et de masse atomique 16
c	on a aussi le cas de l'element supplementaire de charge 18 et de masse 27 
c	identique a diff_z16 avec derivees analytiques

c	les coefficients de la matice de diffusion ne sont non nuls que sur la
c	diagonale et la premiere colonne.

c	Dimensions dans le programme appelant
c	d(nbelem,nbelem), dd(nbelem,nbelem,nbelem), v(nbelem), dv(nbelem,nbelem)

c	convention de notation :
c	equation de diffusion dXi/dt=dFi/dm + nuclear, i=1,nbelem
c	Fi=4 pi r**2 ro (4 pi r**2 ro D.dX/dm - Vi Xi)

c	d=D=(di1, di2,... din) avec Dij coefficient de d Xj / dm
c	dans le produit scalaire D.dX/dm=sum d_ij d Xj / dm

c	pour ligne d'indice i
c	v(i) coefficient de x_i,
c	dv(i,k)=dv(nbelem*(k-1)+i)=derivee v_i / x_k
c	seule la premiere colonne de dv
c	est non nulle (pas de derivees / Xi, i .ne. 1)
c	d(i,j)=dv(nbelem*(j-1)+i)=coefficient d_ij de d x_j / dm
c	dd(i,j,k)=dd(nbelem*(nbelem*(k-1)+j-1)+i)= derivee de d_ij / x_k

c	les derivees / Z et W ne sont pas prise en compte dans cette routine

c	Auteur: P.Morel, OCA
c	Conseils: J. Matias, DASGAL, G. Alecian, Evry
c	adaptation a CESAM3: P. Morel, Departement J.D. Cassini, O.C.A.,
c	Observatoire de Nice

c	CESAM4

c	routine derivee de diff_z16, puis de diff_der
c	19 08 96 : remplacement ln lambda par Cij (c.a.d. cs)
c	13 03 97 : remplacement de iz par iw pour la diffusion
c		turbulente au centre
c	11 12 98 : coherence avec le cas general, modification de v, dv, d, dd
c	18 12 98 : abondances par mole en entree
c	05 01 99 : yVy=-xVx au lieu de (1-x-z)Vy=xVx
c	07 01 00 : adjonction de d_mel

c entrees
c	melange=.true.: on est dans une ZC
c	p, t, r, l, m, ro: donnees au point de calcul
c	xi: composition chimique, par mole
c	kap: opacite 
c	gradad, gradrad: gradients
c	terminaisons x : derivees/ X1 (ie H) par gramme
c	mstar: masse avec perte de masse
c	r_zc, lim : rayons des limites ZR/ZC, nombre de limites

c sorties
c	d0, dd : coefficients d_ij de d x_j / d m et derivees / x_k
c	v0, dv : coefficients v_i de x_i et derivees / x_k

	implicit none

	include 'cesam.parametres'
	include 'ctephy.common'
	include 'evol_chim.common'	
	include 'modele.common'

	integer i,j,k,lim

	real*8	p,t,xchim(pnelem),m,l,r,ro,kap,cs(pnelem,2),as(pnelem,2),
	1	num,den,q,cte1,mstar,thetae,b,c,z,r_zc(1),xi(1),
	2	lnlamb(pnelem,2),grav,gradrad,gradad,d_turb,d_mel,
	3	sa(pnelem,2),vth,unmz,drox,dkapx,d_conv,xsy,dxsyx,dxsyy,
	4	dgradradx,dgradadx,d(1),dd(1),v(1),dv(1),w_diff,
	5	csr(pnelem,2),csx(pnelem,2),lnlambr(pnelem,2),ti,ts,di,ds,ddd,
	6	lnlambx(pnelem,2),cx,qx,gravx,vthx,c1,c1x,c2,c2x,numx,denx
	
	logical melange
	
	logical init
	data init/.true./
	
	save

2000	format(1x,1p8d10.3)
2001	format(1x,1p10d8.1)

	if(init)then
	 init=.false.
	 write(6,*)
	 write(2,*)
	 d_turb=10.d0
	 d_turb=0.d0	 
	 d_conv=1.d13
	 write(6,10)d_conv,d_turb
	 write(2,10)d_conv,d_turb
10	 format(1x,'Diff. micro. de Proffit et Michaux, dans ZR, der. anal. ',/,
	1	1x,'H1 et He4 sont diffuses, les autres sont element trace.',/,
	2	1x,'Dans ZC, Dconv=',1pd10.3,' Coeff. diff. turb.=',1pd10.3)
	 write(2,*)'pour He4: X Vx = - Y Vy ==> Vy = - X / (1 - X - Z) Vx'
	 write(6,*)'pour He4: X Vx = - Y Vy ==> Vy = - X / (1 - X - Z) Vx'	  

c	 limites et coefficients du melange supplementaire

	 ti=0.d0
	 ts=1.d6
	 di=1.d3
	 di=0.d0	 
	 ds=0.d0
	 ddd=(di-ds)/(ti-ts)	
	 write(2,13)di,ds,ti,ts
	 write(6,13)di,ds,ti,ts
13	 format(1x,'avec melange supplementaire pour T < Ts',/,
	1	1x,'Di=',1pd10.3,' Ds=',1pd10.3,' Ti=',1pd10.3,' Ts=',1pd10.3)

	 if(iw .gt. 0)then
	  w_diff=d_conv
c	  print*,'entrer w_diff'
c	  read*,w_diff
	  write(6,11)w_diff
	  write(2,11)w_diff	 
11	  format(t2,' avec diffusion constante du moment angulaire w_diff =',
	1	1pd10.3,//)
	 endif	 
	 
c	 Case mixture of H et He & thetae=1.   :   zi=1  zj=2

	 thetae=1.d0
	 b=15.d0/16.d0*sqrt(2.d0*amu/5.d0/pi)*sqrt(kbol)**5/echarg**4
	 
	 if(iz .gt. 1)then
	  write(6,12)zi(iz),nucleo(iz)
	  write(2,12)zi(iz),nucleo(iz)	  	 
12	  format(t2,'Diffusion de Z, element trace de charge=',1pd10.3,
	1	' et de masse atomique=',1pd10.3,//)
	 endif
	 write(6,*)
	 write(2,*)
	 
c	 masses reduites

	 do i=1,nbelem
	  if(i .ne. iw)then
	   as(i,1)=nucleo(i)*nucleo(1)/(nucleo(i)+nucleo(1))
	   sa(i,1)=sqrt(as(i,1))
	   as(i,2)=nucleo(i)*nucleo(ihe4)/(nucleo(i)+nucleo(ihe4))
	   sa(i,2)=sqrt(as(i,2))
	  endif
	 enddo
	 
c	 convention de signe:
c	 equation de diffusion dX/dt=dF/dm + nuclear
c	 F=4 pi r**2 ro (4 pi r**2 ro D.dX/dm - v X)
c	 le terme en gravite vient de dlnP/dr, il entre dans v (eq.17)
c	 et vth (eq.19).
c	 - Pour v, il y a deja 2 signes - (eq 13 et 17)
c	 mettre - v revient bien a mettre grav < 0
c	 - Pour vth, comme grav > 0, et qu'il y a - dans v (eq.17)
c	 il faut alors mettre - vth pour retablir le signe
c	 en - v dans eq_diffus 
	 
	 cte1=g*msol/rsol**2
	 
	endif		!initialisation

c	mises a 0
	
	do i=1,nbelem
	 v(i)=0.d0
	enddo
	do i=1,nbelem**2	
	 dv(i)=0.d0
	 d(i)=0.d0
	enddo
	do i=1,nbelem**3	
	 dd(i)=0.d0
	enddo

c	zones melangees

	if(melange) then
	 do i=1,nbelem
	  d(nbelem*(i-1)+i)=d_conv
	 enddo
	else
	
	 if(iw .gt. 1)d(nbelem*(iw-1)+iw)=w_diff	!moment angulaire	
	
c	 les equations et les derivees analytiques en xi

c	 les formules de Michaud-Proffit utilisent les abondances par gramme
c	 on transforme les xi par mole en xchim par gramme
c	 a l'issue du calcul les derivees sont prises / xi par mole

	 do i=1,nbelem
	  xchim(i)=xi(i)*nucleo(i)
	 enddo
	 xsy=xchim(1)/xchim(ihe4)
	 dxsyx=xsy/xchim(1)
	 dxsyy=-xsy/xchim(ihe4)
	
c	 le Z
	
	 if(iz .gt. 1)then
	  z=xchim(iz)
	 else
	  z=z0
	 endif
	 unmz=1.-z

c	 logarithme de Coulomb et integrale cij pour H=X et He4=Y

	 do i=1,nbelem
	  if(i .ne. iw)then	    
	   call coulomb_d(zi(i),zi(1),thetae,ro,xchim(1),
	1	t,lnlamb(i,1),lnlambr(i,1),lnlambx(i,1),
	2	cs(i,1),csr(i,1),csx(i,1))
	   lnlambx(i,1)=lnlambx(i,1)+lnlambr(i,1)*drox	!derivee/X
	   csx(i,1)=csx(i,1)+csr(i,1)*drox
	   
	   call coulomb_d(zi(i),zi(ihe4),thetae,ro,xchim(1),
	1	t,lnlamb(i,2),lnlambr(i,2),lnlambx(i,2),
	2	cs(i,2),csr(i,2),csx(i,2))
	   lnlambx(i,2)=lnlambx(i,2)+lnlambr(i,2)*drox	!derivee/X
	   csx(i,2)=csx(i,2)+csr(i,2)*drox
	  endif
	 enddo
	 
c	 lnlambdaij est remplace par cij, remarque sous la formule 17	 
	 
	 c = b*sqrt(t)**5/ro/cs(1,2)/(0.7d0+0.3d0*xchim(1))
	 cx=-c*(0.3d0/(0.7d0+0.3d0*xchim(1))+
	1	drox/ro+csx(1,2)/cs(1,2))	!derivee /X
	 q= 2.d0/sqrt(5.d0)/ro*b*sqrt(t)**5
	 qx=-q*drox/ro				!derivee /X
	 if(r .gt. 0.d0)then
	  grav=cte1*ro*m/p/r**2
	 else
	  grav=0.d0
	 endif	   
	 gravx=grav*drox/ro	!derivee/X
	 if(grav .ne. 0.d0)then
	  vth=0.54d0*b/ro*(4.75d0*xchim(1)+2.25d0)*sqrt(t)**5/(5.d0+cs(1,2))*
	1	gradrad*grav		!formule 19
	  vthx=0.54d0*b/ro*4.75d0*sqrt(t)**5/(5.d0+cs(1,2))*gradrad*grav
	1	+vth*(-drox/ro+dgradradx/gradrad
	2	+gravx/grav-csx(1,2)/(5.d0+cs(1,2)))
	 else
	  vth=0.d0
	  vthx=0.d0
	 endif
c	 write(6,2000)c,q,grav,vth,m,r

	 do i=1,nbelem	
	  if(i .eq. 1)then	!pour l'hydrogene
	   if(c*grav .ne. 0.d0)then	   
	    v(1)=c*(1.25d0+1.125d0*gradrad)*grav*(1.d0-xchim(1))
	    dv(1)=v(1)*(cx/c+1.125d0*dgradradx/(1.25d0+1.125d0*gradrad)
	1	+gravx/grav-1.d0/(1.d0-xchim(1)))  !derivee /X
	   else
	    v(1)=0.d0
	    dv(1)=0.d0
	   endif
	   if(c .ne. 0.d0)then	    
	    d(1)= c*(3.d0+xchim(1))/(1.d0+xchim(1))/
	1		(3.d0+5.d0*xchim(1))
	    dd(1)=d(1)*(cx/c+1.d0/(3.d0+xchim(1))-1.d0/(1.d0+xchim(1))
	1	-5.d0/(3.d0+5.d0*xchim(1)))
	   else
	    d(1)=0.d0
	    dd(1)=0.d0
	   endif
	    	     	   
	  elseif(i .eq. ihe4)then	!pour He4: XVx=-Y Vy ==> Vy=-X/(1-X-Z)Vx
	   d(nbelem*(ihe4-1)+ihe4)=d(1)*xsy	!coeff de d Y
	   dd(nbelem*(ihe4-1)+ihe4)=dd(1)*xsy+d(1)*dxsyx	!der/X1
	   dd(nbelem*(nbelem*(ihe4-1)+ihe4-1)+ihe4)=d(1)*dxsyy	!der/Y
	   v(ihe4)=-v(1)*xsy
	   dv(ihe4)=-dv(1)*xsy-v(1)*dxsyx 			!der/X1
	   dv(nbelem*(ihe4-1)+ihe4)=-v(1)*dxsyy			!der/Y
	
	  elseif(i .ne. iw)then	!pour les autres elements: elements test
	   num=sa(i,1)*cs(i,1)-sa(i,2)*cs(i,2)
	   numx=sa(i,1)*csx(i,1)-sa(i,2)*csx(i,2)
	   den=xchim(1)*num+sa(i,2)*cs(i,2)
	   denx=num+xchim(1)*numx+sa(i,2)*csx(i,2)
	   d(nbelem*(i-1)+i)=q/zi(i)**2/den
	   dd(nbelem*(nbelem*(1-1)+i-1)+i)=		!der/X1
	1	d(nbelem*(i-1)+i)*(qx/q-denx/den)	!derivee d(i,i)/X1
	     
	   c1=1.d0/(1.d0+xchim(1))   -10.d0/(5.d0*xchim(1)+3.d0)
	   c1x=-1./(1.d0+xchim(1))**2+50.d0/(5.d0*xchim(1)+3.d0)**2
	     
	   c2=d(nbelem*(i-1)+i)*c1+xchim(1)*(num/den-0.23d0)*d(1)
	   c2x=dd(nbelem*(nbelem*(1-1)+i-1)+i)*c1+d(nbelem*(i-1)+i)*c1x+
	1	(num/den-0.23d0)*d(1)+xchim(1)*(numx-num*denx/den)/den*d(1)
	2	+xchim(1)*(num/den-0.23d0)*dd(1)
	     
	   d(nbelem*(1-1)+i)=c2*ah/nucleo(i)*xchim(i)
	   dd(nbelem*(nbelem*(1-1)+1-1)+i)=c2x*ah/nucleo(i)*xchim(i)
	   dd(nbelem*(nbelem*(i-1)+1-1)+i)=c2*ah/nucleo(i)	     
	
	   c1=1.d0+zi(i)-nucleo(i)*(5.d0*xchim(1)+3.d0)/4.d0
	   c1x=-nucleo(i)*5.d0/4.d0
	   c2=xchim(1)*(num/den-0.23d0)*v(1)*ah
	   c2x=c2*(1.d0/xchim(1)+dv(1)/v(1)
	1	+(numx-num*denx/den)/den/(num/den-0.23d0))
	   v(i)=d(nbelem*(i-1)+i)*c1*grav+c2-vth
	   dv(i)=dd(nbelem*(nbelem*(1-1)+i-1)+i)*c1*grav
	1	+d(nbelem*(i-1)+i)*c1x*grav
	2	+d(nbelem*(i-1)+i)*c1*gravx+c2x-vthx   
	  endif
	 enddo

c	 diffusion turbulente et de melange

	 if(t .lt. ts)then	!diffusion de melange
	  d_mel=ds+(t-ts)*ddd
	 else
	  d_mel=0.d0
	 endif
	 
	 do i=1,nbelem
	  if(i .eq. iw)then		!pour avoir 0 au centre	   
	   d(nbelem*(i-1)+i)=d(nbelem*(i-1)+i)+d_turb*m
	  else
	   d(nbelem*(i-1)+i)=d(nbelem*(i-1)+i)+d_turb+d_mel
	  endif
	 enddo
	 
c	 derivees par rapport aux abondances par gramme

	 do i=1,nbelem
	  do k=1,nbelem
	   dv(nbelem*(k-1)+i)=dv(nbelem*(k-1)+i)*nucleo(k)
	   do j=1,nbelem
	    dd(nbelem*(nbelem*(k-1)+j-1)+i)=dd(nbelem*(nbelem*(k-1)+j-1)+i)
	1	*nucleo(k)
	   enddo
	  enddo
	 enddo

	endif		!melange	
	
	return
	
	end
