	
c***********************************************************************

	 subroutine eq_diffus(y,cx,aes,aed,bes,bed,fait,mstar,
	1	bp,q,qt,knot,n,bp_t,q_t,qt_t,knot_t,n_t,
	2	r2,m23,r2_t,m23_t,mc_t,mct_t,nc_t,knotc_t,chim_t,dt,
	3	a_mix,nmix,mix,a_mixt,knot_mix,r_zc,lim,
	4	old_m23,new_m23,nm,new_m23t,knotm,	
	5	etat,opa,conv,nuc,coeff_diff)

c	formation des equations pour diffus.f
c	methode de Galerkin ou de Petrov-Galerkin
c	les derivees/xchim sont calculees dans la routine de calcul
c	du coefficient de diffusion
c	on n'a pas a tenir compte de la pression turbulente puisqu'elle
c	est nulle dans les zones radiatives ou il y a diffusion microscopique
c	et qu'il y a melange dans les zones convectives

c	15 07 97 : adjonction de r**2 pour le moment angulaire
c	25 08 97 : mise en place des variables euleriennes
c	11 12 98 : generalisation des coefficients de diffusion
c	20 10 99 : ajout des variables de structure au temps t
c	19 11 99 : suppression de nh1, nhe1, nhe2, lamb

c	en_masse = .true. variables lagrangiennes m23=m**23, r2=r**2
c	en_masse = .false. variables euleriennes m23=m, r2=r

c	avec pression turbulente 7 inconnues 
c	sans pression turbulente 6 inconnues, Ptot=Pgaz

c	Auteur: P. Morel, Departement J.D. Cassini, O.C.A., Observatoire de Nice
c	CESAM4

c entrees
c	y: solution
c	cx: point de calcul en m**2/3
c	dt: pas temporel
c	bp,q,qt,knot,n,r2,m23: modele au temps t+dt
c	bp_t,q_t,qt_t,knot_t,n_t,r2_t,m23_t: modele au temps t
c	mc_t,mct_t,nc_t,knotc_t,chim_t(1): int. comp. chim. temps t
c	a_mix,nmix,mix,a_mixt,knot_mix: interpolation de la nature de la zone
c	fait=1: point interieur, fait=2: point exterieur (perte de Mw)
c	mstar: masse temps t+dt, avec perte de masse
c	old_m23,new_m23,nm,new_m23t,knotm : interpolation m(t+dt)-->m(t)
c	r_zc, lim : rayons des limites ZR/ZC, nombre de limites

c sorties	
c	indice s/d: coefficients de la spline/derivee
c	aes, aed: coefficients de la spline/derivee
c	bes, bed: seconds membres

c external
c	nuc: reactions thermonucleaires
c	coeff_diff: calcul du coefficient de diffusion
c	etat: equation d'etat
c	opa: opacite
c	conv: routine de convection

c_______________________________________________________________________

c	pour chaque spline Nj
c	le systeme est constitue de nbelem produits scalaires

c	les equations:

c	< 3/2 nu^1/2 [ Xi(t+1)-Xi(t) - Psi dt ] , Fi_j > +
c	+ < 32/(3 nu^1/2) pi^2 r^4 ro^2 dt
c	[Di1 dX1/dnu+...+Din dXn/dnu],dFi_j/dnu >
c	- < 4 pi r^2 ro dt Vi Xi , dFi_j/dnu >=0

c	nu: m**2/3 = cx

c_______________________________________________________________________

	implicit none

	include 'cesam.parametres'
	include 'ctephy.common'
	include 'evol_chim.common'
	include 'convection.common'		
	include 'modele.common'
	
	integer	i,knotc_t,lx,n,knot,nc_t,nmix,knot_mix,fait,j,k,kd,nm,
	1	knotm,lim,n_t,knot_t

	real*8	aed(1),aes(1),bed(1),bes(1),y(1),d(pnelem**2),cx,mn,
	1	dcomp(pnelem),p,t,r,l,mstar,old_m23(1),new_m23(1),
	2	jac(pnelem**2),epsilon(5),a_mix(1),mix(1),a_mixt(1),
	4	dt,chim_t(1),mc_t(1),mct_t(1),new_m23t(1),r_zc(1),
	5	comp_t(pnelem),f(pne),dfdx(pne),dcomp_t(pnelem),
	6	r2(1),m23(1),bp(1),q(1),qt(1),ro,drop,drot,drox,
	7	xchim(pnelem),v(pnelem),kap,dkapt,dkapro,dkapx,
	8	dd(pnelem**3),dv(pnelem**2),nuc_m(pnelem),u,dup,dut,dux
	
	real*8	deltax,dcpx,dgradadx,dgradradx,stor0,stor,unpdx,dstor,bid,dx,
	1	bess(pnelem),beds(pnelem),deltap,deltat,dcpp,dcpt,
	2	dgradadp,dgradadt,alfa,beta,gamma1,
	3	r2_t(1),m23_t(1),bp_t(1),q_t(1),qt_t(1),f_t(pne),dfdx_t(pne),
	4	depst,depsro,depsx(pnelem),hhe,be7e,
	5	b8e,n13e,o15e,f17e,delta,cp,gradad,gradrad,dcte2x,dcte3x,
	6	cte10,cte1,cte20,cte2,cte30,cte3,cte10r,cte1r,cte20r,cte2r,
	7	cte30r,cte3r,cte40r,cte4r,cte8,cte9,dcte2rx,dcte3rx
	
	external coeff_diff,etat,opa,conv,nuc

	logical melange,init
	data init/.true./

	save
c________________________________________________________________________

c	le nombre d'inconnues est nbelem

c	y(var,der):
c	var=1... nbelem pour les Xi
c	var=1+nbelem, 2+nbelem... 2*nbelem pour les Fi

c	der=1 pour la fonction
c	der=2 pour derivee premiere, etc..

c  	y(nbelem*(der-1)+var) := derivee d'ordre der(1 pour 0), (2 pour ')
c	de la variable var pour X: 1, .. nbelem, pour F: 1+nbelem, .. 2*nbelem

2000	format(1x,1p8d10.3)
2001	format(1x,1p10d8.1)

c	write(6,*)fait,cx
c	write(6,20000)dt
c	write(6,2001)(y(nbelem*(1-1)+i),i=1,nbelem)
c	pause'entree eq_diffus'

	if(init)then
	 init=.false.
	 dx=1.d-5
	 unpdx=1.d0+dx
	 cte10=1.5
	 cte10r=2.d0/3.d0*rsol**2	 
	 cte20=2.d0/3.d0*(4.d0*pi*rsol**2/msol)**2*secon6
	 cte20r=(8.d0/3.d0*pi*rsol**3/msol)**2*secon6	 
	 cte30=-4.d0*pi*rsol**2/msol*secon6	 
	 cte30r=16.d0/3.d0*pi*rsol**3/msol*secon6	 	 
	 cte40r=-msol*1.d6*4.*pi*100.*1.5!perte de MW a 10 Rext, - car -F*fi
	 cte9=3./16./pi/aradia/clight/g
	 cte8=cte9*lsol/msol		!calcul direct de gradrad
	endif
	 
	goto(100,200),fait

c	M, P, T, R, L au point mu=m**2/3  (m**2/3=cx)

100	mn=sqrt(cx)**3
c	print*,'mu',cx

	if(en_masse)then
	 bid=cx
	else
	 bid=mn
	endif
	call inter('m23',bp,q,qt,n,knot,min(bid,m23(n)),f,dfdx,r2,m23)
c	call inter('m23',bp_t,q_t,qt_t,n_t,knot_t,min(bid,m23_t(n_t)),
c	1	f_t,dfdx_t,r2_t,m23_t)	 

c	on prend les moyennes geometriques des pressions et temperatures
c	entre les temps t et t+dt (modif du 20 10 99 P. Morel)

	if(pturb)then		!avec pression turbulente 7 inconnues
c	 p=exp((f(7)+f_t(7))/2.d0)
	 p=exp(f(7))	 
	else			!sans pression turbulente 6 inconnues
c	 p=exp((f(1)+f_t(1))/2.d0)
	 p=exp(f(1))	 
	endif
c	t=exp((f(2)+f_t(2))/2.d0)
	t=exp(f(2))	
	if(en_masse)then
	 r=sqrt(abs(f(3)))
	 l=sqrt(abs(f(4)))**3	 
	else
	 r=abs(f(3))
	 l=f(4)	
	endif
c	print*,'r',r	
c	write(6,2000)p,t,r,l,mn

c	est-on dans une zone melangee ?

	u=min(cx,a_mix(nmix))	!u: vT
	call sbsp1dn(1,mix,a_mix,a_mixt,nmix,2,knot_mix,.true.,u,lx,f,dfdx)
	melange=f(1) .lt. 0.d0
c	print*,cx,f(1),melange

c	la composition chimique au temps t

	if(en_masse)then
	 u=min(cx,new_m23(nm))	!u:vT
	else
	 u=min(mn,new_m23(nm))	!u:vT
	endif	
	call sbsp1dn(1,old_m23,new_m23,new_m23t,nm,2,knotm,.true.,
	1	u,lx,f(1),dfdx)	!masse au temps t f(1)
	if(.not.en_masse)f(1)=f(1)**(2./3.)
	f(1)=min(f(1),mc_t(nc_t))		
	call sbsp1dn(nbelem,chim_t,mc_t,mct_t,nc_t,m_ch,
	1	knotc_t,.true.,f(1),lx,comp_t,dcomp_t)
c	write(6,2000)(comp_t(i),i=1,nbelem),(y(i),i=1,nbelem)
	
c	la composition chimique par gramme

	do i=1,nbelem
	 xchim(i)=y(i)
	enddo
	call chim_gram(xchim,dcomp,nuc_m)	!dcomp VT

c	la densite, le gradient adiabatique et leurs derivees/X
c	si les coefficient de diffusion dependent de gradad, mettre
c	call etat(p,t,xchim,.true.,

	call etat(p,t,xchim,.true.,
	1	ro,drop,drot,drox,u,dup,dut,dux,
	2	delta,deltap,deltat,deltax,cp,dcpp,dcpt,dcpx,
	3	gradad,dgradadp,dgradadt,dgradadx,alfa,beta,gamma1)
	 		
c	write(6,2000)ro,gradad
	
c	l'opacite et le gradient radiatif

	call opa(xchim,t,ro,kap,dkapt,dkapro,dkapx)
	gradrad=cte8*l*p*kap/mn/t**4
	dgradradx=gradrad/kap*(dkapx+dkapro*drox)
	 
c	write(6,2000)ro,gradad,gradrad
	 
c	le coefficient de diffusion

	call coeff_diff(melange,p,t,r,l,mn,ro,drox,kap,dkapx,
	1	gradrad,dgradradx,gradad,dgradadx,r_zc,lim,mstar,
	2	y,d,dd,v,dv)
	
c	les reactions thermonucleaires Xdot

	call nuc(t,ro,y,dcomp,jac,.true.,2,
	1	epsilon,depst,depsro,depsx,hhe,be7e,b8e,n13e,o15e,f17e)
	
c	if(iw .gt. 1)then
c	 dcomp(iw)=-secon6*rsol*r*y(iw)*1.d-7
c	 jac(nbelem*(iw-1)+iw)=dcomp(iw)/y(iw)
c	endif

c	les equations:

c	< 3/2 nu^1/2 [ Xi(t+1)-Xi(t) - Psi dt ] , Fi_j > +
c	+ < 32/(3 nu^1/2) pi^2 r^4 ro^2 dt
c	[Di1 dX1/dnu+...+Din dXn/dnu],dFi_j/dnu >
c	- < 4 pi r^2 ro dt Vi Xi , dFi_j/dnu >=0

	cte1=cte10*sqrt(cx)
	cte2=cte20*r**4*ro**2/sqrt(cx)*dt
	cte3=cte30*r**2*ro*dt
	drox=drox*ah/ro		!d ro / d x_1 a ro pres
	dcte2x=cte2*2.d0*drox	
	dcte3x=cte3*drox	
	if(iw .gt. 1)then	!pour le moment angulaire
	 cte1r=cte1*cte10r*r**2	
	 cte2r=cte20r*(r**3*ro)**2*dt/sqrt(cx)
	 dcte2rx=cte2r*2.d0*drox	
	 cte3r=cte30*r**3*ro*dt
	 dcte3rx=cte3r*drox	 
	 cte4r=cte40r*mdot*dt	!perte de MW a 10 Rext
	endif

c	pour bes(spline):	3/2 nu^1/2 [ Xi(t+1)-Xi(t) - Psi dt ]
c	pour bed(derivee de la spline): 32/(3 nu^1/2) pi^2 r^4 ro^2 dt *
c				*  (Dt+Di+Dm) dXi/dnu  -4 pi r^2 ro dt Vi Xi

c	bes : coefficient de la spline du produit scalaire
c	bed : coefficient de la derivee de la spline du produit scalaire
	 
	do i=1,nbelem
	 if(i .ne. iw)then
	  bes(i)=cte1*(y(i)-comp_t(i)-dt*dcomp(i))
	  bed(i)=cte3*v(i)*y(i)	!coeff. de Xi: vitesse
	  do j=1,nbelem		!produit scalaire D . dx/dm	
	   bed(i)=bed(i)+cte2*d(nbelem*(j-1)+i)*y(nbelem+j)
	  enddo	!j
	
c	  derivees

c  	  y(nbelem*(der-1)+var) := derivee d'ordre der(1 pour 0), (2 pour ')

c	  ae(nbelem,nbelem,2)=
c	  ae(equation,variable,derivee+1)=ae(nbelem*(nbelem*(k-1)+j-1)+i)
c	  coefficient de la (k-1)-ieme derivee de la j-ieme variable de la i-ieme
c	  equation

c	  aes : derivees/ Xi, Xi' des coeff. de la spline du produit scalaire
c	  aed : derivees/ Xi, Xi' des coeff. de la der. du produit scalaire

c	  derivee dans bed de cte3 v_i x_i + SUM d_ij x_j' / x_k

	  aed(i)=dcte3x*v(i)*y(i)				!der / x_1
	  aed(nbelem*(i-1)+i)=aed(nbelem*(i-1)+i)+cte3*v(i)	!der / x_i
	  do k=1,nbelem
	   aed(nbelem*(k-1)+i)=aed(nbelem*(k-1)+i)+
	1	cte3*dv(nbelem*(k-1)+i)*y(i)			!der / x_i	  
	  enddo
	  do j=1,nbelem
	   aed(nbelem*(nbelem+j-1)+i)=cte2*d(nbelem*(j-1)+i)	!der / x_i'
	   aed(i)=aed(i)+dcte2x*d(nbelem*(j-1)+i)*y(nbelem+j)	!der / x_1
	   do k=1,nbelem	   
	    aed(nbelem*(k-1)+i)=aed(nbelem*(k-1)+i)
	1	+cte2*dd(nbelem*(nbelem*(k-1)+j-1)+i)*y(nbelem+j)
	   enddo
	  enddo
	  
c	  derivee dans bes de cte1*(y(i)-comp_t(i)-dt*dcomp(i)) / x_i

	  aes(nbelem*(i-1)+i)=cte1		!der / x_i
	  do j=1,nbelem	
	   aes(nbelem*(j-1)+i)=aes(nbelem*(j-1)+i)
	1	-cte1*dt*jac(nbelem*(j-1)+i)	!der / x_j (jacobien)
	  enddo

	 else	!pour le moment angulaire
	  bes(i)=cte1r*(y(i)-comp_t(i)-dt*dcomp(i))
	  bed(i)=cte3r*v(i)*y(i)	!coeff. de Xi: vitesse
	  do j=1,nbelem		!produit scalaire D . dx/dm	
	   bed(i)=bed(i)+cte2r*d(nbelem*(j-1)+i)*y(nbelem+j)
	  enddo	!j
	
c	  derivee dans bed
c	  cte3r*v(i)*y(i) + SUM cte2r*d(nbelem*(j-1)+i)*y(nbelem+j)/ x_k

	  aed(i)=dcte3rx*v(i)*y(i)				!der / x_1
	  aed(nbelem*(i-1)+i)=aed(nbelem*(i-1)+i)+cte3r*v(i)	!der / x_i
	  do k=1,nbelem
	   aed(nbelem*(k-1)+i)=aed(nbelem*(k-1)+i)+
	1	cte3r*dv(nbelem*(k-1)+i)*y(i)			!der / x_i  
	  enddo
	  do j=1,nbelem
	   aed(nbelem*(nbelem+j-1)+i)=cte2r*d(nbelem*(j-1)+i)	!der / x_i'
	   aed(i)=aed(i)+dcte2rx*d(nbelem*(j-1)+i)*y(nbelem+j)	!der / x_1
	   do k=1,nbelem	   
	    aed(nbelem*(k-1)+i)=aed(nbelem*(k-1)+i)
	1	+cte2r*dd(nbelem*(nbelem*(k-1)+j-1)+i)*y(nbelem+j)
	   enddo
	  enddo
	  
c	  derivee dans bes de cte1r*(y(i)-comp_t(i)-dt*dcomp(i)) / x_i

	  aes(nbelem*(i-1)+i)=cte1r		!der / x_i
	  do j=1,nbelem	
	   aes(nbelem*(j-1)+i)=aes(nbelem*(j-1)+i)
	1	-cte1r*dt*jac(nbelem*(j-1)+i)	!der / x_j (jacobien)
	  enddo

	 endif	!iw	
	enddo	!i
	
c	write(6,2001)(bes(i),i=1,nbelem)
c	write(6,2001)(bed(i),i=1,nbelem)	
c	do kd=1,2
c	 print*,kd
c	 do k=1,nbelem
c	  write(6,2001)(aes(nbelem*(nbelem*(kd-1)+k-1)+i),i=1,nbelem)
c	  print*
c	  write(6,2001)(aed(nbelem*(nbelem*(kd-1)+k-1)+i),i=1,nbelem)
c	 enddo
c	enddo
c	pause	

c	verification des derivees

	if(.false.)then
c	if(.true. .and. .not.melange)then

	 print*,'nbelem=',nbelem
	 print*,'p,t,r,l,mn,cx,ro,cte1'	
	 write(6,2000)p,t,r,l,mn,cx,ro,cte1
	
c	 les equations et les derivees numeriques en Xi

	 print*,'bes/bed initiaux'
	 write(6,2000)(bes(i),i=1,nbelem)
	 write(6,2000)(bed(i),i=1,nbelem)
	 
	 do kd=1,2		!derivees 0 et '
	  do k=0,nbelem		!0 pour la fonction
	
	   if(k .ne. 0)then
	    stor0=y(nbelem*(kd-1)+k)
	    stor=stor0*unpdx
	    if(stor .eq. 0.d0)stor=dx
	    y(nbelem*(kd-1)+k)=stor
	    dstor=stor-stor0
	   endif

c	   la composition chimique par gramme

	   do i=1,nbelem
	    xchim(i)=y(i)
	   enddo
	   call chim_gram(xchim,dcomp,nuc_m)	!dcomp VT

c	   la densite et le gradient adiabatique

	   call etat(p,t,xchim,.true.,
	1	ro,drop,drot,drox,u,dup,dut,dux,
	2	delta,deltap,deltat,deltax,cp,dcpp,dcpt,dcpx,
	3	gradad,dgradadp,dgradadt,dgradadx,alfa,beta,gamma1)

c	   l'opacite et le gradient radiatif

	   call opa(xchim,t,ro,kap,dkapt,dkapro,dkapx)
	   gradrad=cte8*l*p*kap/mn/t**4
	 
c	   le coefficient de diffusion

	   call coeff_diff(melange,p,t,r,l,mn,ro,drox,kap,dkapx,
	1	gradrad,dgradradx,gradad,dgradadx,r_zc,lim,mstar,
	2	y,d,dd,v,dv)

c	   les reactions thermonucleaires Xdot

	   call nuc(t,ro,y,dcomp,jac,.false.,2,
	1	epsilon,depst,depsro,depsx,hhe,be7e,b8e,n13e,o15e,f17e)

c	   write(6,2000)ro,(d(i),i=1,nbelem),(dcomp(i),i=1,nbelem)

	   do i=1,nbelem
	    if(i .ne. iw)then
	     bess(i)=cte1*(y(i)-comp_t(i)-dt*dcomp(i))
	     beds(i)=cte3*v(i)*y(i)	!coeff. de Xi: vitesse
	     do j=1,nbelem		!produit scalaire D . dx/dm	
	      beds(i)=beds(i)+cte2*d(nbelem*(j-1)+i)*y(nbelem+j)
	     enddo	!j
	    else	   
	     bess(i)=cte1r*(y(i)-comp_t(i)-dt*dcomp(i))
	     beds(i)=cte3r*v(i)*y(i)	!coeff. de Xi: vitesse
	     do j=1,nbelem		!produit scalaire D . dx/dm	
	      beds(i)=beds(i)+cte2r*d(nbelem*(j-1)+i)*y(nbelem+j)
	     enddo	!j
	    endif
	   enddo	!i

	   if(k .eq. 0)then
	    if(kd .eq. 1)then
	     do i=1,nbelem	
	      bes(i)=bess(i)
	      bed(i)=beds(i)
	     enddo
	     print*,'nouvelles valeurs (k=0)'
	     write(6,2000)(bes(i),i=1,nbelem)
	     write(6,2000)(bed(i),i=1,nbelem)
	     print*
	    endif
	  
	   else		!derivees numerique / Xk
	    print*,'derivee aes, k, kd',k,kd
	    write(6,2000)(aes(nbelem*(nbelem*(kd-1)+k-1)+i),
	1	(bess(i)-bes(i))/dstor,i=1,nbelem)
	    print*,'derivee aed'
	    write(6,2000)(aed(nbelem*(nbelem*(kd-1)+k-1)+i),
	1	(beds(i)-bed(i))/dstor,i=1,nbelem)
	    print*	  
	    y(nbelem*(kd-1)+k)=stor0
	   endif
	  enddo	!k
	  pause'kd suivant'
	 enddo	!kd
	endif		!test

	return

c	perte de moment angulaire on ajoute Fw dt au dernier produit scalaire

200	if(iw .gt. 1)then
	 bes(iw)=y(iw)**3*cte4r
	 aes(nbelem*(nbelem*(1-1)+iw-1)+iw)=3.*bes(iw)/y(iw)
c	 write(6,2000)bes(iw),y(iw)
c	 write(6,2000)y(4),y(5)
c	 print*,y(iw),bes(iw),cte4r
	endif
	
c	perte de masse on ajoute X_i Mdot dt aux derniers produits scalaires
c	cte4=mdot*1.d6*dt		!perte de masse
c200	do i=1,nbelem
c	 bes(i)=y(i)*cte4
c	 aes(nbelem*(nbelem*(1-1)+i-1)+i)=cte4	!derivee/Xi
c	enddo
c	write(6,2000)mstar,cte4,bes(1),y(1),mdot
	 	
	return

	end
