	
c*********************************************************	

	subroutine saha(xchim,t,ro,ioni,z_bar,nel,eta)	
	
c	calcul des taux d'ionisation et charges moyennes
c	potentiels d'ionisation du Handbook of Chemistry and Physics
c	75-ieme edition 1995 p. 10-205

c	on ecrit la formule de saha sous la forme de Mihalas stellar atmosphere
c	formule 5-17, on resout par Newton-Raphson 
c	ro/amu \sum Z_bar(ne) xchim - ne = 0

c	on utilise une adaptation de la correction de Clayton (2-254)
c	pour eviter la recombinaison  des ions a haute temperature
c	(voir notice de CESAM)

c	CESAM4

c	Auteur: P. Morel, Departement J.D. Cassini, O.C.A., Observatoire de Nice

c entrees:
c	xchim: composition chimique par mole
c	t: temperature
c	ro: densite

c sorties:
c	ioni(0:zi(i),i): taux d'ionisation pour chaque element i
c	z_bar: charges moyennes 
c	nel: nombre d'electrons libres
c	eta : degenerescence

	implicit none

	include 'cesam.parametres'
	include 'ctephy.common'
	include 'evol_chim.common'
	include 'modele.common'
	
	integer pf12
	parameter (pf12=101)	!nb. de points de tabulation de F12
	
	integer i,j,k,tour,tourmax,mdg,ndg,knot,l
	
	real*8	xchim(1),t,ro,ioni(0:pzi,pnchim),nel,ki(pzi,pnchim),
	1	u(0:pzi,pnchim),phi(pzi,pnchim),evesk,den,a,
	2	epsi,dnel,z_bar(1),den_max,phi_max,
	3	f12(pf12),f12t(pf12+4),dgce(pf12),degen(5),eta,f12l,cte1,
	4	dep,fin,gsg(pzi,pnchim),nelp,dz_bar(pnchim),dioni(0:pzi,pnchim),
	5	dden,dphi(pzi,pnchim),da,deta,df12l,
	6	kisrd,cte2,p,zeta,x,cla
	
	logical init,no_conv
	data init/.true./,no_conv/.true./
	
	save

c	cubique pour lissage de la correction des poids statistiques
c	entre x=0 et x=p
c	zeta(0,p)=0, zeta'(0,p)=0, zeta(p,p)=1, zeta'(p,p)=0,
c	zeta(x<0,p)=0, zeta(x>p,p)=1
	
	zeta(x,p)=(x/p)**2*(-2.d0*x/p+3.d0)
	
2000	format(1x,1p8d10.3)

	if(init)then
	 init=.false.
	 
c	 verification du nombre max de niveaux d'ionisation

	 a=0
	 do i=1,nchim
	  a=max(a,zi(i))
	 enddo
	 if(nint(a) .gt. pzi)then
	  print*,'la valeur de la charge d''un element est >',pzi
	  print*,'Zi max=',nint(a)
	  print*,'sauf erreur, dans cesam.parametres'
	  print*,'augmenter le parametre pzi a ',nint(a)
	  print*,'et reconstituer la bibliotheque de CESAM'
	  print*,'arret'
	  stop
	 endif
	 
c	 constantes diverses	   		 
	 
	 epsi=1.d-3	!precision sur nel
	 tourmax=30	!limitation du nombre d'iterations sur tour
	 cte1=sqrt(hpl/2.d0/me/kbol*hpl)**3/4.d0/pi	!pour calcul F1/2
	 den_max=1.d15		!max pour le denominateur
	 phi_max=log(den_max)	!max pour ki(j,i)/t+ksi
	 
c	 correction de recombinaison des ions

	 p=4.d0 
	 cte2=echarg**3*sqrt(4.d0*pi/kbol/amu)/kbol
	 	 
c	 identification des elements et formation des tableaux
c	 des potentiels d'ionisation, poids statistiques

	 do i=1,nchim
	  if(nom_elem(i) .eq. ' H1 ')then
	   ki(1,i)=kih	!potentiel d'ionisation 1
	   u(0,i)=2	!poids statistique du fondamental
	   u(1,i)=1	!poids statistique du niveau 1
	  elseif(nom_elem(i) .eq. ' H2 ')then
	   ki(1,i)=kih	!potentiel d'ionisation 1
	   u(0,i)=2	!poids statistique du fondamental
	   u(1,i)=1	!poids statistique du niveau 1
	  elseif(nom_elem(i) .eq. 'He3 ')then
	   ki(1,i)=kihe !potentiel d'ionisation 1
	   ki(2,i)=kihe1 !potentiel d'ionisation 2	   
	   u(0,i)=1	!poids statistique du fondamental
	   u(1,i)=2	!poids statistique du niveau 1
	   u(2,i)=1	!poids statistique du niveau 2
	  elseif(nom_elem(i) .eq. 'He4 ')then
	   ki(1,i)=kihe !potentiel d'ionisation 1
	   ki(2,i)=kihe1 !potentiel d'ionisation 2	   
	   u(0,i)=1	!poids statistique du fondamental
	   u(1,i)=2	!poids statistique du niveau 1
	   u(2,i)=1	!poids statistique du niveau 2
	  elseif(nom_elem(i) .eq. 'Li6 ')then
	   ki(1,i)=5.39172 !potentiel d'ionisation 1
	   ki(2,i)=75.64018 !potentiel d'ionisation 2
	   ki(3,i)=122.45429 !potentiel d'ionisation 3	   
	   u(0,i)=2	!poids statistique du fondamental
	   u(1,i)=1	!poids statistique du niveau 1
	   u(2,i)=2	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	  elseif(nom_elem(i) .eq. 'Li7 ')then
	   ki(1,i)=5.39172 !potentiel d'ionisation 1
	   ki(2,i)=75.64018 !potentiel d'ionisation 2
	   ki(3,i)=122.4529 !potentiel d'ionisation 3	   
	   u(0,i)=2	!poids statistique du fondamental
	   u(1,i)=1	!poids statistique du niveau 1
	   u(2,i)=2	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	  elseif(nom_elem(i) .eq. 'Be7 ')then
	   ki(1,i)=9.32263 !potentiel d'ionisation 1
	   ki(2,i)=18.21116 !potentiel d'ionisation 2
	   ki(3,i)=153.89661 !potentiel d'ionisation 3
	   ki(4,i)=217.71865 !potentiel d'ionisation 4
	   u(0,i)=1	!poids statistique du fondamental
	   u(1,i)=2	!poids statistique du niveau 1
	   u(2,i)=1	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	   u(4,i)=2	!poids statistique du niveau 4
	  elseif(nom_elem(i) .eq. 'Be9 ')then
	   ki(1,i)=9.32263 !potentiel d'ionisation 1
	   ki(2,i)=18.21116 !potentiel d'ionisation 2
	   ki(3,i)=153.89661 !potentiel d'ionisation 3
	   ki(4,i)=217.71865 !potentiel d'ionisation 4
	   u(0,i)=1	!poids statistique du fondamental
	   u(1,i)=2	!poids statistique du niveau 1
	   u(2,i)=1	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	   u(4,i)=2	!poids statistique du niveau 4
	  elseif(nom_elem(i) .eq. 'C12 ')then
	   ki(1,i)=11.26030 !potentiel d'ionisation 1
	   ki(2,i)=24.38332 !potentiel d'ionisation 2
	   ki(3,i)=47.8878 !potentiel d'ionisation 3
	   ki(4,i)=64.4939 !potentiel d'ionisation 4
	   ki(5,i)=391.087 !potentiel d'ionisation 5
	   ki(6,i)=489.99334 !potentiel d'ionisation 6
	   u(0,i)=9	!poids statistique du fondamental
	   u(1,i)=6	!poids statistique du niveau 1 
	   u(2,i)=1	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	   u(4,i)=1	!poids statistique du niveau 4
	   u(5,i)=2	!poids statistique du niveau 5
	   u(6,i)=2	!poids statistique du niveau 6
	  elseif(nom_elem(i) .eq. 'C13 ')then
	   ki(1,i)=11.26030 !potentiel d'ionisation 1
	   ki(2,i)=24.38332 !potentiel d'ionisation 2
	   ki(3,i)=47.8878 !potentiel d'ionisation 3
	   ki(4,i)=64.4939 !potentiel d'ionisation 4
	   ki(5,i)=391.087 !potentiel d'ionisation 5
	   ki(6,i)=489.99334 !potentiel d'ionisation 6
	   u(0,i)=9	!poids statistique du fondamental
	   u(1,i)=6	!poids statistique du niveau 1 
	   u(2,i)=1	!poids statistique du niveau 2
	   u(3,i)=2	!poids statistique du niveau 3
	   u(4,i)=1	!poids statistique du niveau 4
	   u(5,i)=2	!poids statistique du niveau 5
	   u(6,i)=2	!poids statistique du niveau 6
	  elseif(nom_elem(i) .eq. 'N14 ')then
	   ki(1,i)=14.53414 !potentiel d'ionisation 1
	   ki(2,i)=29.296013 !potentiel d'ionisation 2
	   ki(3,i)=47.44924 !potentiel d'ionisation 3
	   ki(4,i)=77.4735 !potentiel d'ionisation 4
	   ki(5,i)=97.8902 !potentiel d'ionisation 5
	   ki(6,i)=552.0718 !potentiel d'ionisation 6
	   ki(7,i)=667.046 !potentiel d'ionisation 7
	   u(0,i)=4	!poids statistique du fondamental
	   u(1,i)=9	!poids statistique du niveau 
	   u(2,i)=6	!poids statistique du niveau
	   u(3,i)=1	!poids statistique du niveau
	   u(4,i)=2	!poids statistique du niveau
	   u(5,i)=1	!poids statistique du niveau
	   u(6,i)=2	!poids statistique du niveau
	   u(7,i)=2	!poids statistique du niveau	   	   	   	   	   
	  elseif(nom_elem(i) .eq. 'N15 ')then
	   ki(1,i)=14.53414 !potentiel d'ionisation 1
	   ki(2,i)=29.296013 !potentiel d'ionisation 2
	   ki(3,i)=47.44924 !potentiel d'ionisation 3
	   ki(4,i)=77.4735 !potentiel d'ionisation 4
	   ki(5,i)=97.8902 !potentiel d'ionisation 5
	   ki(6,i)=552.0718 !potentiel d'ionisation 6
	   ki(7,i)=667.046 !potentiel d'ionisation 7
	   u(0,i)=4	!poids statistique du fondamental
	   u(1,i)=9	!poids statistique du niveau 
	   u(2,i)=6	!poids statistique du niveau
	   u(3,i)=1	!poids statistique du niveau
	   u(4,i)=2	!poids statistique du niveau
	   u(5,i)=1	!poids statistique du niveau
	   u(6,i)=2	!poids statistique du niveau
	   u(7,i)=2	!poids statistique du niveau	   	   	   	   	   
	  elseif(nom_elem(i) .eq. 'O16 ')then
	   ki(1,i)=13.61806 !potentiel d'ionisation 1
	   ki(2,i)=35.11730 !potentiel d'ionisation 2
	   ki(3,i)=54.9355 !potentiel d'ionisation 3
	   ki(4,i)=77.41353 !potentiel d'ionisation 4
	   ki(5,i)=113.8990 !potentiel d'ionisation 5
	   ki(6,i)=138.1197 !potentiel d'ionisation 6
	   ki(7,i)=739.29 !potentiel d'ionisation 7
	   ki(8,i)=871.4101 !potentiel d'ionisation 8
	   u(0,i)=9	!poids statistique du fondamental
	   u(1,i)=4	!poids statistique du niveau 1
	   u(2,i)=9	!poids statistique du niveau 2
	   u(3,i)=6	!poids statistique du niveau 3  
	   u(4,i)=1	!poids statistique du niveau 4	   
	   u(5,i)=2	!poids statistique du niveau 5	   
	   u(6,i)=1	!poids statistique du niveau 6
	   u(7,i)=2	!poids statistique du niveau 7	   
	   u(8,i)=2	!poids statistique du niveau 8	   
	  elseif(nom_elem(i) .eq. 'O17 ')then
	   ki(1,i)=13.61806 !potentiel d'ionisation 1
	   ki(2,i)=35.11730 !potentiel d'ionisation 2
	   ki(3,i)=54.9355 !potentiel d'ionisation 3
	   ki(4,i)=77.41353 !potentiel d'ionisation 4
	   ki(5,i)=113.8990 !potentiel d'ionisation 5
	   ki(6,i)=138.1197  !potentiel d'ionisation 6
	   ki(7,i)=739.29 !potentiel d'ionisation 7
	   ki(8,i)=871.4101 !potentiel d'ionisation 8
	   u(0,i)=9	!poids statistique du fondamental
	   u(1,i)=4	!poids statistique du niveau 1
	   u(2,i)=9	!poids statistique du niveau 2
	   u(3,i)=6	!poids statistique du niveau 3  
	   u(4,i)=1	!poids statistique du niveau 4	   
	   u(5,i)=2	!poids statistique du niveau 5	   
	   u(6,i)=1	!poids statistique du niveau 6
	   u(7,i)=2	!poids statistique du niveau 7	   
	   u(8,i)=2	!poids statistique du niveau 8	   
	 elseif(nom_elem(i) .eq. ' Ex ')then	!Al
	   ki(1,i)=5.98577 !potentiel d'ionisation 1
	   ki(2,i)=18.82856 !potentiel d'ionisation 2
	   ki(3,i)=28.44765 !potentiel d'ionisation 3 
	   ki(4,i)=119.992 !potentiel d'ionisation 4 
	   ki(5,i)=153.825 !potentiel d'ionisation 5
	   ki(6,i)=190.49 !potentiel d'ionisation 6
	   ki(7,i)=241.76 !potentiel d'ionisation 7
	   ki(8,i)=284.66 !potentiel d'ionisation 8
	   ki(9,i)=330.13 !potentiel d'ionisation 9
	   ki(10,i)=398.75 !potentiel d'ionisation 10
	   ki(11,i)=442.00 !potentiel d'ionisation 11
	   ki(12,i)=2085.98 !potentiel d'ionisation 12
	   ki(13,i)=2304.1410 !potentiel d'ionisation 13
	   	   
	   u(0,i)=6	!poids statistique du fondamental
	   u(1,i)=1	!poids statistique du niveau 1
	   u(2,i)=2	!poids statistique du niveau 2
	   u(3,i)=1	!poids statistique du niveau 3
	   u(4,i)=6	!poids statistique du niveau 4
	   u(5,i)=9	!poids statistique du niveau 5
	   u(6,i)=4	!poids statistique du niveau 6
	   u(7,i)=9	!poids statistique du niveau 7
	   u(8,i)=6	!poids statistique du niveau 8
	   u(9,i)=1	!poids statistique du niveau 9
	   u(10,i)=2	!poids statistique du niveau 10
	   u(11,i)=1	!poids statistique du niveau 11
	   u(12,i)=2	!poids statistique du niveau 12
	   u(13,i)=2	!poids statistique du niveau 13
	    
	  elseif(nom_elem(i) .eq. 'Fe56')then
	   ki(1,i)=7.9024d0	!potentiel d'ionisation 1
	   ki(2,i)=16.1878d0	!potentiel d'ionisation 2
	   ki(3,i)=30.652d0	!potentiel d'ionisation 3
	   ki(4,i)=54.8d0	!potentiel d'ionisation 4
	   ki(5,i)=75.0d0	!potentiel d'ionisation 5
	   ki(6,i)=99.1d0	!potentiel d'ionisation 6
	   ki(7,i)=124.98d0	!potentiel d'ionisation 7
	   ki(8,i)=151.06d0	!potentiel d'ionisation 8
	   ki(9,i)=233.6d0	!potentiel d'ionisation 9
	   ki(10,i)=262.1d0	!potentiel d'ionisation 10
	   ki(11,i)=290.2d0	!potentiel d'ionisation 11
	   ki(12,i)=330.8d0	!potentiel d'ionisation 12
	   ki(13,i)=361.0d0	!potentiel d'ionisation 13
	   ki(14,i)=392.2d0	!potentiel d'ionisation 14
	   ki(15,i)=457.d0	!potentiel d'ionisation 15
	   ki(16,i)=489.256d0	!potentiel d'ionisation 16
	   ki(17,i)=1266.d0	!potentiel d'ionisation 17
	   ki(18,i)=1358.d0	!potentiel d'ionisation 18 
	   ki(19,i)=1456.d0	!potentiel d'ionisation 19
	   ki(20,i)=1582.d0	!potentiel d'ionisation 20
	   ki(21,i)=1689.d0	!potentiel d'ionisation 21
	   ki(22,i)=1799.d0	!potentiel d'ionisation 22
	   ki(23,i)=1950.d0	!potentiel d'ionisation 23
	   ki(24,i)=2023.d0	!potentiel d'ionisation 24
	   ki(25,i)=8828.d0	!potentiel d'ionisation 25
	   ki(26,i)=9277.69d0	!potentiel d'ionisation 26
	   	    
	   u(0,i)=25.d0	!poids statistique du fondamental 26 electrons
	   u(1,i)=30.d0	!poids statistique du niveau 1 25 electrons
	   u(2,i)=25.d0	!poids statistique du niveau 2 24 electrons
	   u(3,i)=6.d0	!poids statistique du niveau 3 23 electrons
	   u(4,i)=25.d0	!poids statistique du niveau 4 22 electrons
	   u(5,i)=12.d0	!poids statistique du niveau 5 21 electrons
	   u(6,i)=21.d0	!poids statistique du niveau 6 20 electrons
	   u(7,i)=10.d0	!poids statistique du niveau 7 19 electrons
	   u(8,i)=1.d0	!poids statistique du niveau 8 18 electrons
	   u(9,i)=6.d0	!poids statistique du niveau 9  17 electrons
	   u(10,i)=9.d0	!poids statistique du niveau 10 16 electrons
	   u(11,i)=4.d0	!poids statistique du niveau 11 15 electrons
	   u(12,i)=9.d0	!poids statistique du niveau 12 14 electrons
	   u(13,i)=6.d0	!poids statistique du niveau 13 13 electrons
	   u(14,i)=1.d0	!poids statistique du niveau 14 12 electrons
	   u(15,i)=2.d0	!poids statistique du niveau 15 11 electrons
	   u(16,i)=1.d0	!poids statistique du niveau 16 10 electrons 
	   u(17,i)=6.d0	!poids statistique du niveau 17 9 electrons
	   u(18,i)=9.d0	!poids statistique du niveau 18 8 electrons 
	   u(19,i)=4.d0	!poids statistique du niveau 19 7 electrons
	   u(20,i)=9.d0	!poids statistique du niveau 20 6 electrons
	   u(21,i)=6.d0	!poids statistique du niveau 21 5 electrons
	   u(22,i)=1.d0	!poids statistique du niveau 22 4 electrons
	   u(23,i)=2.d0	!poids statistique du niveau 23 3 electrons
	   u(24,i)=1.d0	!poids statistique du niveau 24 2 electrons
	   u(25,i)=2.d0	!poids statistique du niveau 25 1 electron
	   u(26,i)=1.d0	!poids statistique du niveau 26 0 electron
	   	    
c	  elseif(nom_elem(i) .eq. ' ')then
c	   ki(,i)= !potentiel d'ionisation 
c	   ki(,i)= !potentiel d'ionisation 
c	   u(0,i)=	!poids statistique du fondamental
c	   u(,i)=	!poids statistique du niveau 
c	   u(,i)=	!poids statistique du niveau 

	  else
	   write(6,*)'dans saha, erreur element non prevu nom_elem=',nom_elem(i)
	   write(6,*)'arret'
	   stop
	  endif
	 enddo
	
c	 les ki/k en cgs

	 evesk=eve/kbol
	 do i=1,nchim
	  do j=1,nint(zi(i))
	   ki(j,i)=ki(j,i)*evesk
	   gsg(j,i)=u(j-1,i)/u(j,i)	!rapport des poids statistiques
	  enddo
	 enddo
	 
c	 tabulation des fonctions F12 de Fermi-Dirac de dep a fin
c	 signe oppose de celui de Clayton 

	 ndg=pf12	!nombre de points
	 mdg=4		!ordre des splines
	 dep=-30
	 fin=16
	 a=(fin-dep)/float(ndg-1)
	 do i=1,ndg
	  dgce(i)=dep+a*(i-1)
	  call ferdir(dgce(i),degen)
	  f12(i)=degen(2)
	 enddo
	 	  	 
	 call sbsp1d(dgce,f12,f12t,ndg,mdg,knot,.false.,f12(2),l,eta,a)
	
c	 eta=-1.3d0	!test de la tabulation
c	 eta=3.5d0 
c	 eta=15.d0 
c	 call ferdir(eta,degen)
c	 write(6,2000)eta,degen(2)
c	 f12l=degen(2)
c	 call sbsp1d(dgce,f12,f12t,ndg,mdg,knot,.true.,f12l,l,eta,a)
c	 write(6,2000)f12l,eta
c	 pause	
	 	
	endif
	
c	initialisation du nombre d'electrons libres

	if(t .lt. 5.d3)then
	 nel=xchim(1)/100.d0	!H est 1/100 ionise
	elseif(t .lt. 1.d4)then
	 nel=xchim(1)/50.d0	!H est 1/50 ionise
	elseif(t .lt. 5.d4)then
	 nel=xchim(1)	!H est ionise
	elseif(t .lt. 1.d5)then
	 nel=(xchim(1)+xchim(ihe4)) !H est ionise, He est ionise 1 fois
	else
	 nel=(xchim(1)+2.*xchim(ihe4))!H est ionise, He est ionise 2 fois
	endif
	nel=nel*ro/amu
	
	do i=1,nchim		!initialisation a 1/2 ionisation totale
	 z_bar(i)=zi(i)/2.d0
	enddo

	tour=0
	dnel=1.d3	!pour le while
	nelp=nel
	
	do while(dnel .gt. epsi .and. tour .lt. tourmax)
	 tour=tour+1
	 
c	 degenerescence

	 df12l=cte1/sqrt(t)**3
	 f12l=nel*df12l
	 if(f12l .lt. f12(1))then
	  eta=dgce(1)
	  deta=0.d0
	 elseif(f12l .gt. f12(ndg))then
	  eta=dgce(ndg)
	  deta=0.d0	  
	 else	 
	  call sbsp1d(dgce,f12,f12t,ndg,mdg,knot,.true.,f12l,l,eta,a)
	  deta=a*df12l
	 endif
	 
c	 calcul des ki/Rd (Clayton 2-237, 2-254)

	 kisrd=0.d0
	 do i=1,nchim
	  kisrd=kisrd+z_bar(i)*(z_bar(i)+1.d0)*xchim(i)
	 enddo
	 kisrd=cte2*sqrt(ro*kisrd/t)
c	 print*,kisrd

c	 calcul des phi

	 do i=1,nchim
c	  print*,nom_elem(i)
	  do j=1,nint(zi(i))
	  	    
	   cla=ki(j,i)/kisrd/dble(j)-1.d0	!correction de Clayon Xi/Xid-1
	   if(cla .ge. p)then			!cubique entre 0 et p
	    cla=1.d0
	   elseif(cla .le. 0.d0)then
	    cla=0.d0
	   else
	    cla=zeta(cla,p)
	   endif

	   a=ki(j,i)/t*cla+eta	   
	   da=deta
	   
	   if(a .lt. phi_max)then
	    phi(j,i)=gsg(j,i)*exp(a)*cla
	    dphi(j,i)=phi(j,i)*da 
	   else
	    phi(j,i)=den_max
	    dphi(j,i)=0.d0
	   endif
	  enddo
	 enddo

c	 le denominateur ne doit pas depasser den_max (1.d15)
	
	 do i=1,nchim
	  den=0.d0
	  dden=0.d0
	  j=0
	  do while(den .lt. den_max .and. j .lt. nint(zi(i)))
	   j=j+1
	   dden=dden*phi(j,i)+(den+1.d0)*dphi(j,i)	   
	   den=(den+1.d0)*phi(j,i)
	  enddo
	  den=den+1.d0

c	  le dernier niveau j atteint assure un den < den_max
c	  on met tous les ioni des autres niveaux a 0
	  
	  do k=j+1,nint(zi(i))
	   ioni(k,i)=0.d0
	   dioni(k,i)=0.d0
	  enddo
	  ioni(j,i)=1.d0/den		!le niveau j
	  dioni(j,i)=-ioni(j,i)/den*dden
	  do k=j-1,0,-1
	   ioni(k,i)=ioni(k+1,i)*phi(k+1,i)
	  enddo	  
	 enddo
	
c	 nombre d'electrons libres

	 nel=0.d0
	 dnel=0.d0
	 
c	 calcul des charges moyennes et nel

	 do i=1,nchim
	  z_bar(i)=0.d0
	  dz_bar(i)=0.d0
	  do j=1,nint(zi(i))
	   z_bar(i)=z_bar(i)+ioni(j,i)*j	!nb e libres du a l'element i
	   dz_bar(i)=dz_bar(i)+dioni(j,i)*j
	  enddo	  
	  nel=nel+z_bar(i)*xchim(i)
	  dnel=dnel+dz_bar(i)*xchim(i)
	 enddo
	 nel=nel*ro/amu
	 dnel=dnel*ro/amu-1.d0 
	 dnel=(nel-nelp)/dnel		!second terme de Newton-Raphson
	 nel=nelp-dnel			!nouveau nel
	 dnel=abs(dnel/nel)		!precision relative
c	 write(6,2000)dnel
c	 pause
	 nelp=nel
	enddo
	
	if(tour .ge. tourmax .and. no_conv)then
	 print*,'pas de convergence dans saha au moins 1 fois'
	 write(6,"('t=',1pd10.3,' dnel=',1pd10.3)")t,dnel
c	 no_conv=.false.
	endif
		 
	return
	
	end
