
c******************************************************************

	subroutine etat_mhd(p,t,xchim,deriv,
	1	ro,drop,drot,drox,u,dup,dut,dux,
	2	delta,deltap,deltat,deltax,cp,dcpp,dcpt,dcpx,
	3	gradad,dgradadp,dgradadt,dgradadx,alfa,beta,gamma1)
	
c	equation d'etat MHD de W. Dappen

c	interface MHD/CESAM
c	auteurs G. Berthomieu, P. Morel
c	version 25 02 92

c	modifs
c	06 06 92 W. DAPPEN nouvelles tables, Pression de radiation
c	19 11 99 : suppression de nh1, nhe1, nhe2, lamb

c	adaptation CESAM4 P. Morel

c	les noms des fichiers d'equation d'etat peuvent etre lus
c	dans le fichier *don puis eventuellement modifies dans cette routine

c entree :
c	p : pression
c	t : temperature
c	xchim : composition chimique
c	deriv=.false. : evite le calcul  de certaines derivees

c sortie :
c	ro : densite et derivees
c	u : energie interne et derivees
c	delta, cp, gradad : notations thermodynamiques evidentes, et derivees
c	alpha, beta, gamma1 : notations thermodynamiques evidentes

	implicit none
	
	integer ivarx
	parameter (ivarx = 25)
	
	include	'cesam.parametres'
	include 'modele.common'
	include 'evol_chim.common'
	include 'ctephy.common'

	integer ir(8),ier,i,ntab,ir0

	real*8	p,t,xchim(1),ln10,prad,pcrit,
	1	ro,drop,drot,drox,u,dup,dut,dux,
	2	delta,deltap,deltat,deltax,cp,dcpp,dcpt,dcpx,
	3	gradad,dgradadp,dgradadt,dgradadx,	
	4	alfa,beta,gamma1,aradias3,	
	5	duro,pg,rl0,pg10,t10,xh,gradad0,delta0,cp0,
	6	pg10s,t10s,stor,stor0,dstor,dx,unpdx
	
	logical init,deriv
	data init/.true./

	real*8 varmhd(25)
	common/mhdout/varmhd

	character*40 eostb(8)
	
	save

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccc calls mhd tables ccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

2000	format((1x,1p8d10.3))

	if(init) then ! calculs effectues au premier passage
	 init=.false.
	 ln10=log(10.d0)
	 aradias3=aradia/3.
	 dx=1.d-5
	 unpdx=1.d0+dx
	 
c	 identification des fichiers de donnees
	 
	 f_eos(1)='~/SUN_DATA/mhd1.bin'
	 f_eos(2)='~/SUN_DATA/mhd2.bin'	
	 f_eos(3)='~/SUN_DATA/mhd3.bin'	
	 f_eos(4)='~/SUN_DATA/mhd4.bin'	
	 f_eos(5)='~/SUN_DATA/mhd5.bin'
	 f_eos(6)='~/SUN_DATA/mhd6.bin'	
	 f_eos(7)='~/SUN_DATA/mhd7.bin'	
	 f_eos(8)='~/SUN_DATA/mhd8.bin'
	 
	 do i=1,8
	 eostb(i)=f_eos(i)
	 enddo

c ...... set mhd equation of state ................

c======================================================================
c uses 3 ZAMS-type tables and optionally 5 center-type tables.
c======================================================================

c	 write(6,*)'Equation d''etat MHD, entrer le nombre de tables'
c	 write(6,*) 'number of tables (3 [ZAMS] or 8 [ZAMS + center])?'
c	 read(5,*) ntab
         ntab=8
	 if(ntab .ne. 3 .and. ntab .ne. 8) then
	  write(6,*) 'illegal number of tables. error.'
	  stop
	 endif
	 write(6,*)'Utilisation de l''equation d''etat MHD'
	 write(2,*)'Utilisation de l''equation d''etat MHD'
	 if(ntab .eq. 3)then
	  write(6,*)'utilisation des 3 tables de ZAMS'
	  write(2,*)'utilisation des 3 tables de ZAMS'
	 else
	  write(6,*)'utilisation de 3 tables de ZAMS et de 5 tables de centre'
	  write(2,*)'utilisation de 3 tables de ZAMS et de 5 tables de centre'
	 endif

	 do i=1,8
	  ir(i) = 0
	 enddo
	 ir0   = 10

	 do i=1,ntab
	  ir(i) = ir0 + i
	  open(unit=ir(i),status='old',file=eostb(i),form='unformatted')
	 enddo
	 call mhdst(ir(1),ir(2),ir(3),ir(4),ir(5),ir(6),ir(7),ir(8))
	endif 		 !fin des operations du premier passage
	
	ier     = 0
c       type *,'p,t',p,t
c	Modif W. Dappen ----- pour eviter des appels de p<p_rad
c	pg=p-7.564d-15*t**4/3.
c	---------------- la valeur de la constante crmhd est identique a
c	---------------- celle employe dans le code MHD qui a produit les
c	---------------- tables. Le 1.d-6 est assez arbitraire.

	prad=aradias3*t**4
	pcrit = prad + 1.d-6*prad
	pg = p - prad
	pg=max(pg,pcrit)

	pg10=log10(pg)
	t10=log10(t)
	xh=xchim(1)
	
	call mhdpx(pg10,t10,xh,rl0,ier)
	
        if(ier .eq. 1 .or. VARMHD(4) .EQ. 0.)then
	 print*,'hors des tables MHD, appel a EFF'
	 call etat_eff(p,t,xchim,deriv,
	1	ro,drop,drot,drox,u,dup,dut,dux,
	2	delta,deltap,deltat,deltax,cp,dcpp,dcpt,dcpx,
	3	gradad,dgradadp,dgradadt,dgradadx,alfa,beta,gamma1)
	 return
	endif
	
	ro=10.**rl0
	drop=ro/varmhd(4)
	drot=-drop*varmhd(5)/t
	drox=-drop*varmhd(21)*ln10
	drop=drop/p
	
	u=10.**varmhd(3)/ro
	duro=varmhd(6)-1.d0
	dut=varmhd(7)
	dux=varmhd(22)*ln10
	dup=duro*drop*u/ro
	dut=(dut/t+duro/ro*drot)*u
	dux=(dux+duro/ro*drox)*u
	
	cp=10.**varmhd(9)          !table mhd
	gradad=varmhd(8)            !table mhd
	delta=varmhd(5)/varmhd(4)
	
	alfa=p/ro*drop
	beta=1.d0-prad/p
	gamma1=1./(alfa-delta*gradad)
	
c	nh1 = varmhd(14)	!taux d'ionisation H, He4
c	nhe1 = varmhd(15)
c	nhe2 = varmhd(16)
c	lamb = varmhd(18)	!degenerescence
	
	if(deriv)then
	
	 pg10s=pg10
	 t10s=t10
	 	
c	 derivees / P

	 stor0=p
	 stor=stor0*unpdx
	 if(stor .lt. dx)stor=dx
	 dstor=stor-stor0
	 p=stor
	 pcrit = prad + 1.d-6*prad
	 pg = p - prad
	 pg=max(pg,pcrit)
	 pg10=log10(pg)
	 call mhdpx(pg10,t10,xh,rl0,ier)
	 p=stor0
	 pg10=pg10s
	 
	 gradad0=varmhd(8)            !table mhd
	 delta0=varmhd(5)/varmhd(4)	
	 cp0=10.**varmhd(9)          !table mhd
	 dgradadp=(gradad0-gradad)/dstor
	 deltap=(delta0-delta)/dstor
	 dcpp=(cp0-cp)/dstor
	 	
c	 derivees / T

	 stor0=t
	 stor=stor0*unpdx
	 if(stor .lt. dx)stor=dx
	 dstor=stor-stor0
	 t=stor
	 prad=aradias3*t**4	 
	 pcrit = prad + 1.d-6*prad
	 pg = p - prad
	 pg=max(pg,pcrit)
	 pg10=log10(pg)
	 t10=log10(t)	 
	 call mhdpx(pg10,t10,xh,rl0,ier)
	 t=stor0
	 pg10=pg10s
	 t10=t10s
	 
	 gradad0=varmhd(8)            !table mhd
	 delta0=varmhd(5)/varmhd(4)
	 cp0=10.**varmhd(9)          !table mhd
	 dgradadt=(gradad0-gradad)/dstor
	 deltat=(delta0-delta)/dstor
	 dcpt=(cp0-cp)/dstor
	 	
c	 derivees / X

	 stor0=xh
	 stor=stor0*unpdx
	 if(stor .lt. dx)stor=dx
	 dstor=stor-stor0
	 xh=stor
 	 call mhdpx(pg10,t10,xh,rl0,ier)
	 
	 gradad0=varmhd(8)            !table mhd
	 delta0=varmhd(5)/varmhd(4)
	 cp0=10.**varmhd(9)          !table mhd
	 dgradadx=(gradad0-gradad)/dstor
	 deltax=(delta0-delta)/dstor
	 dcpx=(cp0-cp)/dstor
	endif

        return

        end

c*********************************************************************

	subroutine mhdpx(pgl,tl,xc,rl,ierr)

c computes derivative with respect to X also in ZAMS-table situation.
c changed subroutine is mhdpx1, calling quintd instead of quint.

c=====================================================================
c=====       MHD interpolation package      ==========================
c=====          pressure as argument        ==========================
c=====           interpolation in X         ==========================
c=====                Z fixed               ==========================
c=====================================================================

c     input   pgl = log10 (gas pressure [dyn/cm2])
c             tl  = log10 (temperature [K])
c             xc  = hydrogen abundance (by mass)

c     output  rl  = log10 (density [g/cm3])        : ARGUMENT
c             other (see separate instructions)    : COMMON/MHDOUT/...

c     error   ierr = 1 signals pgl,tl outside the domain of tables
c             (otherwise ierr = 0).

c           s/r mhdstx must be called in main.

	implicit none

	integer ivarc,nchem0
	parameter( ivarc=25,nchem0=6)
	
	real*8	varmhd(ivarc)
	common/mhdout/varmhd
	
	integer irange,ierr
	
	real*8	pgl,tl,xc,rl	

	call mhdpx1(pgl,tl,xc,irange)

	ierr = 0 
	if(irange.eq.0) then
	 write(6,*) 'out of table range. return'
	 ierr = 1
	 return
	endif

	rl =varmhd(1)

	return

	end

c********************************************************************

	subroutine mhdpx1(pgl,tl,x,irange)
      
c	mhdst must be called in main.

c	interpolation in tables with different x and fixed z

	implicit none
	
	integer ivarx,ndimt
	parameter(ivarx=25,ndimt=8)
	
	integer	ix(4),irange,i,itbl,iv,ixmin,ii,id,np,l,inter

	real*8	varout(ndimt,ivarx),xc(ndimt),dvdx(ivarx),vari1(ivarx,4),xx(4),
	1	pgl,tl,x,x0,h,y0,y1,y2,povere,dledlp

c	quantities for interpolation in x
c	tl < tlim1:         lower part of zams tables
c	tlim1 < tl < tlim2: upper part of zams tables
c	tl > tlim2:         centre tables
c	tmini,tmaxi:        temperature interval covered by the tables

	real*8	tlim1,tlim2,tmini,tmaxi
	common/tttt/tlim1,tlim2,tmini,tmaxi

	real*8	varfin(ivarx)	!output	
	common/mhdout/varfin

1000	format(' results from mhdpx2, itbl,x = ',i6,1pe15.7/)
1001	format(12(/1x,1p5e15.6))
5001	format(1x,'******* warning: extrapolation in X (quint) ',
	1	'pgl,tl,x = ',3f12.6)
5011	format(1x,'******* warning: extrapolation in X (lir) ',
	1	'pgl,tl,x = ',3f12.6)
9001	format(' error in mhdpx1. tl out of range. tl,tmini,tmaxi=',
	1	1x,1p3e13.6)


c             read from appropriate tables
c             and fill arrays varout and xc

c------------ can we do it?

	irange = 1
	if(tl .lt. tmini .or. tl .gt. tmaxi) then
	 print 9001,tl,tmini,tmaxi
	 irange = 0
	 return
	endif

	if(tl .lt. tlim1) then	!lower zams tables
	 do i=1,3
	  itbl = -i
	  call mhdpx2(pgl,tl,itbl,varout,xc,ndimt,irange)
c	  print 1000,itbl,xc(i)
c	  print 1001,(varout(i,jj),jj=1,ivarx)
	 enddo

	elseif(tl .lt. tlim2) then	!upper zams tables
	 do i=1,3
	  itbl = i
	  call mhdpx2(pgl,tl,itbl,varout,xc,ndimt,irange)
c	  print 1000,itbl,xc(i)
c	  print 1001,(varout(i,jj),jj=1,ivarx)
	 enddo
	 
	else	!centre tables
	 do i=1,5	!offset (+3) to access center tables
	  itbl = i + 3
	  call mhdpx2(pgl,tl,itbl,varout,xc,ndimt,irange)
c	  print 1000,itbl,xc(itbl)
c	  print 1001,(varout(itbl,jj),jj=1,ivarx)
	 enddo
	endif

c===================== interpolation in x ========================

	if(tl .le. tlim2) then	!quadratic newton (equidistant xc's)
	 x0 = xc(1)
	 h  = xc(2) - xc(1)
	 if(abs(xc(3)-xc(2)-h) .gt. 1.e-4) then
	  write(6,*) 'non-equidistant zams tables. error stop'
	  write(6,*) 'xc(1-3)= ',xc(1),xc(2),xc(3)
	  stop
	 endif

	 if(    ( xc(3).gt.xc(1) .and. (x.gt.xc(3) .or. x.lt.xc(1))) .or.
	1	( xc(3).lt.xc(1) .and. (x.gt.xc(1) .or. x.lt.xc(3))) )
	2	write(6,5001) pgl,tl,x

         do iv=1,ivarx
          y0 = varout(1,iv)
          y1 = varout(2,iv)
          y2 = varout(3,iv)
          call quintd(x,x0,h,y0,y1,y2,varfin(iv),dvdx(iv))
	 enddo
	
         varfin(21) = -dvdx(1)*varfin(4)        
         povere     =  10.d0**(varfin(2)-varfin(3))
         dledlp     =  (povere*(1.d0-varfin(5)) + 1.d0)/varfin(4)
         varfin(22) =  dvdx(3) + varfin(21)*dledlp
         varfin(23) =  dvdx(8) - dvdx(1)*varfin(10)       
         varfin(24) =  dvdx(9) - dvdx(1)*varfin(12)       
         varfin(25) = -5555533333.d0

	else	!cubic lagrangian (arbitrarily spaced xc's)

c	 use 4 highest xc(i)'s, if possible.
c	 distinguish case where the xc(i), i=4..8,
c	 increase or decrease. 

         if(xc(4) .lt. xc(8))then
          ixmin = 5
          if(x .lt. xc(5))ixmin=4
         else
          ixmin = 4
          if(x .lt. xc(7)) ixmin=5
         endif
c
         do i=1,4
	  ix(i)=ixmin+i-1
 	 enddo

         do i =1,4
          xx(i) = xc(ix(i))
c         write(6,*)' xx(i) for lir: i,xx(i),x ',i,xx(i),x
          do iv=1,ivarx
           vari1(iv,i) = varout(ix(i),iv)
	  enddo
	 enddo

         ii=ivarx
         id=ivarx
         np=4
         l=1
         inter=1

         call lir(x,xx,varfin,vari1,ii,id,np,l,inter)

         if(inter .eq. 0) write(6,5011) pgl,tl,x

	endif
	
	return
	
	end      

      
c**************************************************************************      

	subroutine mhdpx2(pgl,tl,itbl,varout,xcomp,ndimt,irange)

	implicit none

	integer	ivarc,ivarx,nchem0
	parameter(ivarc=20,ivarx=25,nchem0=6)
	
	integer	nt1m,nt2m,ntxm,nr1m,nr2m,nrxm	
	parameter( nt1m=39,nt2m=63,ntxm=22,nr1m=139,nr2m=61,nrxm=61)
	
	integer	itbl,ndimt,irange,i
	
	real*8	pgl,tl,varout(ndimt,ivarx),xcomp(ndimt),varo(ivarx),
	1	vvar1(ivarx,4),vvar2(ivarx,4),vy(ivarx)

c.	zams tables (labelled by a,b,c)

	integer	nt1,nr1
	real*8	tdvr1a(nt1m,nr1m,ivarc),tlog1(nt1m),drh1
	common/tab1a/tdvr1a,tlog1,nt1,nr1,drh1
	
	integer	nt2,nr2
	real*8	tdvr2a(nt2m,nr2m,ivarc),tlog2(nt2m),drh2
	common/tab2a/tdvr2a,tlog2,nt2,nr2,drh2
	
	real*8	tdvr1b(nt1m,nr1m,ivarc)
	common/tab1b/tdvr1b
	
	real*8	tdvr2b(nt2m,nr2m,ivarc)
	common/tab2b/tdvr2b
	
	real*8	tdvr1c(nt1m,nr1m,ivarc)
	common/tab1c/tdvr1c
	
	real*8	tdvr2c(nt2m,nr2m,ivarc)
	common/tab2c/tdvr2c
	
	real*8	atwta(nchem0),abuna(nchem0),abfrca(nchem0),gasma
	common/chea/atwta,abuna,abfrca,gasma
	
	real*8	atwtb(nchem0),abunb(nchem0),abfrcb(nchem0),gasmb	
	common/cheb/atwtb,abunb,abfrcb,gasmb
	
	real*8	atwtc(nchem0),abunc(nchem0),abfrcc(nchem0),gasmc
	common/chec/atwtc,abunc,abfrcc,gasmc
	
c	centre tables (labelled by 1,2,3,4,5)

	integer	ntx,nrx
	real*8	tdvrx1(ntxm,nrxm,ivarx),tlogx(ntxm),drhx
	common/tabx1/tdvrx1,tlogx,ntx,nrx,drhx
	
	real*8	tdvrx2(ntxm,nrxm,ivarx)
	common/tabx2/tdvrx2
	
	real*8	tdvrx3(ntxm,nrxm,ivarx)
	common/tabx3/tdvrx3
	
	real*8	tdvrx4(ntxm,nrxm,ivarx)
	common/tabx4/tdvrx4
	
	real*8	tdvrx5(ntxm,nrxm,ivarx)	
	common/tabx5/tdvrx5
	
	
	real*8	atwt1(nchem0),abun1(nchem0),abfrc1(nchem0),gasm1
	common/che1/atwt1,abun1,abfrc1,gasm1
	
	real*8	atwt2(nchem0),abun2(nchem0),abfrc2(nchem0),gasm2	
	common/che2/atwt2,abun2,abfrc2,gasm2
	
	real*8	atwt3(nchem0),abun3(nchem0),abfrc3(nchem0),gasm3	
	common/che3/atwt3,abun3,abfrc3,gasm3
	
	real*8	atwt4(nchem0),abun4(nchem0),abfrc4(nchem0),gasm4	
	common/che4/atwt4,abun4,abfrc4,gasm4
	
	real*8 atwt5(nchem0),abun5(nchem0),abfrc5(nchem0),gasm5	
	common/che5/atwt5,abun5,abfrc5,gasm5

c=================================================================
c
c     nomenclature for accessing the tables
c     =====================================
c
c     zams tables
c
c     itbl = -1   : tdvr1a
c     itbl =  1   : tdvr2a
c     itbl = -2   : tdvr1b
c     itbl =  2   : tdvr2b
c     itbl = -3   : tdvr1c
c     itbl =  3   : tdvr2c
c
c     contral (variable x) tables
c
c     itbl =  4   : tdvrx1
c     itbl =  5   : tdvrx2
c     itbl =  6   : tdvrx3
c     itbl =  7   : tdvrx4
c     itbl =  8   : tdvrx5
c
c     note: the same output variables varout(itbl,.) is used
c           for itbl=1 and -1, 2 and -2, 3 and -3, respectively.
c
c=================================================================

      irange = 1
      call zero(varo,ivarx)
c
c------------------------------------------------------------------
c-------------- main selection of tables --------------------------
c------------------------------------------------------------------
c
c               zams tables
c               ***********
c
      if  (itbl.eq.-1) then
         call intpt (pgl,tl,tdvr1a,nt1m,nr1m,ivarc,tlog1,nt1,nr1,
     .               vvar1,vvar2,vy,varo,irange)
         do 20 i=1,ivarc
20       varout(1,i) = varo(i)
         xcomp(1)    = abfrca(1)
c
      else if (itbl.eq. 1) then
c
         call intpt (pgl,tl,tdvr2a,nt2m,nr2m,ivarc,tlog2,nt2,nr2,
     .               vvar1,vvar2,vy,varo,irange)
         do 30 i=1,ivarc
30       varout(1,i) = varo(i)
         xcomp(1)    = abfrca(1)
c
      else if (itbl.eq.-2) then
c
         call intpt (pgl,tl,tdvr1b,nt1m,nr1m,ivarc,tlog1,nt1,nr1,
     .               vvar1,vvar2,vy,varo,irange)
         do 40 i=1,ivarc
40       varout(2,i) = varo(i)
         xcomp(2)    = abfrcb(1)
c
      else if (itbl.eq. 2) then
c
         call intpt (pgl,tl,tdvr2b,nt2m,nr2m,ivarc,tlog2,nt2,nr2,
     .               vvar1,vvar2,vy,varo,irange)
         do 50 i=1,ivarc
50       varout(2,i) = varo(i)
         xcomp(2)    = abfrcb(1)
c
      else if (itbl.eq.-3) then
c
         call intpt (pgl,tl,tdvr1c,nt1m,nr1m,ivarc,tlog1,nt1,nr1,
     .               vvar1,vvar2,vy,varo,irange)
         do 60 i=1,ivarc
60       varout(3,i) = varo(i)
         xcomp(3)    = abfrcc(1)
c
      else if (itbl.eq. 3) then
c
         call intpt (pgl,tl,tdvr2c,nt2m,nr2m,ivarc,tlog2,nt2,nr2,
     .               vvar1,vvar2,vy,varo,irange)
         do 70 i=1,ivarc
70       varout(3,i) = varo(i)
         xcomp(3)    = abfrcc(1)
c
c               center tables
c               *************
c
      else if (itbl.eq. 4) then
c
         call intpt (pgl,tl,tdvrx1,ntxm,nrxm,ivarx,tlogx,ntx,nrx,
     .               vvar1,vvar2,vy,varo,irange)
         do 100 i=1,ivarx
100      varout(4,i) = varo(i)
         xcomp(4)    = abfrc1(1)
c
      else if (itbl.eq. 5) then
c
         call intpt (pgl,tl,tdvrx2,ntxm,nrxm,ivarx,tlogx,ntx,nrx,
     .               vvar1,vvar2,vy,varo,irange)
         do 110 i=1,ivarx
110      varout(5,i) = varo(i)
         xcomp(5)    = abfrc2(1)
c
      else if (itbl.eq. 6) then
c
         call intpt (pgl,tl,tdvrx3,ntxm,nrxm,ivarx,tlogx,ntx,nrx,
     .               vvar1,vvar2,vy,varo,irange)
         do 120 i=1,ivarx
120      varout(6,i) = varo(i)
         xcomp(6)    = abfrc3(1)
c
      else if (itbl.eq. 7) then
c
         call intpt (pgl,tl,tdvrx4,ntxm,nrxm,ivarx,tlogx,ntx,nrx,
     .               vvar1,vvar2,vy,varo,irange)
         do 130 i=1,ivarx
130      varout(7,i) = varo(i)
         xcomp(7)    = abfrc4(1)
c
      else if (itbl.eq. 8) then
c
         call intpt (pgl,tl,tdvrx5,ntxm,nrxm,ivarx,tlogx,ntx,nrx,
     .               vvar1,vvar2,vy,varo,irange)
         do 140 i=1,ivarx
140      varout(8,i) = varo(i)
         xcomp(8)    = abfrc5(1)
c
      end if
c
c------------------------------------------------------------------
c-------------- end selection of tables --------------------------
c------------------------------------------------------------------
c
      return
      
      end

c*********************************************************************     

	subroutine intpt(pgl,tl,tdvar,ntm,nrm,ivar,tlog,nt,nr,
	1	var1,var2,y,varout,irange)

	implicit none
      
	integer	irm(4),ir(4,4),it(4),ntm,nrm,ivar,nt,nr,irange,n,i,indt,itmax,
	1	m,irmax,j,ilir	,indr,iv,ii,id,np,l,inter
      	
	real*8 tdvar(ntm,nrm,ivar),tlog(ntm),var1(ivar,4),pgl,tl,
	1	var2(ivar,4),varout(ivar),y(ivar),xx(4),pmini,pmaxi


	do n=1,nt
	 if(tlog(n).ge.tl) goto 101
	 it(1)=n
	enddo
c
101   if(it(1).ge.2) it(1)=it(1)-1
      itmax=nt-3
      if(it(1).gt.itmax) it(1)=itmax
      do 150 i=2,4
150   it(i)=it(1)+i-1
c
      do 210 i=1,4
      irm(i)=1
      indt=it(i)
      pmini=tdvar(indt, 1,20)
      pmaxi=tdvar(indt,nr,20)
      if(pgl.gt.pmaxi) then
          print 9002,tl,tlog(indt),pgl,pmaxi
          irange = 0
          return
      end if
      if(pgl.lt.pmini) then
          print 9003,tl,pgl,pmini
      end if
      do 200 m=1,nr
      if(tdvar(indt,m,20).ge.pgl) goto 201
      irm(i)=m
200   continue
201   if(irm(i).ge.2) irm(i)=irm(i)-1
      irmax=nr-3
      if(irm(i).gt.irmax) irm(i)=irmax
210   continue
c
      do 310 i=1,4
      do 310 j=1,4
      ir(j,i)=irm(i)+j-1
310   continue
c
      do 500 ilir=1,4
      indt=it(ilir)
c
      do 400 i=1,4
      indr=ir(i,ilir)
      xx(i)=tdvar(indt,indr,20)
400   continue
c
      do 410 i=1,4
      indr=ir(i,ilir)
      do 410 iv=1,ivar
      var1(iv,i)=tdvar(indt,indr,iv)
410   continue
c
      ii=ivar
      id=ivar
      np=4
      l=1
      inter=1
c
      call lir(pgl,xx,y,var1,ii,id,np,l,inter)
      do 420 iv=1,ivar
420   var2(iv,ilir) = y(iv)
c
500   continue
c
      do 600 i=1,4
      indt=it(i)
600   xx(i) = tlog(indt)
c
      ii=ivar
      id=ivar
      np=4
      l=1
      inter=1
c
      call lir(tl,xx,varout,var2,ii,id,np,l,inter)
c
      return
c
9002  format(' error in intpt. pgl out of range.',
     & ' tl,tlog(it(i)),pgl,pglmax: ',
     & /1x,1p4e13.5)
9003  format(' ********** warning: extrapolation in s/r intpt.',
     & ' tl,pgl,pglmin = ',3f10.6)
     
      end


c**************************************************************************     
      
	subroutine mhdst(ir1,ir2,ir3,ir4,ir5,ir6,ir7,ir8)

	implicit none
	
	integer ivarc,ivarx,nchem0
	parameter(ivarc=20,ivarx=25,nchem0=6)
	
	integer	nt1m,nt2m,ntxm,nr1m,nr2m,nrxm
	parameter( nt1m=39,nt2m=63,ntxm=22,nr1m=139,nr2m=61,nrxm=61)
	
	integer	ir1,ir2,ir3,ir4,ir5,ir6,ir7,ir8,nchem,ntdum,idx,nrdum

	real*8 tddum(nt1m,nr1m,ivarc),tldum(nt1m),tlogd(nt2m),tlogu(nt2m),
	1	tddif0(nt2m,nr2m,ivarc),tddifd(nt2m,nr2m,ivarc),
	2	tddifu(nt2m,nr2m,ivarc),atwd(nchem0),abud(nchem0),
	3	abfrcd(nchem0),atwu(nchem0),abuu(nchem0),abfrcu(nchem0),
	4	drdum      
      
c....... tl<tlim1:       lower part of zams tables
c....... tlim1<tl<tlim2: upper part of zams tables
c....... tl>tlim2:       variable x tables

	real*8 tlim1,tlim2,tmini,tmaxi
	common/tttt/tlim1,tlim2,tmini,tmaxi
	
c..zams tables (labelled by a,b,c)
	
	integer	nt1,nr1
	real*8	tdvr1a(nt1m,nr1m,ivarc),tlog1(nt1m),drh1
	common/tab1a/tdvr1a,tlog1,nt1,nr1,drh1
	
	integer	nt2,nr2
	real*8	tdvr2a(nt2m,nr2m,ivarc),tlog2(nt2m),drh2
	common/tab2a/tdvr2a,tlog2,nt2,nr2,drh2
	
	real*8	tdvr1b(nt1m,nr1m,ivarc)
	common/tab1b/tdvr1b
	
	real*8	tdvr2b(nt2m,nr2m,ivarc)
	common/tab2b/tdvr2b
	
	real*8	tdvr1c(nt1m,nr1m,ivarc)
	common/tab1c/tdvr1c
	
	real*8	tdvr2c(nt2m,nr2m,ivarc)
	common/tab2c/tdvr2c
	
	real*8	atwta(nchem0),abuna(nchem0),abfrca(nchem0),gasma
	common/chea/atwta,abuna,abfrca,gasma
	
	real*8	atwtb(nchem0),abunb(nchem0),abfrcb(nchem0),gasmb	
	common/cheb/atwtb,abunb,abfrcb,gasmb
	
	real*8	atwtc(nchem0),abunc(nchem0),abfrcc(nchem0),gasmc
	common/chec/atwtc,abunc,abfrcc,gasmc
	
c	centre tables (labelled by 1,2,3,4,5)

	integer	ntx,nrx
	real*8	tdvrx1(ntxm,nrxm,ivarx),tlogx(ntxm),drhx
	common/tabx1/tdvrx1,tlogx,ntx,nrx,drhx
	
	real*8	tdvrx2(ntxm,nrxm,ivarx)
	common/tabx2/tdvrx2
	
	real*8	tdvrx3(ntxm,nrxm,ivarx)
	common/tabx3/tdvrx3
	
	real*8	tdvrx4(ntxm,nrxm,ivarx)
	common/tabx4/tdvrx4
	
	real*8	tdvrx5(ntxm,nrxm,ivarx)	
	common/tabx5/tdvrx5
	
	
	real*8	atwt1(nchem0),abun1(nchem0),abfrc1(nchem0),gasm1
	common/che1/atwt1,abun1,abfrc1,gasm1
	
	real*8	atwt2(nchem0),abun2(nchem0),abfrc2(nchem0),gasm2	
	common/che2/atwt2,abun2,abfrc2,gasm2
	
	real*8	atwt3(nchem0),abun3(nchem0),abfrc3(nchem0),gasm3	
	common/che3/atwt3,abun3,abfrc3,gasm3
	
	real*8	atwt4(nchem0),abun4(nchem0),abfrc4(nchem0),gasm4	
	common/che4/atwt4,abun4,abfrc4,gasm4
	
	real*8 atwt5(nchem0),abun5(nchem0),abfrc5(nchem0),gasm5	
	common/che5/atwt5,abun5,abfrc5,gasm5
      
c--------------------------------------------------------------------
c------------ define, with unused statements, storage for variables
c------------ that would otherwise only appear as formal parameters
c------------ in this subroutine. vicious bugs can be the result if
c------------ this storage were not provided (remember: fortran is not
c------------ a recursive language.)
c--------------------------------------------------------------------
      nchem = 0
      drdum = 0.
      ntdum = 0
      nrdum = 0
c--------------------------------------------------------------------
c
c======================= read zams tables ===========================
c
      if(ir1.gt.0) then
         idx = 0
         call mhdst1(ir1,idx,nt1m,nr1m,ivarc,nt2m,nr2m,ivarc,nchem0,
     .               nt1,nr1,nt2,nr2,tlog1,tlog2,tdvr1a,tdvr2a,
     .               drh1,drh2,nchem,atwta,abuna,abfrca,gasma,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir2.gt.0) then
         idx = 0
         call mhdst1(ir2,idx,nt1m,nr1m,ivarc,nt2m,nr2m,ivarc,nchem0,
     .               nt1,nr1,nt2,nr2,tlog1,tlog2,tdvr1b,tdvr2b,
     .               drh1,drh2,nchem,atwtb,abunb,abfrcb,gasmb,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir3.gt.0) then
         idx = 0
         call mhdst1(ir3,idx,nt1m,nr1m,ivarc,nt2m,nr2m,ivarc,nchem0,
     .               nt1,nr1,nt2,nr2,tlog1,tlog2,tdvr1c,tdvr2c,
     .               drh1,drh2,nchem,atwtc,abunc,abfrcc,gasmc,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
c======================= read centre tables =====================
c
      if(ir4.gt.0) then
         idx = 1
         call mhdst1(ir4,idx,nt1m,nr1m,ivarc,ntxm,nrxm,ivarx,nchem0,
     .               ntdum,nrdum,ntx,nrx,tldum,tlogx,tddum,tdvrx1,
     .               drdum,drhx,nchem,atwt1,abun1,abfrc1,gasm1,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir5.gt.0) then
         idx = 1
         call mhdst1(ir5,idx,nt1m,nr1m,ivarc,ntxm,nrxm,ivarx,nchem0,
     .               ntdum,nrdum,ntx,nrx,tldum,tlogx,tddum,tdvrx2,
     .               drdum,drhx,nchem,atwt2,abun2,abfrc2,gasm2,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir6.gt.0) then
         idx = 1
         call mhdst1(ir6,idx,nt1m,nr1m,ivarc,ntxm,nrxm,ivarx,nchem0,
     .               ntdum,nrdum,ntx,nrx,tldum,tlogx,tddum,tdvrx3,
     .               drdum,drhx,nchem,atwt3,abun3,abfrc3,gasm3,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir7.gt.0) then
         idx = 1
         call mhdst1(ir7,idx,nt1m,nr1m,ivarc,ntxm,nrxm,ivarx,nchem0,
     .               ntdum,nrdum,ntx,nrx,tldum,tlogx,tddum,tdvrx4,
     .               drdum,drhx,nchem,atwt4,abun4,abfrc4,gasm4,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
      if(ir8.gt.0) then
         idx = 1
         call mhdst1(ir8,idx,nt1m,nr1m,ivarc,ntxm,nrxm,ivarx,nchem0,
     .               ntdum,nrdum,ntx,nrx,tldum,tlogx,tddum,tdvrx5,
     .               drdum,drhx,nchem,atwt5,abun5,abfrc5,gasm5,
     .               tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .               abud,abuu,abfrcd,abfrcu)
      end if
c
c======================= temperature limits =====================
c
      tmini = tlog1(  1)
      tlim1 = tlog1(nt1)
c
      if (ir4.le.0) then
         tlim2 = tlog2(nt2)
         tmaxi = tlim2
      else
         tlim2 = tlogx(  1)
         tmaxi = tlogx(ntx)
      end if
c
      write(6,*)' before leaving mhdst: compositions of 8 tables '
c
c      print 8002,(atwta(ic),abuna(ic),abfrca(ic),
c     .       ic=1,nchem)
c      print 8003,gasma
c      print 8002,(atwtb(ic),abunb(ic),abfrcb(ic),
c     .       ic=1,nchem)
c      print 8003,gasmb
c      print 8002,(atwtc(ic),abunc(ic),abfrcc(ic),
c     .       ic=1,nchem)
c      print 8003,gasmc
c
c
c      if(ir4.le.0) goto 500
c
c      print 8002,(atwt1(ic),abun1(ic),abfrc1(ic),
c     .       ic=1,nchem)
c      print 8003,gasm1
c      print 8002,(atwt2(ic),abun2(ic),abfrc2(ic),
c     .       ic=1,nchem)
c      print 8003,gasm2
c      print 8002,(atwt3(ic),abun3(ic),abfrc3(ic),
c     .       ic=1,nchem)
c      print 8003,gasm3
c      print 8002,(atwt4(ic),abun4(ic),abfrc4(ic),
c     .       ic=1,nchem)
c      print 8003,gasm4
c      print 8002,(atwt5(ic),abun5(ic),abfrc5(ic),
c     .       ic=1,nchem)
c      print 8003,gasm5
c
 500  write(6,*)' tlim1,tlim2 = ',tlim1,tlim2
c
8002  format('      at. weight     number ',
     & 'abundance  mass fraction',(/1x,1p3g16.7))
8003  format(/' mean molecular weight = ',f12.7//)
c
      return
      
      end
     
c*********************************************************************
      
      subroutine mhdst1(ir,idx,nt1m,nr1m,ivar1,nt2m,nr2m,ivar2,nchem0,
     .                  nt1,nr1,nt2,nr2,tlog1,tlog2,tdvar1,tdvar2,
     .                  drh1,drh2,nchem,atwt,abun,abfrcs,gasmu,
     .                  tlogd,tlogu,tddif0,tddifd,tddifu,atwd,atwu,
     .                  abud,abuu,abfrcd,abfrcu)


	implicit none
	
	integer ir,idx,nt1m,nr1m,ivar1,nt2m,nr2m,ivar2,nchem0,
	1	nt1,nr1,nt2,nr2,nchem,ifiles,iidx,ivarr,idxr,irescr,iresco,
	2	ic,jt,n,m,iv	
	
	real*8	tlog1(nt1m),tdvar1(nt1m,nr1m,ivar1),ddx,
	1	tlog2(nt2m),tdvar2(nt2m,nr2m,ivar2),
	2	atwt(nchem0),abun(nchem0),abfrcs(nchem0),
	3	tlogd(nt2m),tlogu(nt2m),tddif0(nt2m,nr2m,ivar1),
	4	tddifd(nt2m,nr2m,ivar1),tddifu(nt2m,nr2m,ivar1),
	5	atwd(nchem0),abud(nchem0),abfrcd(nchem0),gasmu,
	6	atwu(nchem0),abuu(nchem0),abfrcu(nchem0),drh1,drh2,
	7	gasxd,gasxu,cxlim
	
	
c---------------------------------------------------------------
c
      if(idx.eq.0) then
          ifiles = 1
      else
          ifiles = 3
      end if
c
      do 400 iidx=1,ifiles
c
c     read(ir,98,end=1000) ivarr,idxr,irescr,ddx
      read(ir,   end=1000) ivarr,idxr,irescr,ddx
c
      if(ivar1.lt.ivarr) then
         print 9006,ivar1,ivarr
         stop
      end if
c
      if(idx.ne.idxr) then
         print 9007,idx,idxr
         stop
      end if
c
      if(iidx.eq.1) then
          iresco= 0
      else if(iidx.eq.2) then
          iresco=-1
      else if(iidx.eq.3) then
          iresco= 1
      end if
c
      if(iresco.ne.irescr) then
         print 9008,iresco,irescr
         stop
      end if
c
      write(6,*) 'ivarr,idxr,iresco,ddx ',ivarr,idxr,iresco,ddx
c
      if(iidx.eq.1) call rabu(ir,nchem0,nchem,atwt,abun,abfrcs,gasmu)
      if(iidx.eq.2) call rabu(ir,nchem0,nchem,atwd,abud,abfrcd,gasxd)
      if(iidx.eq.3) call rabu(ir,nchem0,nchem,atwu,abuu,abfrcu,gasxu)
c
c     read(ir,1001) nt1,nt2,drh1,drh2
      read(ir     ) nt1,nt2,drh1,drh2
c
      if(idx.eq.1 .and. nt1.ne.0) then
          print 9010,idx,nt1,nt2
          stop
      end if
c
      write(6,*) 'nt1,nt2,drh1,drh2', nt1,nt2,drh1,drh2
c
      if(nt1.gt.0) then
         call rtab(ir,nt1m,nr1m,ivar1,nt1,nr1,tlog1,tdvar1)
      end if
c
      if(idx.eq.1) then
       if(iidx.eq.1) call rtab(ir,nt2m,nr2m,ivar1,nt2,nr2,tlog2,tddif0)
       if(iidx.eq.2) call rtab(ir,nt2m,nr2m,ivar1,nt2,nr2,tlogd,tddifd)
       if(iidx.eq.3) call rtab(ir,nt2m,nr2m,ivar1,nt2,nr2,tlogu,tddifu)
      else if(idx.eq.0) then
       call rtab(ir,nt2m,nr2m,ivar2,nt2,nr2,tlog2,tdvar2)
      end if
c
 400  continue
c
      if(idx.eq.0) goto 450
c
c======================================================================
c........ if idx=1: check tables for correct composition construction
c........           and perform numerical derivatives w.r.t. x
c======================================================================
c
      cxlim = 0.05*abs(ddx)
c
      if( abs(abfrcs(1)-abfrcd(1)-ddx).gt.cxlim  .or.
     .    abs(abfrcs(1)-abfrcu(1)+ddx).gt.cxlim  .or.
     .    abs(abfrcs(2)-abfrcd(2)+ddx).gt.cxlim  .or.
     .    abs(abfrcs(2)-abfrcu(2)-ddx).gt.cxlim )    goto 500
c
      do 420 ic=3,nchem
      if( abs(abfrcs(ic)-abfrcu(ic)).gt.cxlim )      goto 500
      if( abs(abfrcs(ic)-abfrcd(ic)).gt.cxlim )      goto 500
 420  continue
c
      do 430 jt=1,nt2
      if(tlog2(jt).ne.tlogd(jt)) goto 600
      if(tlog2(jt).ne.tlogu(jt)) goto 600
 430  continue
c
      print 8001
c
c======================================================================
c....... numerical derivatives w.r.t. x
c======================================================================
c
      do 440 n =1,nt2
      do 440 m =1,nr2
      do 435 iv=1,ivar1
435   tdvar2(n,m,iv)=tddif0(n,m,iv)
c
c>>>>>>>>>> extended set of variables (tdvar2(n,m,ivar1+1...ivar2))
c>>>>>>>>>> for t-rho regions with inhomogeneous composition.
c
c>>>>>>>>>> in the comments,r and t denote log10(rho) and log10(t).
c
c---------- dlog10(p)/dx,dlog10(u)/dx,ddelad/dx,dlog10(cp)/dx
c
      tdvar2(n,m,21)=(tddifu(n,m, 2)-tddifd(n,m, 2))/(2.*ddx)
      tdvar2(n,m,22)=(tddifu(n,m, 3)-tddifd(n,m, 3))/(2.*ddx)
      tdvar2(n,m,23)=(tddifu(n,m, 8)-tddifd(n,m, 8))/(2.*ddx)
      tdvar2(n,m,24)=(tddifu(n,m, 9)-tddifd(n,m, 9))/(2.*ddx)
c................... space-holder variable (like var(20))
      tdvar2(n,m,25)=8888844444.
c
c
  440 continue
c
c======================================================================
c                   normal exit
c======================================================================
c
 450  print 8002,(atwt(ic),abun(ic),abfrcs(ic),
     .       ic=1,nchem)
      print 8003,gasmu
c
      return
c
c======================================================================
c                   error exit and error messages
c======================================================================
c
 500  print 9800
      do 550 ic=1,nchem
      print 9810,ic,abfrcs(ic)
      print 9810,ic,abfrcd(ic)
      print 9810,ic,abfrcu(ic)
      print 9820
 550  continue
      stop
c
 600  print 9850
      do 650 jt=1,nt2
      print 9860,jt,tlogd(jt),tlog2(jt),tlogu(jt)
 650  continue
      stop
 1000 print 9900,ir,idx
      stop
c
c======================================================================
c
 98   format(1x,3i5,f13.5)
 99   format(1x,i5,(/1x,3e15.7))
1001  format(2i5,2f10.6)
c
8001  format(' correct table construction for x-derivatives.',
     .       ' centroid composition is:'//)
8002  format('      at. weight     number ',
     & 'abundance  mass fraction',(/1x,1p3g16.7))
8003  format(/' mean molecular weight = ',f12.7//)
c
9006  format(' error in mhdst1. ivarr read from table is',
     & ' bigger than the value used in the commons.',
     & ' ivar,ivarr= ',/1x,2i8)
9007  format(' error in mhdst1. idxr read from table is incorrect',
     & ' idx,idxr= ',/1x,2i8)
9008  format(' error in mhdst1. irescr read from table is incorrect',
     & ' iresco,irescr= ',/1x,2i8)
9010  format(' error in mhdst1. nt1 and idx are inconsistent',
     & ' idx,nt1,nt2 ',/1x,3i8)
c
9800  format(' error in table construction for x-derivatives',
     &       ' central, lower, upper table: n(element),abfrcs(n)'//)
9810  format(1x,i5,f15.9)
9820  format(/)
9850  format(' error in table construction for x-derivatives:',
     &       ' temperatures wrong: j,tlow(j),tcent(j),tupp(j)'//)
9860  format(1x,i5,3f15.9)
9900  format(' eof reached in input file. error stop. ir,idx = ',2i5)
      end
      

c***************************************************************************
      
	subroutine rtab(ir,ntm,nrm,ivar,nt,nr,tl,tdvar)
            
c to allow derivatives with respect to X also in the ZAMS-tables situation,
c the subroutines mhdpx1 and mhdrx1 have been modified. they call an 
c enhanced quadratic-interpolation subroutine, quintd, instead of the
c usual quint. the file here contains both quint and quintd.

	implicit none
c
c....... nt is input; nr,tl,tdvar are output,n,nrr

	integer	ir,ntm,nrm,ivar,nt,nr,n,nrr,j,iv

	real*8	tl(ntm),tdvar(ntm,nrm,ivar)
c
      do 10 n = 1, nt
c     read(ir,1001) nrr,tl(n)
      read(ir     ) nrr,tl(n)
ccc      write(30,1011) nrr,tl(n)
      if(n.eq.1) nr=nrr
ccccc write(6,*)'n,nr,tl(n) ',n,nr,tl(n)
c
c....... check limits and number of densitiy points of table ..........
      if ( nt.gt.ntm .or.  nr.gt.nrm ) then
         print 9001,nt,nr,ntm,nrm
         stop
      end if
      if ( n.gt.1 .and. nrr.ne.nr ) then
         print 9011,n,nr,nrr
         stop
      end if
c....... end check ....................................................
c
      do 10 j = 1, nr
c     read(ir,1002) (tdvar(n,j,iv),iv=1,ivar)
      read(ir     ) (tdvar(n,j,iv),iv=1,ivar)
cc      write(30,1012) (tdvar(n,j,iv),iv=1,ivar)
10    continue
c
      return
1001  format(i5,e16.8)
1002  format(5e16.8)
1011  format(i5,1pe16.8)
1012  format(1p5e16.8)
9001  format(' error in rtab. too small parameters: ',
     & /' nt,nr,ntm,nrm = ',4i5)
9011  format(' error in rtab. wrong density construction: ',
     & /' n,nr,nrr = ',3i5)
      end
      
c****************************************************************      
      
	subroutine rabu(ir,nchem0,nchem,atwt,abun,abfrcs,gasmu)
	
	implicit none
	
	integer ir,nchem0,nchem,ic

	real*8	atwt(nchem0),abun(nchem0),abfrcs(nchem0),gasmu
c
c....... nchem,atwt,abun,abfrcs are output
c
c     read(ir,99) nchem,(atwt(ic),abun(ic),abfrcs(ic),
c    .       ic=1,nchem),gasmu
      read(ir   ) nchem,(atwt(ic),abun(ic),abfrcs(ic),
     .       ic=1,nchem),gasmu
      if(nchem0.lt.nchem) then
         print 9009,nchem0,nchem
         stop
      end if
c
      return
 99   format(1x,i5,(/1x,3e15.7))
9009  format(' error in rabu. nchem read from table is',
     & ' bigger than the value used in the commons.',
     & ' nchem0,nchem= ',/1x,2i8)
      end
      
c************************************************************      
      
	subroutine quint(x,x0,h,y0,y1,y2,y)
      
	implicit none
	
	real*8	x,x0,h,y0,y1,y2,y,d1,d2,t	
c
c...... quadratic interpolation for equidistant points
c...... y0=y(x0),y1=y(x1),y2=y(x2); h=x1-x0=x2-x1;
c...... computes y=y(x)
c
      d1 = y1 - y0
      d2 = y2 - 2.d0*y1 + y0
      t  = (x - x0)/h
c
      y  = y0 + t*d1 + 0.5d0*t*(t-1.d0)*d2
c
      return
      
      end
      
c********************************************************

	subroutine quintd(x,x0,h,y0,y1,y2,y,dydx)
	
	implicit none
	
	real*8	x0,h,y0,y1,y2,y,dydx,d1,d2,rh,t,x,dydt
	
c...... quadratic interpolation for equidistant points
c...... y0=y(x0),y1=y(x1),y2=y(x2); h=x1-x0=x2-x1;
c...... computes y=y(x) and dydx. 
c
      d1 = y1 - y0
      d2 = y2 - 2.d0*y1 + y0
      rh = 1.d0/h
      t  = (x - x0)*rh
c
      y       = y0 + t*d1 + 0.5d0*t*(t-1.d0)*d2
      dydt    = d1 + (t - 0.5 d0)*d2
      dydx    = dydt*rh
c
      return
      
      end
      
c***************************************************************      
      
	subroutine  lir(z,zi,y,yi,ii,id,nt,l,inter)
	
	implicit none
	
	integer	ii,id,nt,l,inter,n
	data n/-1/
		
	integer il,ir,ir1,ird,iir,j,i,m,k
		
	real*8	z,zi(1),y(1),yi(1),a(4),diff,y1,y2,y3,y4,z4,z1,z2,z3,z12,z34,yy	
c	
c     subroutine  lir1(z,zi,y,yi,ii,id,nt,l,inter)
c
c
c
c                interpolation/extrapolation routine
c
c
c     for a such that z=zi(a),  sets y(i)=yi(i,a), i=1,ii
c
c     zi(n),yi(i,n) must be supplied for n=1,nt and i=1,ii
c     id is first dimension of yi
c
c     inter is set to 1 for interpolation and 0 for extrapolation
c
c     if l.le.1, scan to find the zi(n) which immediately bound z
c                starts at n=1
c     if l.gt.1, scan starts from value of n from previous call of lir
c
c
c     lir use cubic interpolation/extrapolation unless nt.lt.4
c     lir1 use linear interpolation/extrapolation
c
c     note
c     ****
c     most of the computation is performed in single precision

      il=0
      go to 1
      entry lir1(z,zi,y,yi,ii,id,nt,l,inter)
      il=1
    1 continue
      ir=1
c
c     check nt and reset il if necessary
      if(nt.lt.2) go to 101
      if(nt.lt.4) il=1
c
c     addressing constants
      inter=1
      ir1=ir-1
      ird=ir*id
      iir=(ii-1)*ir+1
      j=(nt-1)*ir+1
      diff=zi(j)-zi(1)
c
c     set index for start of search
      n=(n-2)*ir+1
      if(l.le.1.or.n.lt.1) n=1
c
c     determine position of z within zi
    2 if(n.gt.j) go to 8
      if(diff) 4,102,3
    3 if(zi(n)-z) 5,6,9
    4 if(zi(n)-z) 9,6,5
    5 n=n+ir
      go to 2
c
c     set y when z lies on a mesh point
    6 j=(n-1)*id
      do 7 i=1,iir
      y(i)=yi(i+j)
    7 if(y(i).eq.0.) y(i+ir1)=0.
      go to 30
c
c     control when z does not lie on a mesh point
    8 inter=0
    9 if(n.le.1) inter=0
      if(il.eq.1) go to 20
c
c     cubic interpolation/extrapolation
c
c     pivotal point (m) and point (k) closest to z
   10 m=n
      k=3
      if(n.gt.1+ir) go to 11
      m=1+ir+ir
      k=n
   11 if(n.lt.j) go to 12
      m=j-ir
      k=4
c
c
c     weighting factors
   12 y1=zi(m-ir*2)
      y2=zi(m-ir)
      y3=zi(m)
      y4=zi(m+ir)
c
      z1=z-y1
      z2=z-y2
      z3=z-y3
      z4=z-y4
c
   13 z12=z1*z2
      z34=z3*z4
c
   14 a(1)=z2*z34/((y1-y2)*(y1-y3)*(y1-y4))
      a(2)=z1*z34/((y2-y1)*(y2-y3)*(y2-y4))
      a(3)=z12*z4/((y3-y1)*(y3-y2)*(y3-y4))
      a(4)=z12*z3/((y4-y1)*(y4-y2)*(y4-y3))
c
c     correct a(k)
   15 diff=a(1)+a(2)+a(3)+a(4)
      a(k)=(1.d0+a(k))-diff
c
c     compute y
   16 m=(m-1)/ir-3
      m=m*ird
      do 18 i=1,iir
      k=i+m
      yy=0.
      do 17 j=1,4
      k=k+ird
      diff=yi(k)
   17 yy=yy+a(j)*diff
      y(i)=yy
   18 if(y(i).eq.0.) y(i+ir1)=0.
      go to 30
c
c     linear interpolation/extrapolation
   20 if(n.eq.1) n=1+ir
      if(n.gt.j) n=j
      z1=zi(n)
      y1=(z1-z)/(z1-zi(n-ir))
      y2=1.0-y1
      j=(n-1)*id
      m=j-ird
      do 21 i=1,iir,ir
      y(i)=y1*yi(i+m)+y2*yi(i+j)
   21 if(y(i).eq.0.) y(i+ir1)=0.
c
c     reset n
   30 n=(n+ir-1)/ir
      return
c
c
c     diagnostics
  101 write(6,1001) nt
      return
  102 write(6,1002) zi(1),nt,zi(j)
      return
c
 1001 format(/1x,10('*'),5x,'there are fewer than two data points in',
     *      ' lir     nt =',i4,5x,10('*')/)
 1002 format(/1x,10('*'),5x,'extreme values of independent variable',
     *      ' equal in lir',5x,10('*')/16x,'zi(1) =',1pe13.5,',   ',
     *       'zi(',i4,') =',1pe13.5/)
c
      end
      
            
      
      
      
      
      
      
 
      
      
      
      
      
      

      
      
